function ARAnimation() {
    var self = this;
    //self.ee = ee;
    self.uid = null;
    self.ar = null;
    self.effect_type = null;
    self.loop = null;
    self.start_time = 0;
    self.duration = 3000;
    self.tween_type = 'linear.none';

    self.tween = null;
    self.sel = null;

    self.htmlids = {};
}
ARAnimation.prototype.saveData = function() {
    var self = this;
    var data = {
        uid: self.uid,
        ar: self.ar,
        effect_type: self.effect_type,
        tween_type: self.tween_type,
        start_time: self.start_time,
        duration: self.duration,
        loop: self.loop,

        sel: self.sel,

        original: {
            ori_position: self.ori_position,
            ori_rotation: self.ori_rotation,
            ori_scale: self.ori_scale,
            ori_opacity: self.ori_opacity,
        },

        target: {
            target_position: self.target_position,
            target_rotation: self.target_rotation,
            target_scale: self.target_scale,
            target_opacity: self.target_opacity,
        }
    };
    return data;
};
//ARAnimation.prototype.create = function(uid, arobj, effect_type, range, loop, sel, tween_type) {
ARAnimation.prototype.create = function(data) {
    var self = this;
    self.uid = data.uid;
    self.setOriginal(data);
    self.setBetween(data.effect_range);

    self.effect_type = data.effect_type;
    self.loop        = data.loop;
    self.tween_type  = data.tween_type? data.tween_type : 'linear.none';
    self.sel         = data.sel? data.sel : {x: true, y: true, z: true};

    self.createAnimation();

    return self;
};
ARAnimation.prototype.setOriginal = function(data) {
    var self = this;
    delete self.ar;
    self.ar = data.ar;
    self.ori_position   = data.ori_position? copyObj(data.ori_position) : copyObj(self.ar.position);
    self.ori_rotation   = data.ori_rotation? copyObj(data.ori_rotation) : copyObj({x: self.ar.rotation.x, y: self.ar.rotation.y, z: self.ar.rotation.z});
    self.ori_scale      = data.ori_scale? copyObj(data.ori_scale) : copyObj(self.ar.scale);
    self.ori_opacity    = data.ori_opacity? Number(data.ori_opacity) : Number(self.ar.material.opacity);

    self.setTargetPosition(data.target_position);
    self.setTargetRotation(data.target_rotation);
    self.setTargetScale(data.target_scale);
    self.setTargetOpacity(data.target_opacity);
};
ARAnimation.prototype.setTargetOpacity = function(val) {
    var self = this;

    self.target_opacity = (self.effect_type=='fadein')? 1 : 0;

    self.end_opacity = self.target_opacity;
};
ARAnimation.prototype.setTargetScale = function(val) {
    var self = this;

    self.target_scale = {
        x: val? val.x : 2,
        y: val? val.y : 2,
        z: val? val.z : 2
    };

    self.end_scale = {
        x: self.ori_scale.x + self.target_scale.x,
        y: self.ori_scale.y + self.target_scale.y,
        z: self.ori_scale.z + self.target_scale.z
    };
};
ARAnimation.prototype.setTargetRotation = function(val) {
    var self = this;

    self.target_rotation = {
        x: val? val.x : Math.PI*2,
        y: val? val.y : Math.PI*2,
        z: val? val.z : Math.PI*2
    };

    self.end_rotation = {
        x: self.ori_rotation.x - self.target_rotation.x,
        y: self.ori_rotation.y - self.target_rotation.y,
        z: self.ori_rotation.z - self.target_rotation.z
    };
};
ARAnimation.prototype.setTargetPosition = function(val) {
    var self = this;

    self.target_position = {
        x: val? val.x : 10,
        y: val? val.y : 10,
        z: val? val.z : 10
    };

    self.end_position = {
        x: self.ori_position.x + self.target_position.x,
        y: self.ori_position.y + self.target_position.y,
        z: self.ori_position.z + self.target_position.z
    };
};
ARAnimation.prototype.initARObjAsOriginal = function() {
    var self = this;
    var type = self.effect_type;
    try {
        switch (type) {
            case 'move':
                self.ar.position.x = self.ori_position.x;
                self.ar.position.y = self.ori_position.y;
                self.ar.position.z = self.ori_position.z;
                break;
            case 'scale':
                self.ar.scale.x = self.ori_scale.x;
                self.ar.scale.y = self.ori_scale.y;
                self.ar.scale.z = self.ori_scale.z;
                break;
            case 'rotate':
                self.ar.rotation.x = self.ori_rotation.x;
                self.ar.rotation.y = self.ori_rotation.y;
                self.ar.rotation.z = self.ori_rotation.z;
                break;
            case 'fadein':
                self.ar.material.opacity = self.ori_opacity;
                break;
            case 'fadeout':
                self.ar.material.opacity = self.ori_opacity;
                break;
        }
    } catch (e) {
        console.warn(e);
    }
};
ARAnimation.prototype.initARObjAsEndVal = function() {
    var self = this;
    var type = self.effect_type;

    switch (type) {
        case 'move':
            if(self.sel.x === true) self.ar.position.x = self.end_position.x;
            if(self.sel.y === true) self.ar.position.y = self.end_position.y;
            if(self.sel.z === true) self.ar.position.z = self.end_position.z;
            break;
        case 'scale':
            if(self.sel.x === true) self.ar.scale.x = self.end_scale.x;
            if(self.sel.y === true) self.ar.scale.y = self.end_scale.y;
            if(self.sel.z === true) self.ar.scale.z = self.end_scale.z;
            break;
        case 'rotate':
            if(self.sel.x === true) self.ar.rotation.x = self.end_rotation.x;
            if(self.sel.y === true) self.ar.rotation.y = self.end_rotation.y;
            if(self.sel.z === true) self.ar.rotation.z = self.end_rotation.z;
            break;
        case 'fadein':
            self.ar.material.opacity = self.end_opacity;
            break;
        case 'fadeout':
            self.ar.material.opacity = self.end_opacity;
            break;
    }
};
ARAnimation.prototype.calcCurrTime = function (time) {
    var self = this;
    var cur_t = (Number(time) - Number(self.start_time)) % Number(self.duration) +  Number(self.start_time);
    return cur_t;
};
ARAnimation.prototype.refresh = function(time) {
    var self = this;
    var cur_t = Number(time);

    if(self.loop === true && cur_t > Number(self.start_time) + Number(self.duration)) {
        cur_t = self.calcCurrTime(time);
    }

    var isBetween = (cur_t >= Number(self.start_time)
        && cur_t <= Number(self.start_time) + Number(self.duration))
        ? true : false;


    if(self.tween && isBetween === true) {
        self.seekToTime(cur_t);
        return;
    }
    if(!self.tween && isBetween === true) {
        self.createAnimation();
        self.seekToTime(cur_t);
        return;
    }
    if(isBetween === false) {
        if(cur_t < self.start_time) {
            self.initARObjAsOriginal();
            return;
        }
        if(cur_t > self.start_time + self.duration) {
            if(self.loop === true) {
                self.seekToTime(cur_t);
                return;
            } else {
                self.initARObjAsEndVal();
                return;
            }
        }
    }
};
ARAnimation.prototype.resetAnimation = function(range, cur_t) {
    var self = this;

    self.setBetween(range);
    self.createAnimation();
    self.refresh(cur_t);
};
ARAnimation.prototype.seekToTime = function(time) {
    var self = this;
    if(self.tween) {
        self.tween.update(Number(time));
    }
};
ARAnimation.prototype.addExternalEvent = function() {
    var self = this;
};
ARAnimation.prototype.removeExternalEvent = function() {
    var self = this;

};
ARAnimation.prototype.setBetween = function(range) {
    var self = this;
    self.start_time = Number(range.min)*1000;
    self.duration = (Number(range.max) - Number(range.min))*1000;
};
ARAnimation.prototype.remove = function() {
    var self = this;
    self.initARObjAsOriginal();
    if(self.tween) {
        TWEEN.remove(self.tween);
        self.tween = null;
    }
};
ARAnimation.prototype.tweenMethod = function() {
    var self = this;
    var type = self.tween_type;
    var method = tweenMethod(type);

    return method;
};

ARAnimation.prototype.createAnimation = function() {
    var self = this;
    var type = self.effect_type;

    switch (type) {
        case 'move':
            self.createMove();
            break;
        case 'scale':
            self.createScale();
            break;
        case 'rotate':
            self.createRotate();
            break;
        case 'fadein':
            self.createFadeIn();
            break;
        case 'fadeout':
            self.createFadeOut();
            break;
    }
};
ARAnimation.prototype.createMove = function() {
    var self = this;
    var method = self.tweenMethod();

    var start_time = Number(self.start_time);
    var duration = Number(self.duration);
    var start = copyObj(self.ori_position);
    var end = copyObj(self.end_position);

    self.remove();

    self.tween = new TWEEN.Tween( start )
        .to( end, duration )
        .easing( method );

    self.tween.aruid = self.ar.uuid;
    self.tween.tween_type = self.tween_type;

    if(self.loop === true) {
        //self.tween.repeat(Infinity);
    }


    self.tween.onUpdate(function(e) {
        if(self.sel.x === true) self.ar.position.x = e.x;
        if(self.sel.y === true) self.ar.position.y = e.y;
        if(self.sel.z === true) self.ar.position.z = e.z;
    });

    self.tween.start(start_time);
};
ARAnimation.prototype.createFadeIn = function() {
    var self = this;

    var method = self.tweenMethod();

    self.start_opacity  = self.effect_type=='fadein'? 0 : 1;
    self.end_opacity    = self.effect_type=='fadein'? 1 : 0;

    var start_time = Number(self.start_time);
    var duration = Number(self.duration);
    var start = {opacity: Number(self.start_opacity)};
    var end = {opacity: Number(self.end_opacity)};

    self.remove();

    self.tween = new TWEEN.Tween( start )
        .to( end, duration )
        .easing( method );

    self.tween.aruid = self.ar.uuid;
    self.tween.tween_type = self.tween_type;

    if(self.loop === true) {
        //self.tween.repeat(Infinity);
    }


    self.tween.onUpdate(function(e) {
        self.ar.material.opacity = e.opacity;
    });

    self.tween.start(start_time);
};
ARAnimation.prototype.createFadeOut = function() {
    var self = this;

    var method = self.tweenMethod();

    self.start_opacity  = self.effect_type=='fadein'? 0 : 1;
    self.end_opacity    = self.effect_type=='fadein'? 1 : 0;

    var start_time = Number(self.start_time);
    var duration = Number(self.duration);
    var start = {opacity: Number(self.start_opacity)};
    var end = {opacity: Number(self.end_opacity)};

    self.remove();

    self.tween = new TWEEN.Tween( start )
        .to( end, duration )
        .easing( method );

    self.tween.aruid = self.ar.uuid;
    self.tween.tween_type = self.tween_type;

    if(self.loop === true) {
        //self.tween.repeat(Infinity);
    }


    self.tween.onUpdate(function(e) {
        self.ar.material.opacity = e.opacity;
    });

    self.tween.start(start_time);
};
ARAnimation.prototype.createScale = function() {
    var self = this;

    var method = self.tweenMethod();

    var start_time = Number(self.start_time);
    var duration = Number(self.duration);
    var start = copyObj(self.ori_scale);
    var end = copyObj(self.end_scale);

    self.remove();

    self.tween = new TWEEN.Tween( start )
        .to( end, duration )
        .easing( method );

    self.tween.aruid = self.ar.uuid;
    self.tween.tween_type = self.tween_type;

    if(self.loop === true) {
        //self.tween.repeat(Infinity);
    }


    self.tween.onUpdate(function(e) {
        if(self.sel.x === true) self.ar.scale.x = e.x;
        if(self.sel.y === true) self.ar.scale.y = e.y;
        if(self.sel.z === true) self.ar.scale.z = e.z;
    });

    self.tween.start(start_time);
};

ARAnimation.prototype.createRotate = function() {
    var self = this;

    var method = self.tweenMethod();

    var start_time = Number(self.start_time);
    var duration = Number(self.duration);
    var start = copyObj(self.ori_rotation);
    var end = copyObj(self.end_rotation);

    self.remove();

    self.tween = new TWEEN.Tween( start )
        .to( end, duration )
        .easing( method );

    self.tween.aruid = self.ar.uuid;
    self.tween.tween_type = self.tween_type;

    if(self.loop === true) {
        //self.tween.repeat(Infinity);
    }


    self.tween.onUpdate(function(e) {
        if(self.sel.x === true) self.ar.rotation.x = e.x;
        if(self.sel.y === true) self.ar.rotation.y = e.y;
        if(self.sel.z === true) self.ar.rotation.z = e.z;
    });

    self.tween.start(start_time);
};