function AREffect(ee, ar) {
    var self = this;

    self.ee = ee;
    self.ar = ar;
    self.uid = null; // Note: effect slider uid
    self.targetuid = null; // ARObject uid = parent uid
    self.effect_range = null;
    self.effect_type = null;
    self.tween_type = null;
    self.loop = null;
    self.sel = null;

    self.slider = null;
    self.htmlids = null;
    self.color = null;

    self.order = null;

    self.original = {};
    self.target = {};

    self.tween = null;

    self.isSelected = false;
}
AREffect.prototype.create = function(data, insertTarget) {
    var self = this;

    self.setData(data);

    self.createAnimation();

    self.addExternalEvents();

    var effectData = self.getData();
    self.slider = new EffectSlider(self.ee, self.ar).create(effectData, insertTarget);

    self.focusEffect();

    self.refresh();

    return self;
};
AREffect.prototype.remove = function() {
    var self = this;

    self.unfocusEffect();

    self.removeExternalEvents();

    self.removeAnimation();

    self.slider.remove();


    Object.keys(self).forEach(function (key) {
        delete self[key];
    });
    self.__proto__ = {};
};
/**
 * effect_range, effect_type, targetuid is requires, others is optional.
 * @param data
 */
AREffect.prototype.setData = function(data) {
    var self = this;

    self.uid = data.uid? data.uid : generateUUID();
    self.color = data.color? data.color : randomColor();
    self.effect_range = {
        min: Number(data.effect_range.min).toFixed(2),
        max: Number(data.effect_range.max).toFixed(2)
    };
    self.effect_type = data.effect_type;
    self.targetuid = data.targetuid;

    self.loop = data.hasOwnProperty('loop')? data.loop : false;
    self.order = data.hasOwnProperty('order')? data.order : 10000000; // Always in last position with creating one effect.

    self.sel = data.sel? data.sel : {x: true, y: true, z: true};

    self.tween_type = data.tween_type? data.tween_type : 'linear.none';

    self.setOriginal(data.original);
    self.setTarget(data.target);
};
AREffect.prototype.getData = function(){
    var self = this;
    var original = {};
    var target = {};

    switch (self.effect_type) {
        case 'rotate':
            original.rotation = copyObj(self.original.rotation);
            target.rotation = copyObj(self.target.rotation);
            break;
        case 'scale':
            original.scale = copyObj(self.original.scale);
            target.scale = copyObj(self.target.scale);
            break;
        case 'fadein':
            original.opacity = (self.original.opacity);
            target.opacity = (self.target.opacity);
            break;
        case 'fadeout':
            original.opacity = (self.original.opacity);
            target.opacity = (self.target.opacity);
            break;
        case 'move':
            original.position = copyObj(self.original.position);
            target.position = copyObj(self.target.position);
            break;
    }
    var data = {
        effect_type: self.effect_type,
        uid: self.uid,
        targetuid: self.targetuid,
        effect_range: copyObj(self.effect_range),
        loop: self.loop,
        tween_type: self.tween_type,
        sel: copyObj(self.sel),
        color: self.color,
        order: self.order,
        start_time: self.start_time,
        duration: self.duration,
        original: original,
        target: target,
    };
    return data;
};
AREffect.prototype.setOriginal = function(data) {
    var self = this;

    if(data && data.hasOwnProperty('position')) {
        self.setOriginalPosition(data.position);
    } else {
        self.setOriginalPosition();
    }

    if(data && data.hasOwnProperty('rotation')) {
        self.setOriginalRotation(data.rotation);
    } else {
        self.setOriginalRotation();
    }

    if(data && data.hasOwnProperty('scale')) {
        self.setOriginalScale(data.scale);
    } else {
        self.setOriginalScale();
    }

    if(data && data.hasOwnProperty('opacity')) {
        self.setOriginalOpacity(data.opacity);
    } else {
        self.setOriginalOpacity();
    }
};
AREffect.prototype.setOriginalPosition = function(data) {
    var self = this;
    if(data && data.hasOwnProperty('x')) {
        self.original.position = copyObj(data);
    } else {
        self.original.position = copyObj({
            x: self.ar.position.x,
            y: self.ar.position.y,
            z: self.ar.position.z
        });
    }
};
AREffect.prototype.setOriginalRotation = function(data) {
    var self = this;
    if(data && data.hasOwnProperty('x')) {
        self.original.rotation = copyObj(data);
    } else {
        self.original.rotation = copyObj({
            x: self.ar.rotation.x,
            y: self.ar.rotation.y,
            z: self.ar.rotation.z
        });
    }
};
AREffect.prototype.setOriginalScale = function(data) {
    var self = this;
    if(data && data.hasOwnProperty('x')) {
        self.original.scale = copyObj(data);
    } else {
        self.original.scale = copyObj({
            x: self.ar.scale.x,
            y: self.ar.scale.y,
            z: self.ar.scale.z
        });
    }
};
AREffect.prototype.setOriginalOpacity = function(data) {
    var self = this;

    var conf = getObjConf(self.ar.userData.type);

    var opacity = 1;
    if(data) {
        opacity = Number(data);
    } else {
        if(conf.ctg == 'light') {
            opacity = 1;
        } else if(conf.ctg == 'model' || conf.ctg == 'group_widget') {
            self.ar.traverse( function ( child ) {
                if(child.isMesh ) {
                    opacity = Number(child.material.opacity);
                }
            });
        } else {
            opacity = Number(self.ar.material.opacity);
        }
    }

    self.original.opacity = opacity;
};
AREffect.prototype.setTarget = function(data) {
    var self = this;

    if(data && data.hasOwnProperty('position')) {
        self.setTargetPosition(data.position);
    } else {
        self.setTargetPosition();
    }

    if(data && data.hasOwnProperty('rotation')) {
        self.setTargetRotation(data.rotation);
    } else {
        self.setTargetRotation();
    }

    if(data && data.hasOwnProperty('scale')) {
        self.setTargetScale(data.scale);
    } else {
        self.setTargetScale();
    }

    if(data && data.hasOwnProperty('opacity')) {
        self.setTargetOpacity(data.opacity);
    } else {
        self.setTargetOpacity();
    }
};
AREffect.prototype.setTargetOpacity = function(data) {
    var self = this;

    self.target.opacity = (self.effect_type=='fadein')? self.original.opacity : 0; //1 : 0;

    self.end_opacity = self.target.opacity;
};
AREffect.prototype.setTargetScale = function(data) {
    var self = this;

    self.target.scale = {
        x: data? data.x : 2,
        y: data? data.y : 2,
        z: data? data.z : 2
    };

    self.end_scale = {
        x: self.original.scale.x * self.target.scale.x,
        y: self.original.scale.y * self.target.scale.y,
        z: self.original.scale.z * self.target.scale.z
    };
};
AREffect.prototype.setTargetRotation = function(data) {
    var self = this;

    self.target.rotation = {
        x: data? data.x : Math.PI*2,
        y: data? data.y : Math.PI*2,
        z: data? data.z : Math.PI*2
    };

    self.end_rotation = {
        x: self.original.rotation.x - self.target.rotation.x,
        y: self.original.rotation.y - self.target.rotation.y,
        z: self.original.rotation.z - self.target.rotation.z
    };
};
AREffect.prototype.setTargetPosition = function(data) {
    var self = this;

    self.target.position = {
        x: data? data.x : 10,
        y: data? data.y : 10,
        z: data? data.z : 10
    };

    self.end_position = {
        x: self.original.position.x + self.target.position.x,
        y: self.original.position.y + self.target.position.y,
        z: self.original.position.z + self.target.position.z
    };
};
AREffect.prototype.createAnimation = function() {
    var self = this;

    self.setBetween();

    var type = self.effect_type;

    switch (type) {
        case 'move':
            self.createMove();
            break;
        case 'scale':
            self.createScale();
            break;
        case 'rotate':
            self.createRotate();
            break;
        case 'fadein':
            self.createFadeIn();
            break;
        case 'fadeout':
            self.createFadeOut();
            break;
    }
};

AREffect.prototype.removeAnimation = function() {
    var self = this;
    self.initARObjAsOriginal();
    if(self.tween) {
        TWEEN.remove(self.tween);
        self.tween = null;
    }
};

AREffect.prototype.setBetween = function(data) {
    var self = this;
    var range = data? data : self.effect_range;
    self.start_time = Number(range.min)*1000;
    self.duration = (Number(range.max) - Number(range.min))*1000;
};

AREffect.prototype.tweenMethod = function() {
    var self = this;
    var type = self.tween_type;
    var method = tweenMethod(type);

    return method;
};

AREffect.prototype.createMove = function() {
    var self = this;
    var method = self.tweenMethod();

    var start_time = Number(self.start_time);
    var duration = Number(self.duration);
    var start = copyObj(self.original.position);
    var end = copyObj(self.end_position);

    self.removeAnimation();

    self.tween = new TWEEN.Tween( start )
        .to( end, duration )
        .easing( method );

    self.tween.aruid = self.ar.uuid;
    self.tween.tween_type = self.tween_type;

    if(self.loop === true) {
        //self.tween.repeat(Infinity);
    }


    self.tween.onUpdate(function(e) {
        if(self.sel.x === true) self.ar.position.x = e.x;
        if(self.sel.y === true) self.ar.position.y = e.y;
        if(self.sel.z === true) self.ar.position.z = e.z;
    });

    self.tween.start(start_time);
};
AREffect.prototype.createFadeIn = function() {
    var self = this;

    var method = self.tweenMethod();

    //self.original.opacity  = self.effect_type=='fadein'? 0 : self.original.opacity;
    //self.end_opacity    = self.effect_type=='fadein'? 1 : 0;

    var start_time = Number(self.start_time);
    var duration = Number(self.duration);
    var start = {opacity: self.effect_type=='fadein'? 0 : self.original.opacity};
    var end = {opacity: self.effect_type=='fadein'? self.original.opacity : 0};// 1:0

    self.removeAnimation();

    self.tween = new TWEEN.Tween( start )
        .to( end, duration )
        .easing( method );

    self.tween.aruid = self.ar.uuid;
    self.tween.tween_type = self.tween_type;

    if(self.loop === true) {
        //self.tween.repeat(Infinity);
    }

    self.tween.onUpdate(function(e) {
        self.setOpacity(e.opacity);
    });

    self.tween.start(start_time);
};

AREffect.prototype.createFadeOut = function() {
    var self = this;

    var method = self.tweenMethod();

    //self.original.opacity  = self.effect_type=='fadein'? 0 : self.original.opacity;
    //self.end_opacity    = self.effect_type=='fadein'? 1 : 0;

    var start_time = Number(self.start_time);
    var duration = Number(self.duration);
    var start = {opacity: self.effect_type=='fadein'? 0 : self.original.opacity};
    var end = {opacity: self.effect_type=='fadein'? self.original.opacity : 0};// 1:0

    self.removeAnimation();

    self.tween = new TWEEN.Tween( start )
        .to( end, duration )
        .easing( method );

    self.tween.aruid = self.ar.uuid;
    self.tween.tween_type = self.tween_type;

    if(self.loop === true) {
        //self.tween.repeat(Infinity);
    }

    self.tween.onUpdate(function(e) {
        self.setOpacity(e.opacity);
    });

    self.tween.start(start_time);
};

AREffect.prototype.createScale = function() {
    var self = this;

    var method = self.tweenMethod();

    var start_time = Number(self.start_time);
    var duration = Number(self.duration);
    var start = copyObj(self.original.scale);
    var end = copyObj(self.end_scale);

    self.removeAnimation();

    self.tween = new TWEEN.Tween( start )
        .to( end, duration )
        .easing( method );

    self.tween.aruid = self.ar.uuid;
    self.tween.tween_type = self.tween_type;

    if(self.loop === true) {
        //self.tween.repeat(Infinity);
    }

    self.tween.onUpdate(function(e) {
        if(self.sel.x === true) self.ar.scale.x = e.x;
        if(self.sel.y === true) self.ar.scale.y = e.y;
        if(self.sel.z === true) self.ar.scale.z = e.z;
    });

    self.tween.start(start_time);
};

AREffect.prototype.createRotate = function() {
    var self = this;

    var method = self.tweenMethod();

    var start_time = Number(self.start_time);
    var duration = Number(self.duration);
    var start = copyObj(self.original.rotation);
    var end = copyObj(self.end_rotation);

    self.removeAnimation();

    self.tween = new TWEEN.Tween( start )
        .to( end, duration )
        .easing( method );

    self.tween.aruid = self.ar.uuid;
    self.tween.tween_type = self.tween_type;

    if(self.loop === true) {
        //self.tween.repeat(Infinity);
    }

    self.tween.onUpdate(function(e) {
        if(self.sel.x === true) self.ar.rotation.x = e.x;
        if(self.sel.y === true) self.ar.rotation.y = e.y;
        if(self.sel.z === true) self.ar.rotation.z = e.z;
    });

    self.tween.start(start_time);
};

AREffect.prototype.initARObjAsOriginal = function() {
    var self = this;
    var type = self.effect_type;
    try {
        switch (type) {
            case 'move':
                self.setPosition(self.original.position);
                break;
            case 'scale':
                self.setScale(self.original.scale);
                break;
            case 'rotate':
                self.setRotation(self.original.rotation);
                break;
            case 'fadein':
                self.setOpacity(self.original.opacity);
                break;
            case 'fadeout':
                self.setOpacity(self.original.opacity);
                break;
        }
    } catch (e) {
        console.warn(e);
    }

    AREditor.workspace.addOutline();
};
AREffect.prototype.setOpacity = function(data) {
    var self = this;
    var conf = getObjConf(self.ar.userData.type);

    var opacity = Number(data);
    if(conf.ctg == 'model' || conf.ctg == 'group_widget') {
        self.ar.traverse( function ( child ) {
            if(child.isMesh ) {
                child.material.opacity = opacity;
            }
        });
    } else {
        self.ar.material.opacity = opacity;
    }
};
AREffect.prototype.setPosition = function(data) {
    var self = this;
    self.ar.position.x = data.x;
    self.ar.position.y = data.y;
    self.ar.position.z = data.z;
};
AREffect.prototype.setRotation = function(data) {
    var self = this;
    self.ar.rotation.x = data.x;
    self.ar.rotation.y = data.y;
    self.ar.rotation.z = data.z;
};
AREffect.prototype.setScale = function(data) {
    var self = this;
    self.ar.scale.x = data.x;
    self.ar.scale.y = data.y;
    self.ar.scale.z = data.z;
};
AREffect.prototype.initARObjAsEndVal = function() {
    var self = this;
    var type = self.effect_type;

    switch (type) {
        case 'move':
            if(self.sel.x === true) self.ar.position.x = self.end_position.x;
            if(self.sel.y === true) self.ar.position.y = self.end_position.y;
            if(self.sel.z === true) self.ar.position.z = self.end_position.z;
            break;
        case 'scale':
            if(self.sel.x === true) self.ar.scale.x = self.end_scale.x;
            if(self.sel.y === true) self.ar.scale.y = self.end_scale.y;
            if(self.sel.z === true) self.ar.scale.z = self.end_scale.z;
            break;
        case 'rotate':
            if(self.sel.x === true) self.ar.rotation.x = self.end_rotation.x;
            if(self.sel.y === true) self.ar.rotation.y = self.end_rotation.y;
            if(self.sel.z === true) self.ar.rotation.z = self.end_rotation.z;
            break;
        case 'fadein':
            self.setOpacity(self.end_opacity);
            break;
        case 'fadeout':
            self.setOpacity(self.end_opacity);
            break;
    }

    AREditor.workspace.addOutline();
};

AREffect.prototype.calcCurrTime = function (time) {
    var self = this;
    var cur_t = (Number(time) - Number(self.start_time)) % Number(self.duration) +  Number(self.start_time);
    return cur_t;
};
AREffect.prototype.refreshAnimation = function(time) {
    var self = this;
    var cur_t = Number(time);

    if(self.loop === true && cur_t > Number(self.start_time) + Number(self.duration)) {
        cur_t = self.calcCurrTime(time);
    }

    var isBetween = (cur_t >= Number(self.start_time)
        && cur_t <= Number(self.start_time) + Number(self.duration))
        ? true : false;


    if(self.tween && isBetween === true) {
        self.seekToTime(cur_t);
        return;
    }
    if(!self.tween && isBetween === true) {
        self.createAnimation();
        self.seekToTime(cur_t);
        return;
    }
    if(isBetween === false) {
        if(cur_t < self.start_time) {
            self.initARObjAsOriginal();
            return;
        }
        if(cur_t > self.start_time + self.duration) {
            if(self.loop === true) {
                self.seekToTime(cur_t);
                return;
            } else {
                self.initARObjAsEndVal();
                return;
            }
        }
    }
};
AREffect.prototype.resetAnimation = function(cur_t) {
    var self = this;

    self.setBetween();
    self.createAnimation();
    self.refreshAnimation(cur_t);

    AREditor.workspace.addOutline();
};
AREffect.prototype.seekToTime = function(time) {
    var self = this;
    if(self.tween) {
        self.tween.update(Number(time));
    }
};
AREffect.prototype.refresh = function() {
    var self = this;

    var cur_t = Number(self.getMainPos())*1000;

    self.resetAnimation(cur_t);
    self.initSettings();

    var mainRange = AREditor.mainslider.getRange();
    self.slider.updateSlider(mainRange);
};
AREffect.prototype.getMainPos = function() {
    var pos = AREditor.mainslider.getPos();
    return pos;
};
AREffect.prototype.focusEffect = function() {
    var self = this;

    self.isSelected = true;

    self.initSettings();

    self.slider.focusSlider();
};
AREffect.prototype.unfocusEffect = function() {
    var self = this;

    self.isSelected = false;

    self.hideSettings();

    self.slider.unfocusSlider();
};
AREffect.prototype.initSettings = function() {
    var self = this;

    var data = self.getData();

    EffectSettings.init(data);
    return;
};
AREffect.prototype.hideSettings = function() {
    var self = this;

    EffectSettings.hideSettings();
    return;
};
AREffect.prototype.addHistory = function(undoData, redoData) {
    AREditor.history.add({
        undo: function() {
            AREditor.ee.trigger('setting_effect_log', [undoData]);
        },
        redo: function() {
            AREditor.ee.trigger('setting_effect_log', [redoData]);
        }
    });
};
AREffect.prototype.addExternalEvents = function() {
    var self = this;

    self.bind_update_object_workspace = function(objData) {
        /**
         * If you want to apply to all effects of target object, uncheck isSelected value.
         */
        if(self.isSelected === true && self.objectUID(objData) == self.targetuid) {
            //self.ar = arobj;
            self.setOriginal(objData);
            self.setTarget(copyObj(self.target)); // Modified by jjb 2019.06.09.
            self.refresh();
        }
    }.bind(self);
    self.ee.on('update_object_workspace', self.bind_update_object_workspace);

    self.bind_setting_start = function(ani_obj, val) {
        if(self.uid == ani_obj.uid) {
            var effect_old = self.getData();

            var start = Number(val);
            var min = Number(start).toFixed(2);
            var max = Number(start+(self.effect_range.max-self.effect_range.min)).toFixed(2);
            var effect_range = {min: min, max: max};
            self.effect_range = effect_range;
            self.slider.effect_range = self.effect_range;

            self.refresh();

            var effect_new = self.getData();

            self.addHistory(effect_old, effect_new);
        }
    }.bind(self);
    self.ee.on('setting_start', self.bind_setting_start);

    self.bind_setting_duration = function(ani_obj, val) {
        if(self.uid == ani_obj.uid) {
            var effect_old = self.getData();

            var duration = Number(val);
            var min = Number(self.effect_range.min);
            var max = Number(duration+min).toFixed(2);
            var effect_range = {min: min, max: max};
            self.effect_range = effect_range;
            self.slider.effect_range = self.effect_range;

            self.refresh();

            var effect_new = self.getData();

            self.addHistory(effect_old, effect_new);
        }
    }.bind(self);
    self.ee.on('setting_duration', self.bind_setting_duration);

    self.bind_setting_loop = function(ani_obj, val) {
        if(self.uid == ani_obj.uid) {
            var effect_old = self.getData();

            self.loop = val;
            self.slider.loop = val;

            self.refresh();
            self.slider.refreshLoopIcon();

            var effect_new = self.getData();

            self.addHistory(effect_old, effect_new);
        }
    }.bind(self);
    self.ee.on('setting_loop', self.bind_setting_loop);

    self.bind_setting_easing = function(ani_obj, val) {
        if(self.uid == ani_obj.uid) {
            var effect_old = self.getData();

            self.tween_type = val;

            self.refresh();

            var effect_new = self.getData();

            self.addHistory(effect_old, effect_new);
        }
    }.bind(self);
    self.ee.on('setting_easing', self.bind_setting_easing);

    self.bind_setting_x_chk = function(ani_obj, val) {
        if(self.uid == ani_obj.uid) {
            var effect_old = self.getData();

            self.sel.x = val;

            self.refresh();

            var effect_new = self.getData();

            self.addHistory(effect_old, effect_new);
        }
    }.bind(self);
    self.ee.on('setting_x_chk', self.bind_setting_x_chk);

    self.bind_setting_y_chk = function(ani_obj, val) {
        if(self.uid == ani_obj.uid) {
            var effect_old = self.getData();

            self.sel.y = val;

            self.refresh();

            var effect_new = self.getData();

            self.addHistory(effect_old, effect_new);
        }
    }.bind(self);
    self.ee.on('setting_y_chk', self.bind_setting_y_chk);

    self.bind_setting_z_chk = function(ani_obj, val) {
        if(self.uid == ani_obj.uid) {
            var effect_old = self.getData();

            self.sel.z = val;

            self.refresh();

            var effect_new = self.getData();

            self.addHistory(effect_old, effect_new);
        }
    }.bind(self);
    self.ee.on('setting_z_chk', self.bind_setting_z_chk);

    self.bind_setting_xyz = function(ani_obj, val) {
        if(self.uid == ani_obj.uid) {
            var effect_old = self.getData();

            var eff_type = self.effect_type;
            switch (eff_type) {
                case 'scale':
                    self.setTargetScale(val);
                    break;
                case 'rotate':
                    self.setTargetRotation(val);
                    break;
                case 'move':
                    self.setTargetPosition(val);
                    break;
            }
            self.refresh();

            var effect_new = self.getData();

            self.addHistory(effect_old, effect_new);
        }
    }.bind(self);
    self.ee.on('setting_x', self.bind_setting_xyz);
    self.ee.on('setting_y', self.bind_setting_xyz);
    self.ee.on('setting_z', self.bind_setting_xyz);

    self.bind_change_effect_range = function(uid, targetuid, range) {
        if(self.uid == uid && self.targetuid == targetuid) {
            self.effect_range = range;
            self.refresh();
            self.focusEffect();
        }
    }.bind(self);
    self.ee.on('change_effect_range', self.bind_change_effect_range);

    self.bind_unfocus_effect = function() {
        self.unfocusEffect();
    }.bind(self);
    self.ee.on('selected_object', self.bind_unfocus_effect);
    self.ee.on('created_object', self.bind_unfocus_effect);
    self.ee.on('select_timeline', self.bind_unfocus_effect);
    self.ee.on('unselected_object', self.bind_unfocus_effect);

    self.bind_select_effect_slider = function(targetuid, effectuid) {
        if(self.targetuid == targetuid && self.uid == effectuid) {
            self.focusEffect();
        }
    }.bind(self);
    self.ee.on('select_effect_slider', self.bind_select_effect_slider);
};
AREffect.prototype.removeExternalEvents = function() {
    var self = this;
    self.ee.off('selected_object',      self.bind_unfocus_effect);
    self.ee.off('created_object',       self.bind_unfocus_effect);
    self.ee.off('select_timeline',      self.bind_unfocus_effect);
    self.ee.off('unselected_object',    self.bind_unfocus_effect);
    self.ee.off('select_effect_slider', self.bind_select_effect_slider);

    self.ee.off('update_object_workspace', self.bind_update_object_workspace);

    self.ee.off('setting_start',    self.bind_setting_start);
    self.ee.off('setting_duration', self.bind_setting_duration);
    self.ee.off('setting_loop',     self.bind_setting_loop);
    self.ee.off('setting_easing',   self.bind_setting_easing);
    self.ee.off('setting_x_chk',    self.bind_setting_x_chk);
    self.ee.off('setting_y_chk',    self.bind_setting_y_chk);
    self.ee.off('setting_z_chk',    self.bind_setting_z_chk);
    self.ee.off('setting_x',        self.bind_setting_xyz);
    self.ee.off('setting_y',        self.bind_setting_xyz);
    self.ee.off('setting_z',        self.bind_setting_xyz);
};
AREffect.prototype.objectUID = function(obj) {
    var self = this;
    if(obj) {
        return obj.userData.uid;
    }

    return self.targetuid;
};