function ARLight(ee) {
    var self = this;
    
    self.uid = null; // 오브젝트 아이디
    
    self.ee = ee;

    self.ar = null; // workspace에 창조된 오브젝트
    self.selected = false;
    self.visible = true;
    self.color = null;
    self.groundColor = null; // For HemisphereLighting
    self.shadow = null;
    self.intensity = null;
    self.distance = null;
    self.angle = null; // For SpotLighting
    self.position = null;
    self.type = null;
    self.label = null;
    self.targetuid = null; // 조명이 비치는 오브젝트, if targetuid is null, target is default as scene.

    self.effects = [];
    self.effectsData = [];
}
ARLight.prototype.create = function(data, callback) {
    var self = this;

    self.setData(data);
    var objData = self.getData();

    self.createObjectOnWS(objData, function(result) {
        var obj = result;

        if(!obj) {
            console.warn('Cannot create ar object on workspace: ', objData);
            self.ee.trigger('fail_create_object', [objData]);
            return null;
        }

        self.ar = obj;

        self.setHtmlIds();
        self.createTimeline();

        /**
         * If has effects property in 'data' then must create effects and add to effects array.
         *
         */
        self.effects = [];
        self.effectsData.forEach(function(item) {
            self.createEffect(item);
        });

        self.focusObject();

        self.ee.trigger('created_object', [self]); // Modified by jjb 2019.06.09. 'created_light' is changed to 'created_object'

        self.addExternalEvent();

        /**
         * After creating ar object, finally reset object data as ar object data
         * like position, scale, rotation, etc...
         */
        self.setData(self.getData());

        //self.initSettings();
        return callback(self);
    });
};
ARLight.prototype.setData = function(data) {
    var self = this;
    var conf = getObjConf(data.type);

    self.uid            = data.uid?         data.uid            : generateUUID();
    self.type           = data.type?        data.type           : '';
    self.color          = data.color?       data.color          : (conf.color? conf.color : '#FFFFFF');
    self.groundColor    = data.groundColor? data.groundColor    : '#eaa92c';
    self.position       = data.position?    data.position       : {x: 100, y: 100, z: 0};
    self.label          = data.label?       data.label          : defaultLabel(data.type);

    self.visible        = data.hasOwnProperty('visible')?   data.visible : true;
    self.shadow         = data.hasOwnProperty('shadow')?    data.shadow : null;

    self.intensity      = typeof data.intensity === 'number'?       data.intensity     : 1;
    self.distance       = typeof data.distance === 'number'?        data.distance      : 0;
    self.angle          = typeof data.angle === 'number'?           data.angle         : Math.PI/6;
    self.targetuid      = data.targetuid? data.targetuid : null;

    self.effectsData    = data.effects?     data.effects        : [];
};
ARLight.prototype.getData = function() {
    var self = this;
    var data = {
        type: self.type,
        uid: self.uid,
        color: self.color,
        groundColor: self.groundColor,
        label: self.label,
        visible: self.visible,
        shadow: copyObj(self.shadow),

        opacity: 1, // Not used, but for regular format!!

        intensity: self.intensity,
        distance: self.distance,
        angle: self.angle,
        targetuid: self.ar? self.ar.userData.targetuid : self.targetuid,

        position: self.ar? copyObj({
            x: self.ar.position.x,
            y: self.ar.position.y,
            z: self.ar.position.z,
        }) : self.position,
        //ar: self.ar,
        effects: self.getEffectsData(),
    };
    return data;
};
ARLight.prototype.getEffectsData = function() {
    var self = this;
    var data = [];
    self.effects.forEach(function(item) {
        data.push(item.getData());
    });
    return data;
};
ARLight.prototype.objectUID = function() {
    return this.uid;
};
ARLight.prototype.addExternalEvent = function() {
    var self = this;

    self.bind_update_seek = function(pos) {
        console.log('update_seek');
        self.refreshEffects(pos);
    }.bind(self);
    self.ee.on('update_seek', self.bind_update_seek);

    self.bind_create_effect = function(data) {
        console.log('create_effect', data);
        var uid = data.targetuid;
        if(self.uid == uid) {
            var effect = self.createEffect(data);
            var saveData = effect.getData();
            AREditor.history.add({
                undo: function() {
                    console.log('create_effect.undo.remove_effect', saveData);
                    AREditor.ee.trigger('remove_effect_log', [saveData]);
                },
                redo: function() {
                    console.log('create_effect.redo.create_effect', saveData);
                    AREditor.ee.trigger('create_effect_log', [saveData]);
                }
            });
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('create_effect', self.bind_create_effect);

    self.bind_create_effect_log = function(data) {
        console.log('create_effect_log', data);
        var uid = data.targetuid;
        if(self.uid == uid) {
            var effect = self.createEffect(data);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('create_effect_log', self.bind_create_effect_log);

    self.bind_remove_effect_log = function(data) {
        console.log('remove_effect_log', data);
        var uid = data.targetuid;
        if(self.uid == uid) {
            var effect = self.removeEffect(data.uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('remove_effect_log', self.bind_remove_effect_log);

    self.bind_remove_effect = function (data) {
        console.log('remove_effect', data);
        var uid = data.targetuid;
        if(self.uid == uid) {
            var effect = self.getEffectByUID(data.uid);
            var saveData = copyObj(effect.getData());
            self.removeEffect(data.uid);
            AREditor.history.add({
                undo: function() {
                    console.log('remove_effect.undo.create_effect', saveData);
                    AREditor.ee.trigger('create_effect_log', [saveData]);
                },
                redo: function() {
                    console.log('remove_effect.redo.remove_effect', saveData);
                    AREditor.ee.trigger('remove_effect_log', [saveData]);
                }
            });
        }
    }.bind(self);
    self.ee.on('remove_effect', self.bind_remove_effect);

    self.bind_setting_effect_log = function(effect_data) {
        var targetuid = effect_data.targetuid;
        if(self.uid == targetuid) {
            self.removeEffect(effect_data.uid);
            self.createEffect(effect_data);
        }
    }.bind(self);
    self.ee.on('setting_effect_log', self.bind_setting_effect_log);

    self.bind_clicked_effect_range = function(obj) {
        console.log('clicked_effect_range');
        var uid = obj.targetuid;
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('clicked_effect_range', self.bind_clicked_effect_range);


    self.bind_select_effect_slider = function(targetuid, effectuid) {
        console.log('select_effect_slider');
        var uid = targetuid;
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }

        var cur_main_pos = Number(self.getMainPos());
        self.refreshEffects(cur_main_pos);
    }.bind(self);
    self.ee.on('select_effect_slider', self.bind_select_effect_slider);


    self.bind_created_effect_slider = function(obj) {
        console.log('created_effect_slider');
        var uid = obj.targetuid;
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('created_effect_slider', self.bind_created_effect_slider);

    self.bind_selected_object = function(obj) {
        console.log('selected_object');
        var uid = obj.userData.uid;
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('selected_object', self.bind_selected_object);


    self.bind_created_object = function(obj) {
        console.log('created_object', obj);
        var uid = obj.uid;

        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('created_object', self.bind_created_object);

    self.bind_unselected_object = function(obj) {
        console.log('unselected_object');
        self.unfocusObject(self.uid);
    }.bind(self);
    self.ee.on('unselected_object', self.bind_unselected_object);

    self.bind_update_object_workspace = function(obj) {
        if(self.uid == obj.userData.uid) {
            console.log('update_object_workspace', obj);
            self.setData(self.getData());
            self.focusObject();
            //self.initSettings();
        } else {
            self.unfocusObject();
        }
    }.bind(self);
    self.ee.on('update_object_workspace', self.bind_update_object_workspace);

    self.bind_select_timeline = function(uid) {
        console.log('select_timeline');
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('select_timeline', self.bind_select_timeline);

    self.bind_set_label = function(uid, val) {
        console.log('set_label: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setLabel(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_label', self.bind_set_label);

    self.bind_set_color = function(uid, val) {
        console.log('set_color: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setColor(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_color', self.bind_set_color);

    self.bind_set_intensity = function(uid, val) {
        console.log('set_intensity: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setIntensity(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_intensity', self.bind_set_intensity);

    self.bind_set_position = function(uid, val) {
        console.log('set_position: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setPosition(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_position', self.bind_set_position);

    self.bind_set_visible = function(uid, val) {
        console.log('set_visible: ' , uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setVisible(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_visible', self.bind_set_visible);

    self.bind_set_intensity = function(uid, val) {
        console.log('set_intensity: ' , uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setIntensity(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_intensity', self.bind_set_intensity);

    self.bind_set_distance = function(uid, val) {
        console.log('set_distance: ' , uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setDistance(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_distance', self.bind_set_distance);

    self.bind_set_angle = function(uid, val) {
        console.log('set_angle: ' , uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setAngle(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_angle', self.bind_set_angle);

    self.bind_set_groundColor = function(uid, val) {
        console.log('set_groundColor: ' , uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setGroundColor(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_groundColor', self.bind_set_groundColor);

    self.bind_setting_object_log = function(data) {
        console.log('setting_object_log: ', data);
        if(self.uid == data.uid) {
            self.setObject(data);
            self.focusObject();
        } else {
            self.unfocusObject();
        }
    }.bind(self);
    self.ee.on('setting_object_log', self.bind_setting_object_log);
};
ARLight.prototype.showSettings = function() {
    LightSettings.show();
};
ARLight.prototype.hideSettings = function() {
    LightSettings.hide();
};
ARLight.prototype.setVisible = function(val) {
    var self = this;
    self.visible = val;
    var data = self.getData();
    self.setObject(data);
};
ARLight.prototype.setLabel = function(val) {
    var self = this;
    self.label = val;
    var data = self.getData();
    data.label = self.label;
    self.setObject(data);
};
ARLight.prototype.setColor = function(val) {
    var self = this;
    self.color = val;
    var data = self.getData();
    //data.color = self.color;
    self.setObject(data);
};
ARLight.prototype.setGroundColor = function(val) {
    var self = this;
    self.groundColor = val;
    var data = self.getData();
    self.setObject(data);
};
ARLight.prototype.setIntensity = function(val) {
    var self = this;
    self.intensity = val;
    var data = self.getData();
    //data.intensity = self.intensity;
    self.setObject(data);
};
ARLight.prototype.setDistance = function(val) {
    var self = this;
    self.distance = val;
    var data = self.getData();
    self.setObject(data);
};
ARLight.prototype.setAngle = function(val) {
    var self = this;
    self.angle = val;
    var data = self.getData();
    self.setObject(data);
};
ARLight.prototype.setPosition = function(val) {
    var self = this;
    self.position = val;
    var data = self.getData();
    data.position = self.position;
    self.setObject(data);
};
ARLight.prototype.setObject = function(data) {
    var self = this;
    var objData = data? data : self.getData();
    self.setData(objData);
    AREditor.workspace.setLightData(self.ar, objData);

    self.initSettings();
};
ARLight.prototype.addHistory = function(undoData, redoData) {
    AREditor.history.add({
        undo: function() {
            AREditor.ee.trigger('setting_object_log', [undoData]);
        },
        redo: function() {
            AREditor.ee.trigger('setting_object_log', [redoData]);
        }
    });
};
ARLight.prototype.removeExternalEvent = function() {
    var self = this;
    self.ee.off('create_effect', self.bind_create_effect);

    self.ee.off('selected_object', self.bind_selected_object);
    self.ee.off('created_object', self.bind_created_object);
    self.ee.off('unselected_object', self.bind_unselected_object);
    self.ee.off('update_object_workspace', self.bind_update_object_workspace);
    self.ee.off('select_timeline', self.bind_select_timeline);

    self.ee.off('set_label', self.bind_set_label);
    self.ee.off('set_color', self.bind_set_color);
    self.ee.off('set_position', self.bind_set_position);
    self.ee.off('setting_object_log', self.bind_setting_object_log);

    self.ee.off('set_visible', self.bind_set_visible);
    self.ee.off('set_intensity', self.bind_set_intensity);
    self.ee.off('set_distance', self.bind_set_distance);
    self.ee.off('set_angle', self.bind_set_angle);
    self.ee.off('set_groundColor', self.bind_set_groundColor);

    self.ee.off('clicked_effect_range', self.bind_clicked_effect_range);
    self.ee.off('select_effect_slider', self.bind_select_effect_slider);
    self.ee.off('created_effect_slider', self.bind_created_effect_slider);
    self.ee.off('create_effect', self.bind_create_effect);
    self.ee.off('remove_effect', self.bind_remove_effect);
    self.ee.off('create_effect_log', self.bind_create_effect_log);
    self.ee.off('remove_effect_log', self.bind_remove_effect_log);
    self.ee.off('update_seek', self.bind_update_seek);
};
ARLight.prototype.removeEffect = function(uid) {
    var self = this;
    self.effects = self.effects.filter(function(item) {
        if(item.uid == uid) {
            item.remove();
            return false;
        }
        return true;
    });

    var cur_main_pos = Number(self.getMainPos());
    self.refreshEffects(cur_main_pos);
};
ARLight.prototype.getEffectByUID = function(uid) {
    var self = this;
    var effect = null;
    self.effects.forEach(function (item) {
        if(item.uid == uid) {
            effect = item;
        }
    });
    return effect;
};
ARLight.prototype.splitEffectsByType = function() {
    var self = this;
    var split_effects = [];
    g_effect_type.forEach(function(item) {
        var effects = [];
        self.effects.forEach(function(effect) {
            if(item.type == effect.effect_type) {
                effects.push(effect);
            }
        });
        split_effects.push({type: item.type, effects: effects});
    });

    return split_effects;
};
ARLight.prototype.loopEffectsList = function() {
    var self = this;
    var effects = [];
    self.effects.forEach(function(item) {
        if(item.loop === true) {
            effects.push(item);
        }
    });
    return effects;
};
ARLight.prototype.effectsOnCurTime = function(pos) {
    var self = this;
    var effects = [];
    self.effects.forEach(function(item) {
        if(item.loop !== true) {
            if (Number(pos) >= Number(item.effect_range.min) && Number(pos) <= Number(item.effect_range.max)) {
                effects.push(item);
            }
        }
    });
    return effects;
};
ARLight.prototype.nearestEffectsCurTimeByMax = function(pos) {
    var self = this;
    var candidates = [];
    var split_effects = self.splitEffectsByType();
    split_effects.forEach(function(item) {
        var effects = item.effects;
        var candidate = null;
        effects.forEach(function(effect) {
            if(effect.loop !== true) {
                if (Number(pos) > Number(effect.effect_range.max)) {
                    if (candidate == null) {
                        candidate = effect;
                    } else {
                        if (effect.effect_range.max >= candidate.effect_range.max) {
                            candidate = effect;
                        }
                    }
                }
            }
        });
        if(candidate) {
            candidates.push(candidate);
        }
    });

    return candidates;
};
ARLight.prototype.nearestEffectsCurTimeByMin = function(pos) {
    var self = this;
    var candidates = [];
    var split_effects = self.splitEffectsByType();
    split_effects.forEach(function(item) {
        var effects = item.effects;
        var candidate = null;
        effects.forEach(function(effect) {
            if(effect.loop !== true) {
                if (Number(pos) < Number(effect.effect_range.min)) {
                    if (candidate == null) {
                        candidate = effect;
                    } else {
                        if (effect.effect_range.min <= candidate.effect_range.min) {
                            candidate = effect;
                        }
                    }
                }
            }
        });
        if(candidate) {
            candidates.push(candidate);
        }
    });

    return candidates;
};
ARLight.prototype.refreshEffects = function(pos) {
    var self = this;
    var loopEffectList = self.loopEffectsList();
    if(loopEffectList.length > 0) {
        loopEffectList.forEach(function(item) {
            item.resetAnimation(Number(pos) * 1000);
        });
    }

    var effectsOnCurTime = self.effectsOnCurTime(pos);
    if(effectsOnCurTime.length > 0) {
        effectsOnCurTime.forEach(function(item) {
            item.resetAnimation(Number(pos) * 1000);
        });
        return;
    }

    var nearestEffectsCurTimeByMax = self.nearestEffectsCurTimeByMax(pos);
    if(nearestEffectsCurTimeByMax.length > 0) {
        nearestEffectsCurTimeByMax.forEach(function(item) {
            item.resetAnimation(Number(pos) * 1000);
        });
        return;
    }

    var nearestEffectsCurTimeByMin = self.nearestEffectsCurTimeByMin(pos);
    if(nearestEffectsCurTimeByMin.length > 0) {
        nearestEffectsCurTimeByMin.forEach(function(item) {
            item.resetAnimation(Number(pos) * 1000);
        });
        return;
    }
};
ARLight.prototype.remove = function() {
    var self = this;

    self.removeExternalEvent();
    self.unfocusObject();
    self.removeEffects();
    self.removeTimeline();
    self.removeOnWS();

    self.ee = null;
    self.ar = null;

    Object.keys(self).forEach(function (key) {
        delete self[key];
    });
    self.__proto__ = {};
};
ARLight.prototype.removeOnWS = function() {
    var self = this;
    var ar = self.ar;
    AREditor.workspace.removeObject(ar);
};
/**
 * Create object on workspace
 */
ARLight.prototype.createObjectOnWS = function(data, callback) {
    var self = this;
    var userData = data? data : self.getData();

    var workspace = AREditor.workspace;
    workspace.createObject(userData, function(result) {
        return callback(result);
    });
};
ARLight.prototype.focusObject = function(uid) {
    var self = this;
    var objuid = uid? uid : self.uid;

    AREditor.workspace.focusObjectByUID(objuid);
    self.selected = true;

    self.focusTimeline(objuid);

    self.initSettings();

    //$('.side2').css('visibility', 'hidden');
    $('#effect_fadein').hide();
    $('#effect_fadeout').hide();
    $('#effect_scale').hide();
    $('#effect_rotate').hide();
    $('#effect_move').show();
};
ARLight.prototype.unfocusObject = function(uid) {
    var self = this;
    var objuid = uid? uid : self.uid;

    self.unfocusTimeline(objuid);

    self.selected = false;
};
ARLight.prototype.initSettings = function() {
    var self = this;
    var data = self.getData();
    LightSettings.init(data);
};
ARLight.prototype.setHtmlIds = function() {
    var self = this;
    var uid = self.uid;

    self.htmlids = {
        timeline_container_id: 'items',

        item_root_id: 'item_root_' + uid,
        item_title_id: 'item_title_' + uid,
        item_slider_id: 'item_slider_' + uid,
        item_effect_cls: 'item_effect',
        item_left_cls: 'item_left',
        item_right_cls: 'item_right',
        item_cls: 'item',

        effect_widget_cls: 'effectWidget'
    };
};
ARLight.prototype.createTimeline = function() {
    var self = this;

    var htmlids = self.htmlids;

    var uid = self.uid;
    var type = self.type;

    var target_id = htmlids.timeline_container_id;
    var target = document.getElementById(target_id);

    var item_root_id = htmlids.item_root_id;
    var item_classname = htmlids.item_cls;
    var item_title_id = htmlids.item_title_id;
    var item_slider_id = htmlids.item_slider_id;

    var item_left_cls = htmlids.item_left_cls;
    var item_right_cls = htmlids.item_right_cls;

    var item_root = document.createElement('div');
    item_root.id = item_root_id;
    item_root.className = item_classname;
    item_root.dataset = {
        uid: uid,
        type: type
    };
    item_root.setAttribute('data-uid', uid);
    item_root.setAttribute('data-type', type);

    var html = "<div id='"+item_title_id+"' class='"+item_left_cls+"'>"+defaultLabel(type)+"</div>"
        +"<div id='"+item_slider_id+"' class='"+item_right_cls+"'></div>";

    item_root.innerHTML = html;
    item_root.onclick = function(e) {
        //self.focusTimeline(uid);
    };
    target.appendChild(item_root);
    document.getElementById(item_title_id).onclick = function(e) {
        self.ee.trigger('select_timeline', [uid]);
    };

    self.timeline = item_root;

    self.effectDragDrop();
};
ARLight.prototype.effectDragDrop = function() {
    var self = this;
    var uid = self.uid;
    var htmlids = self.htmlids;

    var item_slider_id = htmlids.item_slider_id;

    $('#'+item_slider_id).droppable( {
        accept: "#effect_move",
        drop: function(event, ui) {
            var draggable = ui.draggable;
            //var uid = event.target.parentElement.dataset.uid;
            var target_id = event.target.id;
            var effect_type = draggable.attr('data-effect');
            var data = {effect_type: effect_type, targetuid: self.uid};
            self.ee.trigger('create_effect', [data]);
        }
    });
};
ARLight.prototype.getMainPos = function() {
    var self = this;
    var pos = AREditor.mainslider.getPos();
    return pos;
};
ARLight.prototype.createEffect = function(data) {
    var self = this;
    var uid = self.uid;
    var effect_type = data.effect_type;
    var effect_range = null;

    var cur_main_pos = Number(self.getMainPos());
    effect_range = {
        min: data.effect_range? Number(data.effect_range.min).toFixed(2) : cur_main_pos,
        max: data.effect_range? Number(data.effect_range.max).toFixed(2) : cur_main_pos + 3
    };
    data.targetuid = uid;
    data.effect_range = effect_range;
    if(!data.hasOwnProperty('order')) {
        data.order = self.effects.length;
    }

    var insertTarget = self.getInsertTarget(data.order);

    var effect = new AREffect(self.ee, self.ar).create(data, insertTarget);

    self.effects.push(effect);
    self.sortEffects();

    return effect;
};
ARLight.prototype.getInsertTarget = function(order) {
    var self = this;
    var effects = [].concat(self.effects);
    effects.push({order: order});
    var sort = function(a, b) {
        return Number(a.order) - Number(b.order);
    };
    effects.sort(sort);
    var target = null;
    effects.forEach(function(item, index) {
        if(order == item.order) {
            if(effects[index-1]) {
                target = {target: 'after', uid: effects[index-1].uid};
            } else if(effects[index+1]) {
                target = {target: 'before', uid: effects[index+1].uid};
            } else {
                target = {target: 'self', uid: null};
            }
        }
    });
    return target;
};
ARLight.prototype.sortEffects = function() {
    var self = this;

    var sort = function(a, b) {
        return Number(a.order) - Number(b.order);
    };
    self.effects.sort(sort);
};
ARLight.prototype.focusTimeline = function(uid) {
    var self = this;
    var htmlids = self.htmlids;

    var objuid = uid? uid : self.uid;

    var item_root_id = htmlids.item_root_id;
    var item_title_id = htmlids.item_title_id;

    /*
    var ooo1 = document.getElementsByClassName(htmlids.item_left_cls);
    for(var j=0; j<ooo1.length; j++) {
        ooo1[j].style.backgroundColor = 'unset';
    }
    */

    var item_title = document.getElementById(item_title_id);
    var item_root = document.getElementById(item_root_id);

    if(item_title) {
        item_title.style.backgroundColor = '#D9D9D9';
        item_title.innerHTML = self.label;
    }
    self.focusScroll();
};
ARLight.prototype.unfocusTimeline = function(uid) {
    var self = this;
    var htmlids = self.htmlids;

    var objuid = uid? uid : self.uid;

    var item_root_id = htmlids.item_root_id;
    var item_title_id = htmlids.item_title_id;

    var item_title = document.getElementById(item_title_id);
    var item_root = document.getElementById(item_root_id);

    if(item_title) {
        item_title.style.backgroundColor = 'unset';
    }
};
ARLight.prototype.focusScroll = function() {
    var self = this;
    var htmlids = self.htmlids;
    var item_root_id = htmlids.item_root_id;
    var item_root = document.getElementById(item_root_id);

    if(item_root) {
        var topPos = item_root.offsetTop;
        document.getElementById(htmlids.timeline_container_id).scrollTop = topPos;
        //item_root.scrollIntoView();
    }
};
ARLight.prototype.removeEffects = function() {
    var self = this;
    self.effects.forEach(function(item) {
        item.remove();
    });
};
ARLight.prototype.removeTimeline = function() {
    var self = this;

    self.timeline.remove();
};