function ARObject(ee) {
    var self = this;
    
    self.uid = null; // 오브젝트 아이디, workspace에 창조된 오브젝트 uuid로 대치???
    
    self.ee = ee;

    self.ar = null; // workspace에 창조된 오브젝트
    self.timeline = null; // 타임라인 마당
    self.effects = []; // 타임라인에 추가된 이펙트들
    self.effectsData = [];

    self.htmlids = null;

    self.selected = false;

    self.visible = true;
    self.color = null;
    self.opacity = null;
    self.transparent = null;
    self.scale = null;
    self.position = null;
    self.rotation = null;
    self.type = null;
    self.label = null;
    self.texture = null;
    self.link = null;

    self.material = null;
    self.shadow = null;
}
ARObject.prototype.create = function(data, callback) {
    var self = this;

    self.setData(data);
    var objData = self.getData();

    self.createObjectOnWS(objData, function(result) {
        var obj = result;

        if(!obj) {
            console.warn('Cannot create ar object on workspace: ', objData);
            self.ee.trigger('fail_create_object', [objData]);
            return null;
        }

        self.ar = obj;

        self.setHtmlIds();

        self.createTimeline();

        /**
         * If has effects property in 'data' then must create effects and add to effects array.
         *
         */
        self.effects = [];
        self.effectsData.forEach(function(item) {
            self.createEffect(item);
        });



        self.ee.trigger('created_object', [self]);

        self.addExternalEvent();

        /**
         * After creating ar object, finally reset object data as ar object data
         * like position, scale, rotation, etc...
         */
        self.setData(self.getData());

        self.focusObject();

        //self.initSettings();
        return callback(self);
    });
};
ARObject.prototype.setData = function(data) {
    var self = this;
    var conf = getObjConf(data.type);

    self.uid            = data.uid?         data.uid            : generateUUID();
    self.type           = data.type?        data.type           : '';
    self.color          = data.color?       data.color          : (conf.color? conf.color : '#EEEEEE');
    self.scale          = data.scale?       data.scale          : {x: 1, y: 1, z: 1};
    self.position       = data.position?    data.position       : {x: 0, y: 0, z: 0};
    self.rotation       = data.rotation?    data.rotation       : {x: 0, y: 0, z: 0};
    self.label          = data.label?       data.label          : defaultLabel(data.type);
    self.texture        = data.texture?     data.texture        : null;
    self.link           = data.link?        data.link           : null;
    self.resource       = data.resource?    data.resource       : null;
    self.effectsData    = data.effects?     data.effects        : [];
    self.shadow         = data.shadow?      data.shadow         : null;
    self.material       = data.material?    data.material       : 'basic';
    self.visible        = data.hasOwnProperty('visible')?       data.visible       : true;
    self.transparent    = data.hasOwnProperty('transparent')?   data.transparent   : true;
    self.opacity        = typeof data.opacity === 'number'?        data.opacity : 1;

    if(conf.ctg != 'element') {
        if(!self.resource) {
            self.resource = {type: self.type, title: conf.title? conf.title : ''};
        } else if(!self.resource.hasOwnProperty('title')) {
            self.resource.title = conf.title? conf.title : '';
        }
    }

    if(conf.type == 'fbx') {
        if(!self.resource) {
            self.resource = {};
        }
        if(!Array.isArray(self.resource.actionList) || self.resource.actionList.length == 0) {
            self.resource.actionList = [];
            var action = AREditor.workspace.getFirstAction(self.ar);
            if(action) {
                self.resource.actionList.push(action);
            }
        }
    }
};
ARObject.prototype.getData = function() {
    var self = this;
    var data = {
        type: self.type,
        uid: self.uid,
        color: self.color,
        label: self.label,
        visible: self.visible,

        texture: copyObj(self.texture),
        link: copyObj(self.link),
        resource: copyObj(self.resource),

        opacity: self.opacity,
        transparent: self.transparent,

        material: self.material,
        shadow: copyObj(self.shadow),

        scale: self.ar? copyObj({
            x: self.ar.scale.x,
            y: self.ar.scale.y,
            z: self.ar.scale.z,
        }) : (self.scale? self.scale : {x: 1, y: 1, z: 1}),
        position: self.ar? copyObj({
            x: self.ar.position.x,
            y: self.ar.position.y,
            z: self.ar.position.z,
        }) : (self.position? self.position: {x: 0, y: 0, z: 0}),
        rotation: self.ar? copyObj({
            x: self.ar.rotation.x,
            y: self.ar.rotation.y,
            z: self.ar.rotation.z,
        }) : (self.rotation? self.rotation: {x: 0, y: 0, z: 0}),

        effects: self.getEffectsData(),
        //ar: self.ar,
    };
    return data;
};

ARObject.prototype.getEffectsData = function() {
    var self = this;
    var data = [];
    self.effects.forEach(function(item) {
        data.push(item.getData());
    });
    return data;
};
ARObject.prototype.objectUID = function() {
    return this.uid;
};
ARObject.prototype.addExternalEvent = function() {
    var self = this;

    self.bind_create_effect = function(data) {
        console.log('create_effect', data);
        var uid = data.targetuid;
        if(self.uid == uid) {
            var effect = self.createEffect(data);
            var saveData = effect.getData();
            AREditor.history.add({
                undo: function() {
                    console.log('create_effect.undo.remove_effect', saveData);
                    AREditor.ee.trigger('remove_effect_log', [saveData]);
                },
                redo: function() {
                    console.log('create_effect.redo.create_effect', saveData);
                    AREditor.ee.trigger('create_effect_log', [saveData]);
                }
            });
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('create_effect', self.bind_create_effect);

    self.bind_create_effect_log = function(data) {
        console.log('create_effect_log', data);
        var uid = data.targetuid;
        console.log(self);
        if(self.uid == uid) {
            var effect = self.createEffect(data);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('create_effect_log', self.bind_create_effect_log);

    self.bind_remove_effect_log = function(data) {
        console.log('remove_effect_log', data);
        var uid = data.targetuid;
        if(self.uid == uid) {
            var effect = self.removeEffect(data.uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('remove_effect_log', self.bind_remove_effect_log);


    self.bind_remove_effect = function (data) {
        console.log('remove_effect', data);
        var uid = data.targetuid;
        if(self.uid == uid) {
            var effect = self.getEffectByUID(data.uid);
            var saveData = copyObj(effect.getData());
            self.removeEffect(data.uid);
            AREditor.history.add({
                undo: function() {
                    console.log('remove_effect.undo.create_effect', saveData);
                    AREditor.ee.trigger('create_effect_log', [saveData]);
                },
                redo: function() {
                    console.log('remove_effect.redo.remove_effect', saveData);
                    AREditor.ee.trigger('remove_effect_log', [saveData]);
                }
            });
        }
    }.bind(self);
    self.ee.on('remove_effect', self.bind_remove_effect);


    self.bind_setting_effect_log = function(effect_data) {
        var targetuid = effect_data.targetuid;
        if(self.uid == targetuid) {
            self.removeEffect(effect_data.uid);
            self.createEffect(effect_data);
        }
    }.bind(self);
    self.ee.on('setting_effect_log', self.bind_setting_effect_log);


    self.bind_selected_object = function(obj) {
        console.log('selected_object');
        var uid = obj.userData.uid;
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('selected_object', self.bind_selected_object);


    self.bind_created_object = function(obj) {
        console.log('created_object', obj);
        var uid = obj.uid;

        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('created_object', self.bind_created_object);


    self.bind_select_timeline = function(uid) {
        console.log('select_timeline');
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('select_timeline', self.bind_select_timeline);

    self.bind_clicked_effect_range = function(obj) {
        console.log('clicked_effect_range');
        var uid = obj.targetuid;
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('clicked_effect_range', self.bind_clicked_effect_range);


    self.bind_select_effect_slider = function(targetuid, effectuid) {
        console.log('select_effect_slider');
        var uid = targetuid;
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }

        var cur_main_pos = Number(self.getMainPos());
        self.refreshEffects(cur_main_pos);
    }.bind(self);
    self.ee.on('select_effect_slider', self.bind_select_effect_slider);


    self.bind_created_effect_slider = function(obj) {
        console.log('created_effect_slider');
        var uid = obj.targetuid;
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('created_effect_slider', self.bind_created_effect_slider);


    self.bind_unselected_object = function(obj) {
        console.log('unselected_object');
        self.unfocusObject(self.uid);
    }.bind(self);
    self.ee.on('unselected_object', self.bind_unselected_object);


    self.bind_update_seek = function(pos) {
        console.log('update_seek');
        self.refreshEffects(pos);
    }.bind(self);
    self.ee.on('update_seek', self.bind_update_seek);

    self.bind_update_object_workspace = function(obj) {
        if(self.uid == obj.userData.uid) {
            console.log('update_object_workspace', obj);
            self.setData(self.getData());
            self.focusObject();
            //self.initSettings();
        } else {
            self.unfocusObject();
        }
    }.bind(self);
    self.ee.on('update_object_workspace', self.bind_update_object_workspace);

    self.bind_set_label = function(uid, val) {
        console.log('set_label: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setLabel(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_label', self.bind_set_label);

    self.bind_set_color = function(uid, val) {
        console.log('set_color: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setColor(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_color', self.bind_set_color);

    self.bind_set_opacity = function(uid, val) {
        console.log('set_opacity: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setOpacity(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_opacity', self.bind_set_opacity);

    self.bind_set_scale = function(uid, val) {
        console.log('set_scale: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setScale(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_scale', self.bind_set_scale);

    self.bind_set_position = function(uid, val) {
        console.log('set_position: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setPosition(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_position', self.bind_set_position);

    self.bind_set_rotation = function(uid, val) {
        console.log('set_rotation: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setRotation(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_rotation', self.bind_set_rotation);

    self.bind_set_texture = function(uid, val) {
        console.log('set_texture: ' , uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setTexture(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_texture', self.bind_set_texture);

    self.bind_set_resource = function(uid, val) {
        console.log('set_resource: ' , uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setResource(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_resource', self.bind_set_resource);

    self.bind_set_link = function(uid, val) {
        console.log('set_link: ' , uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setLink(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_link', self.bind_set_link);

    self.bind_set_visible = function(uid, val) {
        console.log('set_visible: ' , uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setVisible(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_visible', self.bind_set_visible);

    self.bind_set_shadow = function(uid, val) {
        console.log('set_shadow: ' , uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setShadow(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_shadow', self.bind_set_shadow);

    self.bind_set_material = function(uid, val) {
        console.log('set_material: ' , uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setMaterial(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);

            /**
             * If changed material type, remove and create object in Editor.js
             */
            self.ee.trigger('reset_material', [newData]);
        }
    }.bind(self);
    self.ee.on('set_material', self.bind_set_material);

    self.bind_setting_object_log = function(data) {
        console.log('setting_object_log: ', data);
        if(self.uid == data.uid) {
            if(self.material != data.material) {
                self.ee.trigger('reset_material', [data]);
                return;
            }
            self.setObject(data);
            self.focusObject();
        } else {
            self.unfocusObject();
        }
    }.bind(self);
    self.ee.on('setting_object_log', self.bind_setting_object_log);
};
ARObject.prototype.showSettings = function() {
    ObjectSettings.show();
};
ARObject.prototype.hideSettings = function() {
    ObjectSettings.hide();
};
ARObject.prototype.setMaterial = function(val) {
    var self = this;
    self.material = val;
    var data = self.getData();
    self.setObject(data);
};
ARObject.prototype.setShadow = function(val) {
    var self = this;
    self.shadow = copyObj(val);
    var data = self.getData();
    self.setObject(data);
};
ARObject.prototype.setVisible = function(val) {
    var self = this;
    self.visible = val;
    var data = self.getData();
    self.setObject(data);
};
ARObject.prototype.setLabel = function(val) {
    var self = this;
    self.label = val;
    var data = self.getData();
    data.label = self.label;
    self.setObject(data);
};
ARObject.prototype.setColor = function(val) {
    var self = this;
    self.color = val;
    var data = self.getData();
    data.color = self.color;
    self.setObject(data);
};
ARObject.prototype.setOpacity = function(val) {
    var self = this;
    self.opacity = val;
    var data = self.getData();
    data.opacity = self.opacity;
    self.setObject(data);
};
ARObject.prototype.setScale = function(val) {
    var self = this;
    self.scale = val;
    var data = self.getData();
    data.scale = self.scale;
    self.setObject(data);
};
ARObject.prototype.setPosition = function(val) {
    var self = this;
    self.position = val;
    var data = self.getData();
    data.position = self.position;
    self.setObject(data);
};
ARObject.prototype.setRotation = function(val) {
    var self = this;
    self.rotation = val;
    var data = self.getData();
    data.rotation = self.rotation;
    self.setObject(data);
};
ARObject.prototype.setTexture = function(val) {
    var self = this;
    self.texture = val;
    var data = self.getData();
    self.setObject(data);
};
ARObject.prototype.setResource = function(val) {
    var self = this;
    self.resource = val;
    var data = self.getData();
    self.setObject(data);
};
ARObject.prototype.setLink = function(val) {
    var self = this;
    self.link = val;
    var data = self.getData();
    self.setObject(data);
};
ARObject.prototype.setObject = function(data) {
    var self = this;
    var objData = data? data : self.getData();
    self.setData(objData);
    AREditor.workspace.setData(self.ar, objData);

    //self.initSettings();
};
ARObject.prototype.addHistory = function(undoData, redoData) {
    AREditor.history.add({
        undo: function() {
            AREditor.ee.trigger('setting_object_log', [undoData]);
        },
        redo: function() {
            AREditor.ee.trigger('setting_object_log', [redoData]);
        }
    });
};
ARObject.prototype.removeExternalEvent = function() {
    var self = this;
    self.ee.off('selected_object', self.bind_selected_object);
    self.ee.off('created_object', self.bind_created_object);
    self.ee.off('select_timeline', self.bind_select_timeline);
    self.ee.off('clicked_effect_range', self.bind_clicked_effect_range);
    self.ee.off('select_effect_slider', self.bind_select_effect_slider);
    self.ee.off('created_effect_slider', self.bind_created_effect_slider);
    self.ee.off('unselected_object', self.bind_unselected_object);
    self.ee.off('update_seek', self.bind_update_seek);
    self.ee.off('update_object_workspace', self.bind_update_object_workspace);

    self.ee.off('create_effect', self.bind_create_effect);
    self.ee.off('remove_effect', self.bind_remove_effect);
    self.ee.off('create_effect_log', self.bind_create_effect_log);
    self.ee.off('remove_effect_log', self.bind_remove_effect_log);

    self.ee.off('set_label', self.bind_set_label);
    self.ee.off('set_color', self.bind_set_color);
    self.ee.off('set_opacity', self.bind_set_opacity);
    self.ee.off('set_scale', self.bind_set_scale);
    self.ee.off('set_position', self.bind_set_position);
    self.ee.off('set_rotation', self.bind_set_rotation);
    self.ee.off('set_texture', self.bind_set_texture);
    self.ee.off('set_resource', self.bind_set_resource);
    self.ee.off('set_link', self.bind_set_link);
    self.ee.off('setting_object_log', self.bind_setting_object_log);

    self.ee.off('set_visible', self.bind_set_visible);
    self.ee.off('set_material', self.bind_set_material);
};
ARObject.prototype.getEffectByUID = function(uid) {
    var self = this;
    var effect = null;
    self.effects.forEach(function (item) {
        if(item.uid == uid) {
            effect = item;
        }
    });
    return effect;
};
ARObject.prototype.splitEffectsByType = function() {
    var self = this;
    var split_effects = [];
    g_effect_type.forEach(function(item) {
        var effects = [];
        self.effects.forEach(function(effect) {
            if(item.type == effect.effect_type) {
                effects.push(effect);
            }
        });
        split_effects.push({type: item.type, effects: effects});
    });

    return split_effects;
};
ARObject.prototype.loopEffectsList = function() {
    var self = this;
    var effects = [];
    self.effects.forEach(function(item) {
        if(item.loop === true) {
            effects.push(item);
        }
    });
    return effects;
};
ARObject.prototype.effectsOnCurTime = function(pos) {
    var self = this;
    var effects = [];
    self.effects.forEach(function(item) {
        if(item.loop !== true) {
            if (Number(pos) >= Number(item.effect_range.min) && Number(pos) <= Number(item.effect_range.max)) {
                effects.push(item);
            }
        }
    });
    return effects;
};
ARObject.prototype.nearestEffectsCurTimeByMax = function(pos) {
    var self = this;
    var candidates = [];
    var split_effects = self.splitEffectsByType();
    split_effects.forEach(function(item) {
        var effects = item.effects;
        var candidate = null;
        effects.forEach(function(effect) {
            if(effect.loop !== true) {
                if (Number(pos) > Number(effect.effect_range.max)) {
                    if (candidate == null) {
                        candidate = effect;
                    } else {
                        if (effect.effect_range.max >= candidate.effect_range.max) {
                            candidate = effect;
                        }
                    }
                }
            }
        });
        if(candidate) {
            candidates.push(candidate);
        }
    });

    return candidates;
};
ARObject.prototype.nearestEffectsCurTimeByMin = function(pos) {
    var self = this;
    var candidates = [];
    var split_effects = self.splitEffectsByType();
    split_effects.forEach(function(item) {
        var effects = item.effects;
        var candidate = null;
        effects.forEach(function(effect) {
            if(effect.loop !== true) {
                if (Number(pos) < Number(effect.effect_range.min)) {
                    if (candidate == null) {
                        candidate = effect;
                    } else {
                        if (effect.effect_range.min <= candidate.effect_range.min) {
                            candidate = effect;
                        }
                    }
                }
            }
        });
        if(candidate) {
            candidates.push(candidate);
        }
    });

    return candidates;
};
ARObject.prototype.refreshEffects = function(pos) {
    var self = this;
    var loopEffectList = self.loopEffectsList();
    if(loopEffectList.length > 0) {
        loopEffectList.forEach(function(item) {
            item.resetAnimation(Number(pos) * 1000);
        });
    }

    var effectsOnCurTime = self.effectsOnCurTime(pos);
    if(effectsOnCurTime.length > 0) {
        effectsOnCurTime.forEach(function(item) {
            item.resetAnimation(Number(pos) * 1000);
        });
        return;
    }

    var nearestEffectsCurTimeByMax = self.nearestEffectsCurTimeByMax(pos);
    if(nearestEffectsCurTimeByMax.length > 0) {
        nearestEffectsCurTimeByMax.forEach(function(item) {
            item.resetAnimation(Number(pos) * 1000);
        });
        return;
    }

    var nearestEffectsCurTimeByMin = self.nearestEffectsCurTimeByMin(pos);
    if(nearestEffectsCurTimeByMin.length > 0) {
        nearestEffectsCurTimeByMin.forEach(function(item) {
            item.resetAnimation(Number(pos) * 1000);
        });
        return;
    }
};
ARObject.prototype.removeEffect = function(uid) {
    var self = this;
    self.effects = self.effects.filter(function(item) {
        if(item.uid == uid) {
            item.remove();
            return false;
        }
        return true;
    });

    var cur_main_pos = Number(self.getMainPos());
    self.refreshEffects(cur_main_pos);
};
ARObject.prototype.remove = function() {
    var self = this;

    self.removeExternalEvent();
    self.unfocusObject();
    self.removeEffects();
    self.removeTimeline();
    self.removeOnWS();

    self.ee = null;
    self.ar = null;
    self.timeline = null;
    self.effects = [];

    Object.keys(self).forEach(function (key) {
        delete self[key];
    });
    self.__proto__ = {};
};
ARObject.prototype.removeOnWS = function() {
    var self = this;
    var ar = self.ar;
    AREditor.workspace.removeObject(ar);
};
ARObject.prototype.removeTimeline = function() {
    var self = this;

    self.timeline.remove();
};
ARObject.prototype.removeEffects = function() {
    var self = this;
    self.effects.forEach(function(item) {
        item.remove();
    });
};
ARObject.prototype.setHtmlIds = function() {
    var self = this;
    var uid = self.uid;

    self.htmlids = {
        timeline_container_id: 'items',

        item_root_id: 'item_root_' + uid,
        item_title_id: 'item_title_' + uid,
        item_slider_id: 'item_slider_' + uid,
        item_effect_cls: 'item_effect',
        item_left_cls: 'item_left',
        item_right_cls: 'item_right',
        item_cls: 'item',

        effect_widget_cls: 'effectWidget'
    };
};

/**
 * Create object on workspace
 */
ARObject.prototype.createObjectOnWS = function(data, callback) {
    var self = this;
    var userData = data? data : self.getData();

    var workspace = AREditor.workspace;
    workspace.createObject(userData, function(result) {
        return callback(result);
    });
};
ARObject.prototype.createTimeline = function() {
    var self = this;

    var htmlids = self.htmlids;

    var uid = self.uid;
    var type = self.type;

    var target_id = htmlids.timeline_container_id;
    var target = document.getElementById(target_id);

    var item_root_id = htmlids.item_root_id;
    var item_classname = htmlids.item_cls;
    var item_title_id = htmlids.item_title_id;
    var item_slider_id = htmlids.item_slider_id;

    var item_left_cls = htmlids.item_left_cls;
    var item_right_cls = htmlids.item_right_cls;

    var item_root = document.createElement('div');
    item_root.id = item_root_id;
    item_root.className = item_classname;
    item_root.dataset = {
        uid: uid,
        type: type
    };
    item_root.setAttribute('data-uid', uid);
    item_root.setAttribute('data-type', type);

    var html = "<div id='"+item_title_id+"' class='"+item_left_cls+"'>"+defaultLabel(type)+"</div>"
        +"<div id='"+item_slider_id+"' class='"+item_right_cls+"'></div>";

    item_root.innerHTML = html;
    item_root.onclick = function(e) {
        //self.focusTimeline(uid);
    };
    target.appendChild(item_root);
    document.getElementById(item_title_id).onclick = function(e) {
        self.ee.trigger('select_timeline', [uid]);
    };

    self.timeline = item_root;

    self.effectDragDrop();
};
ARObject.prototype.effectDragDrop = function() {
    var self = this;
    var uid = self.uid;
    var htmlids = self.htmlids;

    var item_slider_id = htmlids.item_slider_id;

    $('#'+item_slider_id).droppable( {
        accept: "."+htmlids.effect_widget_cls,
        drop: function(event, ui) {
            var draggable = ui.draggable;
            //var uid = event.target.parentElement.dataset.uid;
            var target_id = event.target.id;
            var effect_type = draggable.attr('data-effect');
            var data = {effect_type: effect_type, targetuid: self.uid};
            self.ee.trigger('create_effect', [data]);
        }
    });
};
ARObject.prototype.getMainPos = function() {
    var self = this;
    var pos = AREditor.mainslider.getPos();
    return pos;
};
ARObject.prototype.createEffect = function(data) {
    var self = this;
    var uid = self.uid;
    var effect_type = data.effect_type;
    var effect_range = null;

    var cur_main_pos = Number(self.getMainPos());
    effect_range = {
        min: data.effect_range? Number(data.effect_range.min).toFixed(2) : cur_main_pos,
        max: data.effect_range? Number(data.effect_range.max).toFixed(2) : cur_main_pos + 3
    };
    data.targetuid = uid;
    data.effect_range = effect_range;
    if(!data.hasOwnProperty('order')) {
        data.order = self.effects.length;
    }

    var insertTarget = self.getInsertTarget(data.order);

    //var effect = new EffectSlider(self.ee, self.ar).create(data, insertTarget);

    var effect = new AREffect(self.ee, self.ar).create(data, insertTarget);

    self.effects.push(effect);
    self.sortEffects();

    return effect;
};
ARObject.prototype.getInsertTarget = function(order) {
    var self = this;
    var effects = [].concat(self.effects);
    effects.push({order: order});
    var sort = function(a, b) {
        return Number(a.order) - Number(b.order);
    };
    effects.sort(sort);
    var target = null;
    effects.forEach(function(item, index) {
        if(order == item.order) {
            if(effects[index-1]) {
                target = {target: 'after', uid: effects[index-1].uid};
            } else if(effects[index+1]) {
                target = {target: 'before', uid: effects[index+1].uid};
            } else {
                target = {target: 'self', uid: null};
            }
        }
    });
    return target;
};
ARObject.prototype.sortEffects = function() {
    var self = this;

    var sort = function(a, b) {
        return Number(a.order) - Number(b.order);
    };
    self.effects.sort(sort);
};
ARObject.prototype.focusObject = function(uid) {
    var self = this;
    var objuid = uid? uid : self.uid;

    AREditor.workspace.focusObjectByUID(objuid);
    self.focusTimeline(objuid);

    self.selected = true;

    self.initSettings();

    //$('.side2').css('visibility', 'visible');
    $('#effect_fadein').show();
    $('#effect_fadeout').show();
    $('#effect_scale').show();
    $('#effect_rotate').show();
    $('#effect_move').show();
};
ARObject.prototype.unfocusObject = function(uid) {
    var self = this;
    var objuid = uid? uid : self.uid;

    self.unfocusTimeline(objuid);

    self.selected = false;
};
ARObject.prototype.focusScroll = function() {
    var self = this;
    var htmlids = self.htmlids;
    var item_root_id = htmlids.item_root_id;
    var item_root = document.getElementById(item_root_id);

    if(item_root) {
        var topPos = item_root.offsetTop;
        document.getElementById(htmlids.timeline_container_id).scrollTop = topPos;
        //item_root.scrollIntoView();
    }
};
ARObject.prototype.focusTimeline = function(uid) {
    var self = this;
    var htmlids = self.htmlids;

    var objuid = uid? uid : self.uid;

    var item_root_id = htmlids.item_root_id;
    var item_title_id = htmlids.item_title_id;

    /*
    var ooo1 = document.getElementsByClassName(htmlids.item_left_cls);
    for(var j=0; j<ooo1.length; j++) {
        ooo1[j].style.backgroundColor = 'unset';
    }
    */


    var item_title = document.getElementById(item_title_id);
    var item_root = document.getElementById(item_root_id);

    if(item_title) {
        item_title.style.backgroundColor = '#D9D9D9';
        item_title.innerHTML = self.label;
    }
    self.focusScroll();
};
ARObject.prototype.unfocusTimeline = function(uid) {
    var self = this;
    var htmlids = self.htmlids;

    var objuid = uid? uid : self.uid;

    var item_root_id = htmlids.item_root_id;
    var item_title_id = htmlids.item_title_id;

    var item_title = document.getElementById(item_title_id);
    var item_root = document.getElementById(item_root_id);

    if(item_title) {
        item_title.style.backgroundColor = 'unset';
    }
};
ARObject.prototype.initSettings = function() {
    var self = this;
    var data = self.getData();
    ObjectSettings.init(data);
};