function ARParticle(ee) {
    var self = this;
    
    self.uid = null; // 오브젝트 아이디
    
    self.ee = ee;

    self.ar = null; // workspace에 창조된 오브젝트
    self.selected = false;
    self.visible = true;
    self.position = null;
    self.rotation = null;
    self.scale = null;
    self.type = null;
    self.label = null;
    self.settings = null;
}
ARParticle.prototype.create = function(data, callback) {
    var self = this;

    self.setData(data);
    var objData = self.getData();

    self.createObjectOnWS(objData, function(result) {
        var obj = result;

        if(!obj) {
            console.warn('Cannot create ar object on workspace: ', objData);
            self.ee.trigger('fail_create_object', [objData]);
            return null;
        }

        self.ar = obj;

        self.setHtmlIds();
        self.createTimeline();

        self.focusObject();

        self.ee.trigger('created_object', [self]); // Modified by jjb 2019.06.09. 'created_light' is changed to 'created_object'

        self.addExternalEvent();

        /**
         * After creating ar object, finally reset object data as ar object data
         * like position, scale, rotation, etc...
         */
        self.setData(self.getData());

        //self.initSettings();
        return callback(self);
    });
};
ARParticle.prototype.setData = function(data) {
    var self = this;
    var conf = getObjConf(data.type);

    self.uid            = data.uid?         data.uid            : generateUUID();
    self.type           = data.type?        data.type           : '';
    self.position       = data.position?    data.position       : {x: 0, y: 0, z: 0};
    self.rotation       = data.rotation?    data.rotation       : {x: 0, y: 0, z: 0};
    self.scale          = data.scale?       data.scale          : {x: 1, y: 1, z: 1};
    self.label          = data.label?       data.label          : defaultLabel(conf.type);

    self.visible        = data.hasOwnProperty('visible')?   data.visible : true;

    self.settings       = data.settings? data.settings : {};
};
ARParticle.prototype.getData = function() {
    var self = this;
    var data = {
        type: self.type,
        uid: self.uid,
        label: self.label,
        visible: self.visible,

        scale: self.ar? copyObj({
            x: self.ar.scale.x,
            y: self.ar.scale.y,
            z: self.ar.scale.z,
        }) : (self.scale? self.scale : {x: 1, y: 1, z: 1}),
        position: self.ar? copyObj({
            x: self.ar.position.x,
            y: self.ar.position.y,
            z: self.ar.position.z,
        }) : (self.position? self.position: {x: 0, y: 0, z: 0}),
        rotation: self.ar? copyObj({
            x: self.ar.rotation.x,
            y: self.ar.rotation.y,
            z: self.ar.rotation.z,
        }) : (self.rotation? self.rotation: {x: 0, y: 0, z: 0}),

        settings: self.ar? copyObj(self.ar.userData.settings) : self.settings,
    };

    return data;
};
ARParticle.prototype.objectUID = function() {
    return this.uid;
};
ARParticle.prototype.addExternalEvent = function() {
    var self = this;

    self.bind_create_effect = function(data) {
        console.log('create_effect', data);
        var uid = data.targetuid;
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('create_effect', self.bind_create_effect);

    self.bind_create_effect_log = function(data) {
        console.log('create_effect_log', data);
        var uid = data.targetuid;
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('create_effect_log', self.bind_create_effect_log);

    self.bind_clicked_effect_range = function(obj) {
        console.log('clicked_effect_range');
        var uid = obj.targetuid;
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('clicked_effect_range', self.bind_clicked_effect_range);

    self.bind_select_effect_slider = function(targetuid, effectuid) {
        console.log('select_effect_slider');
        var uid = targetuid;
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('select_effect_slider', self.bind_select_effect_slider);


    self.bind_created_effect_slider = function(obj) {
        console.log('created_effect_slider');
        var uid = obj.targetuid;
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('created_effect_slider', self.bind_created_effect_slider);

    self.bind_selected_object = function(obj) {
        console.log('selected_object');
        var uid = obj.userData.uid;
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('selected_object', self.bind_selected_object);


    self.bind_created_object = function(obj) {
        console.log('created_object', obj);
        var uid = obj.uid;

        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('created_object', self.bind_created_object);


    self.bind_unselected_object = function(obj) {
        console.log('unselected_object');
        self.unfocusObject(self.uid);
    }.bind(self);
    self.ee.on('unselected_object', self.bind_unselected_object);


    self.bind_update_object_workspace = function(obj) {
        if(self.uid == obj.userData.uid) {
            console.log('update_object_workspace', obj);
            self.setData(self.getData());
            self.focusObject();
            if(typeof self.ar.refreshSettings === 'function') {
                //self.ar.refreshSettings(self.settings);
            }
            AREditor.save();
            //self.initSettings();
        } else {
            self.unfocusObject();
        }
    }.bind(self);
    self.ee.on('update_object_workspace', self.bind_update_object_workspace);

    self.bind_select_timeline = function(uid) {
        console.log('select_timeline');
        if(self.uid == uid) {
            self.focusObject(uid);
        } else {
            self.unfocusObject(uid);
        }
    }.bind(self);
    self.ee.on('select_timeline', self.bind_select_timeline);

    self.bind_set_label = function(uid, val) {
        console.log('set_label: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setLabel(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_label', self.bind_set_label);


    self.bind_set_position = function(uid, val) {
        console.log('set_position: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setPosition(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_position', self.bind_set_position);

    self.bind_set_rotation = function(uid, val) {
        console.log('set_rotation: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setRotation(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_rotation', self.bind_set_rotation);

    self.bind_set_scale = function(uid, val) {
        console.log('set_scale: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setScale(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_scale', self.bind_set_scale);

    self.bind_set_visible = function(uid, val) {
        console.log('set_visible: ' , uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setVisible(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_visible', self.bind_set_visible);

    self.bind_setting_object_log = function(data) {
        console.log('setting_object_log: ', data);
        if(self.uid == data.uid) {
            self.setObject(data);
            self.focusObject();
        } else {
            self.unfocusObject();
        }
    }.bind(self);
    self.ee.on('setting_object_log', self.bind_setting_object_log);
};
ARParticle.prototype.showSettings = function() {
    ParticleSettings.show();
};
ARParticle.prototype.hideSettings = function() {
    ParticleSettings.hide();
};
ARParticle.prototype.setVisible = function(val) {
    var self = this;
    self.visible = val;
    var data = self.getData();
    self.setObject(data);
};
ARParticle.prototype.setLabel = function(val) {
    var self = this;
    self.label = val;
    var data = self.getData();
    data.label = self.label;
    self.setObject(data);
};
ARParticle.prototype.setPosition = function(val) {
    var self = this;
    self.position = val;
    var data = self.getData();
    data.position = self.position;
    self.setObject(data);
};
ARParticle.prototype.setRotation = function(val) {
    var self = this;
    self.rotation = val;
    var data = self.getData();
    data.rotation = self.rotation;
    self.setObject(data);
};
ARParticle.prototype.setScale = function(val) {
    var self = this;
    self.scale = val;
    var data = self.getData();
    data.scale = self.scale;
    self.setObject(data);
};
ARParticle.prototype.setObject = function(data) {
    var self = this;
    var objData = data? data : self.getData();
    self.setData(objData);
    AREditor.workspace.setParticleData(self.ar, objData);

    self.initSettings();
};
ARParticle.prototype.addHistory = function(undoData, redoData) {
    AREditor.history.add({
        undo: function() {
            AREditor.ee.trigger('setting_object_log', [undoData]);
        },
        redo: function() {
            AREditor.ee.trigger('setting_object_log', [redoData]);
        }
    });
};
ARParticle.prototype.removeExternalEvent = function() {
    var self = this;
    self.ee.off('create_effect', self.bind_create_effect);

    self.ee.off('selected_object', self.bind_selected_object);
    self.ee.off('created_object', self.bind_created_object);
    self.ee.off('unselected_object', self.bind_unselected_object);
    self.ee.off('update_object_workspace', self.bind_update_object_workspace);
    self.ee.off('select_timeline', self.bind_select_timeline);

    self.ee.off('set_label', self.bind_set_label);
    self.ee.off('set_color', self.bind_set_color);
    self.ee.off('set_position', self.bind_set_position);
    self.ee.off('setting_object_log', self.bind_setting_object_log);

    self.ee.off('set_visible', self.bind_set_visible);
    self.ee.off('set_intensity', self.bind_set_intensity);
    self.ee.off('set_distance', self.bind_set_distance);
    self.ee.off('set_angle', self.bind_set_angle);
    self.ee.off('set_groundColor', self.bind_set_groundColor);

    self.ee.off('clicked_effect_range', self.bind_clicked_effect_range);
    self.ee.off('select_effect_slider', self.bind_select_effect_slider);
    self.ee.off('created_effect_slider', self.bind_created_effect_slider);
    self.ee.off('create_effect', self.bind_create_effect);
    self.ee.off('remove_effect', self.bind_remove_effect);
    self.ee.off('create_effect_log', self.bind_create_effect_log);
    self.ee.off('remove_effect_log', self.bind_remove_effect_log);
};
ARParticle.prototype.getEffectByUID = function(uid) {
    var self = this;
    var effect = null;
    self.effects.forEach(function (item) {
        if(item.uid == uid) {
            effect = item;
        }
    });
    return effect;
};
ARParticle.prototype.remove = function() {
    var self = this;

    self.removeExternalEvent();
    self.unfocusObject();
    self.removeTimeline();
    self.removeOnWS();

    self.ee = null;
    self.ar = null;

    Object.keys(self).forEach(function (key) {
        delete self[key];
    });
    self.__proto__ = {};
};
ARParticle.prototype.removeOnWS = function() {
    var self = this;
    AREditor.workspace.removeParticle(self.ar);
};
/**
 * Create object on workspace
 */
ARParticle.prototype.createObjectOnWS = function(data, callback) {
    var self = this;
    var userData = data? data : self.getData();

    var workspace = AREditor.workspace;
    workspace.createObject(userData, function(result) {
        return callback(result);
    });
};
ARParticle.prototype.focusObject = function(uid) {
    var self = this;
    var objuid = uid? uid : self.uid;

    AREditor.workspace.focusObjectByUID(objuid);
    self.selected = true;

    self.focusTimeline(objuid);

    //$('.side2').css('visibility', 'hidden');
    $('#effect_fadein').hide();
    $('#effect_fadeout').hide();
    $('#effect_scale').hide();
    $('#effect_rotate').hide();
    $('#effect_move').hide();

    self.initSettings();
};
ARParticle.prototype.unfocusObject = function(uid) {
    var self = this;
    var objuid = uid? uid : self.uid;

    self.unfocusTimeline(objuid);

    self.selected = false;

    if(self.ar.gui) {
        var id = $('#gui_control').attr('data-uid');
        if(id == self.uid) {
            //objById('gui_control').innerHTML = '';
            $('#gui_control').hide();
        }
    }
};
ARParticle.prototype.initSettings = function() {
    var self = this;
    var data = self.getData();
    ParticleSettings.init(data);
    if(typeof self.ar.refreshSettings === "function") {
        self.ar.refreshSettings();
    }

    if(AREditor.isPreview === false) {
        var id = $('#gui_control').attr('data-uid');
        if(id == self.uid) {
            //objById('gui_control').innerHTML = '';
            $('#gui_control').show();
        }
    } else {
        $('#gui_control').hide();
    }
};
ARParticle.prototype.setHtmlIds = function() {
    var self = this;
    var uid = self.uid;

    self.htmlids = {
        timeline_container_id: 'items',

        item_root_id: 'item_root_' + uid,
        item_title_id: 'item_title_' + uid,
        item_slider_id: 'item_slider_' + uid,
        item_effect_cls: 'item_effect',
        item_left_cls: 'item_left',
        item_right_cls: 'item_right',
        item_cls: 'item',

        effect_widget_cls: 'effectWidget'
    };
};
ARParticle.prototype.createTimeline = function() {
    var self = this;

    var htmlids = self.htmlids;

    var uid = self.uid;
    var type = self.type;

    var target_id = htmlids.timeline_container_id;
    var target = document.getElementById(target_id);

    var item_root_id = htmlids.item_root_id;
    var item_classname = htmlids.item_cls;
    var item_title_id = htmlids.item_title_id;
    var item_slider_id = htmlids.item_slider_id;

    var item_left_cls = htmlids.item_left_cls;
    var item_right_cls = htmlids.item_right_cls;

    var item_root = document.createElement('div');
    item_root.id = item_root_id;
    item_root.className = item_classname;
    item_root.dataset = {
        uid: uid,
        type: type
    };
    item_root.setAttribute('data-uid', uid);
    item_root.setAttribute('data-type', type);

    var html = "<div id='"+item_title_id+"' class='"+item_left_cls+"'>"+defaultLabel(type)+"</div>"
        +"<div id='"+item_slider_id+"' class='"+item_right_cls+"'></div>";

    item_root.innerHTML = html;
    item_root.onclick = function(e) {
        //self.focusTimeline(uid);
    };
    target.appendChild(item_root);
    document.getElementById(item_title_id).onclick = function(e) {
        self.ee.trigger('select_timeline', [uid]);
    };

    self.timeline = item_root;
};
ARParticle.prototype.getMainPos = function() {
    var self = this;
    var pos = AREditor.mainslider.getPos();
    return pos;
};
ARParticle.prototype.focusTimeline = function(uid) {
    var self = this;
    var htmlids = self.htmlids;

    var objuid = uid? uid : self.uid;

    var item_root_id = htmlids.item_root_id;
    var item_title_id = htmlids.item_title_id;

    /*
    var ooo1 = document.getElementsByClassName(htmlids.item_left_cls);
    for(var j=0; j<ooo1.length; j++) {
        ooo1[j].style.backgroundColor = 'unset';
    }
    */

    var item_title = document.getElementById(item_title_id);
    var item_root = document.getElementById(item_root_id);

    if(item_title) {
        item_title.style.backgroundColor = '#D9D9D9';
        item_title.innerHTML = self.label;
    }
    self.focusScroll();
};
ARParticle.prototype.unfocusTimeline = function(uid) {
    var self = this;
    var htmlids = self.htmlids;

    var objuid = uid? uid : self.uid;

    var item_root_id = htmlids.item_root_id;
    var item_title_id = htmlids.item_title_id;

    var item_title = document.getElementById(item_title_id);
    var item_root = document.getElementById(item_root_id);

    if(item_title) {
        item_title.style.backgroundColor = 'unset';
    }
};
ARParticle.prototype.focusScroll = function() {
    var self = this;
    var htmlids = self.htmlids;
    var item_root_id = htmlids.item_root_id;
    var item_root = document.getElementById(item_root_id);

    if(item_root) {
        var topPos = item_root.offsetTop;
        document.getElementById(htmlids.timeline_container_id).scrollTop = topPos;
        //item_root.scrollIntoView();
    }
};
ARParticle.prototype.removeTimeline = function() {
    var self = this;

    self.timeline.remove();
};