function ARScene(ee) {
    var self = this;
    
    self.uid = null; // scene 아이디, workspace에 창조된 오브젝트 uuid로 대치???
    
    self.ee = ee;
    self.selected = false;
    self.label = null;
    self.color = null;
    self.bgImage = null;
    self.bg360Image = null;
    self.effects = [];
}
ARScene.prototype.create = function(data, callback) {
    var self = this;

    self.scene = AREditor.workspace.getScene();

    self.setData(data);
    var objData = self.getData();

    self.scene.userData = objData;

    self.focus();

    self.addExternalEvents();

    return callback(self);
};
ARScene.prototype.setData = function(data) {
    var self = this;
    var uid            = data.uid?         data.uid            : generateUUID();
    var label          = data.label?       data.label          : 'scene';
    var color          = data.color?       data.color          : '#CCCCCC';
    var bgImage        = data.bgImage?     data.bgImage        : null;
    var bg360Image     = data.bg360Image?  data.bg360Image     : null;
    var effects        = data.effects?     data.effects        : [];
    var marker         = data.marker?      data.marker         : {};

    self.setUID(uid);
    self.setMarker(marker);
    self.setLabel(label);
    self.setColor(color);
    self.setBGImage(bgImage);
    self.setBG360Image(bg360Image);
    self.setParticleEffects(effects);
};
ARScene.prototype.setUID = function(data) {
    var self = this;
    self.uid = data;
};
ARScene.prototype.setMarker = function(data) {
    var self = this;
    self.marker = data;

    AREditor.workspace.removeMarker();
    if(self.marker && self.marker.url) {
        AREditor.workspace.setMarker(self.marker);
    }
};
ARScene.prototype.setLabel = function(data) {
    var self = this;
    self.label = data;
};
ARScene.prototype.setColor = function(data) {
    var self = this;
    self.color = data;
    AREditor.workspace.setBackgroundColor(self.color);
};
ARScene.prototype.setBGImage = function(data) {
    var self = this;
    self.bgImage = data;
    if(self.bgImage && self.bgImage.hasOwnProperty('url')) {
        if(self.bgImage.enable) {
            AREditor.workspace.setBackgroundImage(self.bgImage);
        } else {
            AREditor.workspace.unsetBackgroundImage();
        }
    } else {
        AREditor.workspace.unsetBackgroundImage();
    }

    self.setParticleEffects(self.effects);
};
ARScene.prototype.setBG360Image = function(data) {
    var self = this;
    self.bg360Image = data;
    if(self.bg360Image && self.bg360Image.hasOwnProperty('url')) {
        if(self.bg360Image.enable) {
            AREditor.workspace.setBackground360Image(self.bg360Image);
        } else {
            AREditor.workspace.unsetBackground360Image();
        }
    } else {
        AREditor.workspace.unsetBackground360Image();
    }
};
ARScene.prototype.setParticleEffects = function(data) {
    var self = this;
    AREditor.workspace.removeAnimationByType('particle_effect');
    self.effects = data;
    self.effects.forEach(function(item) {
        AREditor.workspace.addParticle(item);
    });
};

ARScene.prototype.getData = function() {
    var self = this;
    var data = {
        uid: self.uid,
        marker: copyObj(self.marker),
        label: self.label,
        color: self.color,
        bgImage: copyObj(self.bgImage),
        bg360Image: copyObj(self.bg360Image),
        effects: self.effects,
    };

    return data;
};
ARScene.prototype.initSettings = function() {
    var self = this;
    var data = self.getData();
    SceneSettings.init(data);
};
ARScene.prototype.focus = function () {
    var self = this;
    self.selected = true;
    self.initSettings();
};
ARScene.prototype.addHistory = function(undoData, redoData) {
    AREditor.history.add({
        undo: function() {
            AREditor.ee.trigger('setting_scene_log', [undoData]);
        },
        redo: function() {
            AREditor.ee.trigger('setting_scene_log', [redoData]);
        }
    });
};
ARScene.prototype.addExternalEvents = function() {
    var self = this;

    self.bind_setting_scene_log = function(data) {
        console.log('setting_scene_log: ', data);
        if(self.uid == data.uid) {
            self.setData(data);
            self.focus();
        } else {
            //
        }
    }.bind(self);
    self.ee.on('setting_scene_log', self.bind_setting_scene_log);

    self.bind_set_scene_marker = function(uid, val) {
        console.log('set_scene_marker: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setMarker(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_scene_marker', self.bind_set_scene_marker);

    self.bind_set_scene_label = function(uid, val) {
        console.log('set_scene_color: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setLabel(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_scene_label', self.bind_set_scene_label);

    self.bind_set_scene_color = function(uid, val) {
        console.log('set_scene_color: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setColor(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_scene_color', self.bind_set_scene_color);

    self.bind_set_scene_bgimage = function(uid, val) {
        console.log('set_scene_bgimage: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setBGImage(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_scene_bgimage', self.bind_set_scene_bgimage);

    self.bind_set_scene_bg360image = function(uid, val) {
        console.log('set_scene_bg360image: ', uid, val);
        if(self.uid == uid) {
            var oldData = copyObj(self.getData());
            self.setBG360Image(val);
            var newData = copyObj(self.getData());
            self.addHistory(oldData, newData);
        }
    }.bind(self);
    self.ee.on('set_scene_bg360image', self.bind_set_scene_bg360image);

};
