/**
 * Requires:
 * jquery, EventEmitter
 * workspace.js, EffectSlider.js, MainSlider.js
 */
var AREditor = {
    workspace: null,
    mainslider: null,
    htmlids: null,
    selectedObject: null,
    selectedEffect: null,
    objects: [],
    lights: [],
    particles: [],
    ee: null,

    isPreview: false,

    init: function() {
        var self = this;
        self.setHtmlIds();

        self.ee = new EventEmitter();
        //self.ee = _.extend({}, Backbone.Events);

        var workspace_container = document.getElementById(self.htmlids.workspace_id);
        self.workspace = new Workspace(self.ee, workspace_container);

        self.mainslider = new MainSlider(self.ee).create();

        self.addExternalEvent();
        self.addKeyboardEvent();

        self.objectDragDrop();
        self.effectDragDrop();

        self.history = new UndoManager();
        self.history.setCallback(function() {
            self.onUndoRedo();
            self.save();
        });

        self.onUndoRedo(); // init undo, redo

        self.setAll();

        self.autoSave();

        self.setForPreview();
    },

    setForPreview: function() {
        var self = this;
        if(self.isPreview === false) return;

        self.showGridHelper();
        self.showAxesHelper();
    },

    onUndoRedo: function() {
        var self = this;
        var isPlaying = self.mainslider.isPlaying;
        var hasUndo = self.history.hasUndo();
        var hasRedo = self.history.hasRedo();
        if(!hasUndo || isPlaying === true) {
            objById('undo').disabled = true;
        } else {
            objById('undo').disabled = false;
        }
        if(!hasRedo || isPlaying === true) {
            objById('redo').disabled = true;
        } else {
            objById('redo').disabled = false;
        }
    },

    setScene: function(data) {
        var self = this;
        var options = data? data : {};
        new ARScene(self.ee).create(options, function(result) {
            self.scene = result;
        });
    },

    setAll: function(val) {
        var self = this;
        var data = val? val : localStorage.getItem('jjb')? localStorage.getItem('jjb') : {};

        if(typeof data === 'string') { data = JSON.parse(data); }
        self.loaded = [];
        //if(data.hasOwnProperty('scenes') && Array.isArray(data.scenes) && data.scenes.length>0) {
        if(data.scene) {
            var sceneData = data.scene;
            self.setScene(sceneData);
            var objects = data.objects;

            objects.forEach(function(item) {
                self.createARObject(item, function(result) {
                    if(!result) {
                        console.warn('ERROR_OBJECT: ', item);
                        self.loaded.push({uid: item.uid, loaded: false});
                    } else {
                        self.loaded.push({uid: item.uid, loaded: true});
                    }
                });
            });
            /**
             * Important!!!
             * Must call code the pertaning to lightobject after creating all object.
             * Because target of lightobject might be either object.
             */
            var lights = data.lights;
            lights.forEach(function(item) {
                self.createARLight(item, function(result) {
                    if(!result) {
                        console.warn('ERROR_LIGHT: ', item);
                        self.loaded.push({uid: item.uid, loaded: false});
                    } else {
                        self.loaded.push({uid: item.uid, loaded: true});
                    }
                });
            });

            var particles = data.particles;
            particles.forEach(function(item) {
                self.createARParticle(item, function(result) {
                    if(!result) {
                        console.warn('ERROR_PARTICLE: ', item);
                        self.loaded.push({uid: item.uid, loaded: false});
                    } else {
                        self.loaded.push({uid: item.uid, loaded: true});
                    }
                });
            });
        } else {
            self.setScene({});
        }
        /*
        var history = JSON.parse(localStorage.getItem('history'));
        if(history) {
            self.history.setCommands(history.commands);
            self.history.setIndex(history.index);
            self.history.runCallback();
        }
        */

        //self.save();

        self.isLoadedAll();
    },

    isLoadedAll: function() {
        var self = this;
        var loaded = self.loaded.filter(function(item) {
            if(item.loaded === true) { return true; }
            return false;
        });
        console.log('Loaded ', loaded.length, ' in ', self.loaded.length);
        if(self.objects.length === loaded.length && self.loaded.length === loaded.length) {
            return true;
        }
        return false;
    },

    setHtmlIds: function() {
        var self = this;
        self.htmlids = {
            workspace_id: 'workspace_container',
            object_container_cls: 'object_container',

            timeline_container_id: 'items',

            item_root_id_pre: 'item_root_',
            item_title_id_pre: 'item_title_',
            item_slider_id_pre: 'item_slider_',
            item_effect_cls: 'item_effect',
            item_left_cls: 'item_left',
            item_right_cls: 'item_right',
            item_cls: 'item',

            effect_widget_cls: 'effectWidget',
            effect_name_id: 'effect_name',
            effect_settings_id: 'effect_settings'
        };
    },

    addExternalEvent: function() {
        var self = this;
        self.ee.on('delete_object_ws', function(objData) { // obj : type of mesh on workspace.
            var conf = getObjConf(objData.type);
            var obj = self.getObjByUID(objData.uid);
            if(obj) {
                if(conf.ctg == 'light') {
                    self.removeLight(obj);
                } else if(conf.ctg == 'particle') {
                    self.removeParticle(obj);
                } else {
                    self.removeObject(obj);
                }
            }
        });

        self.ee.on('unselected_object', function() {
            ObjectSettings.hide();
        });

        self.ee.on('reset_material', function(data) {
            self.removeARObject(data.uid);
            self.createARObject(data, function(result) {
                console.log('reset_material', result);
            });
        });
    },

    addObject: function(data) {
        var self = this;

        self.createARObject(data, function(result) {
            var obj = result;
            if(obj) {
                var uid = obj.uid;
                var data = obj.getData();
                self.history.add({
                    undo: function() {
                        self.removeARObject(uid);
                    },
                    redo: function() {
                        self.createARObject(data, function(result) {

                        });
                    }
                });
            }
        });
    },

    createARObject: function(data, callback) {
        var self = this;
        new ARObject(self.ee).create(data, function(result) {
            var obj = result;
            if(obj) {
                self.objects.push(obj);
            }
            return callback(obj);
        });
    },

    removeObject: function(obj) {
        var self = this;
        //var obj = self.getObjByUID(obj.uuid);
        var data = obj.getData();
        var uid = obj.uid;
        self.removeARObject(uid);

        self.history.add({
            undo: function() {
                self.createARObject(data, function(result) {

                });
            },
            redo: function() {
                self.removeARObject(uid);
            }
        });
    },
    removeARObject: function(uid) {
        var self = this;
        self.objects = self.objects.filter(function(item) {
            if(item.uid == uid){
                item.remove();
                return false;
            }
            else return true;
        });
    },

    objectDragDrop: function() {
        var self = this;

        $('.object_container').draggable({
            revert: true,
            revertDuration: 50,
            containment: 'document',
            appendTo: 'body',
            zIndex: 10000,
            helper: 'clone',
        });

        $('#workspace_container').droppable( {
            accept: '.object_container',
            drop: function(event, ui) {
                var pos = self.workspace.getMousePos3D(event);
                var draggable = ui.draggable;

                var type = draggable.attr('data-type');
                var url = draggable.attr('data-url');

                var data = {};
                if(type) data.type = type;
                if(url) data.resource = {url : url, type: type};
                if(pos) data.position = {x: pos.x, y: pos.y, z: pos.z};

                var conf = getObjConf(type);
                if(conf.ctg == 'light') {
                    self.addLight(data);
                } else if(conf.ctg == 'particle') {
                    self.addParticle(data);
                } else {
                    self.addObject(data);
                }
            }
        });
    },

    effectDragDrop: function() {
        var self = this;

        $('.effectWidget').draggable({
            revert: true,
            revertDuration: 50,
            containment: 'document',
            appendTo: 'body',
            zIndex: 10000,
            helper: 'clone',
        });
    },

    addEffect: function(type) {
        var self = this;
        var selected = self.getSelectedObj();
        if(!selected) {
            console.warn('Please select an object!');
            return;
        }
        var data = {effect_type: type, targetuid: selected.uid};
        self.ee.trigger('create_effect', [data]);
        //selected.createEffect(data);
    },

    getSelectedObj: function() {
        var self = this;
        var selected = null;
        self.objects.forEach(function(item) {
            if(item.selected === true) {
                selected = item;
            }
        });
        if(!selected) {
            self.lights.forEach(function(item) {
                if(item.selected === true) {
                    selected = item;
                }
            });
        }
        if(!selected) {
            self.particles.forEach(function(item) {
                if(item.selected === true) {
                    selected = item;
                }
            });
        }

        return selected;
    },

    getObjByUID: function(uid) {
        var self = this;
        var obj = null;
        self.objects.forEach(function(item) {
            if(item.uid == uid) {
                obj = item;
            }
        });
        if(!obj) {
            self.lights.forEach(function(item) {
                if(item.uid == uid) {
                    obj = item;
                }
            });
        }
        if(!obj) {
            self.particles.forEach(function(item) {
                if(item.uid == uid) {
                    obj = item;
                }
            });
        }
        return obj;
    },

    effectLabelByType: function(type) {
        var self = this;
        var label = '';
        g_effect_type.forEach(function(item) {
            if(item.type == type) label = item.label;
        });
        return label;
    },

    setViewMode: function() {
        var self = this;
        //self.ee.trigger('change_view_mode');
        self.workspace.setViewMode();
        ObjectSettings.hide();
        objById('scene_rotate_state').checked = false;
    },

    addKeyboardEvent: function() {
        var self = this;
        var ctrlkey = false;
        window.addEventListener('keydown', function(event) {
            switch (event.keyCode) {
                case 17:
                    ctrlkey = true;
                    break;
                case 90: // z, undo
                    if(ctrlkey === true) {
                        event.preventDefault();
                        if(self.history.hasUndo() && self.mainslider.isPlaying === false) {
                            self.history.undo();
                        }
                    }
                    break;
                case 89: // y, redo
                    if(ctrlkey === true) {
                        event.preventDefault();
                        if(self.history.hasRedo() && self.mainslider.isPlaying === false) {
                            self.history.redo();
                        }
                    }
                    break;

            }
        }, false);

        window.addEventListener( 'keyup', function ( event ) {
            switch ( event.keyCode ) {
                case 17: // Ctrl
                    ctrlkey = false;
                    break;
            }
        } );
    },

    selectTool: function(type) {
        this.workspace.selectTool(type);
    },

    activeWorkspace: function() {
        document.querySelectorAll('input').forEach(function(item) {
            item.blur();
        });
    },


    showGridHelper: function() {
        var self = this;
        var state = objById('scene_grid_state').checked;
        AREditor.workspace.gridHelper.visible = state;
    },
    showAxesHelper: function() {
        var self = this;
        var state = objById('scene_axes_state').checked;
        AREditor.workspace.axesHelper.visible = state;
    },
    setRotateState: function() {
        var self = this;
        var state = objById('scene_rotate_state').checked;
        AREditor.workspace.orbitControl.autoRotate = state;
    },

    saveAsImage: function() {
        var self = this;
        var imgData, imgNode;
        var strDownloadMime = "image/octet-stream";

        try {
            var strMime = "image/png";
            imgData = self.workspace.glRenderer.domElement.toDataURL(strMime);

            self.saveFile(imgData.replace(strMime, strDownloadMime), "test.png");

        } catch (e) {
            console.log(e);
            return;
        }
    },
    saveFile: function(base64str, filename) {
        var link = document.createElement('a');
        if (typeof link.download === 'string') {
            document.body.appendChild(link); //Firefox requires the link to be in the body
            link.download = filename;
            link.href = base64str;
            link.click();
            document.body.removeChild(link); //remove the link when done
        } else {
            window.location.replace(uri);
        }
    },

    getSaveData: function() {
        var self = this;

        var sceneData = self.scene.getData();
        var objectsData = [];
        var lightsData = [];
        var particlesData = [];
        
        self.objects.forEach(function(item) {
            objectsData.push(item.getData());
        });
        self.lights.forEach(function(item) {
            lightsData.push(item.getData());
        });
        self.particles.forEach(function(item) {
            particlesData.push(item.getData());
        });

        //sceneData.lights = lightsData;
        //sceneData.objects = objectsData;

        var data = {};
        data.appName = 'First my builder';
        data.userID = 'unique user id in db';
        data.publishID = 'unique app id in db';
        //data.marker = {url: 'marker url', id: 'marker id in db'};
        data.scene = sceneData;
        data.objects = objectsData;
        data.lights = lightsData;
        data.particles = particlesData;

        return data;
    },

    publish: function() {
        var self = this;
        var data = self.save();
        //var data = self.getSaveData();
        var preData = jsonBeautify(data);

        var html = '<pre>' + preData + '</pre>';
        var box = new Box({
            contentsType: 'html',
            boxContents: html,
            boxEffect: 'fade',
            isShowCloseBtn: true,
            boxWidth: 500,
            boxHeight: 500,
            isHideByClick: true,
            isHideByDbClick: false
        });
        box.loadBox();
    },

    autoSave: function() {
        var self = this;
        if(self.isPreview === false) {
            self.save();
            setTimeout(function () {
                self.autoSave();
            }, 60000);
        }
    },
    save: function() {
        var self = this;
        var data = self.getSaveData();
        var dataStr = JSON.stringify(data);
        localStorage.setItem('jjb', dataStr);

        var history = {
            commands: self.history.getCommands(),
            index: self.history.getIndex()
        };
        localStorage.setItem('history', JSON.stringify(history));

        console.log('all changes saved', new Date());
        return data;
    },

    preview: function() {
        var self = this;
        self.save();
        var url = './play.html';
        //window.open(url, '_blank');
        var box = new Box({
            contentsType: 'iframe',
            boxContents: url,
            boxEffect: 'fade',
            isShowCloseBtn: false,
            isHideByClick: true});
        box.loadBox();
    },

    addLight: function(data) {
        var self = this;

        self.createARLight(data, function(result) {
            var obj = result;
            if(obj) {
                var uid = obj.uid;
                var data = obj.getData();
                self.history.add({
                    undo: function() {
                        self.removeARLight(uid);
                    },
                    redo: function() {
                        self.createARLight(data, function(result) {

                        });
                    }
                });
            }
        });
    },
    removeLight: function(obj) {
        var self = this;
        //var obj = self.getObjByUID(obj.uuid);
        var data = obj.getData();
        var uid = obj.uid;
        self.removeARLight(uid);

        self.history.add({
            undo: function() {
                self.createARLight(data, function(result) {

                });
            },
            redo: function() {
                self.removeARLight(uid);
            }
        });
    },
    createARLight: function(data, callback) {
        var self = this;
        new ARLight(self.ee).create(data, function(result) {
            var obj = result;
            if(obj) {
                self.lights.push(obj);
            }
            return callback(obj);
        });
    },
    removeARLight: function(uid) {
        var self = this;
        self.lights = self.lights.filter(function(item) {
            if(item.uid == uid){
                item.remove();
                return false;
            }
            else return true;
        });
    },

    addParticle: function(data) {
        var self = this;

        self.createARParticle(data, function(result) {
            var obj = result;
            if(obj) {
                var uid = obj.uid;
                var data = obj.getData();
                self.history.add({
                    undo: function() {
                        self.removeARParticle(uid);
                    },
                    redo: function() {
                        self.createARParticle(data, function(result) {

                        });
                    }
                });
            }
        });
    },
    removeParticle: function(obj) {
        var self = this;
        //var obj = self.getObjByUID(obj.uuid);
        var data = obj.getData();
        var uid = obj.uid;
        self.removeARParticle(uid);

        self.history.add({
            undo: function() {
                self.createARParticle(data, function(result) {

                });
            },
            redo: function() {
                self.removeARParticle(uid);
            }
        });
    },
    createARParticle: function(data, callback) {
        var self = this;
        new ARParticle(self.ee).create(data, function(result) {
            var obj = result;
            if(obj) {
                self.particles.push(obj);
            }
            return callback(obj);
        });
    },
    removeARParticle: function(uid) {
        var self = this;
        self.particles = self.particles.filter(function(item) {
            if(item.uid == uid){
                item.remove();
                return false;
            }
            else return true;
        });
    },
};