var EffectSettings = {
    ani: null,
    ee: null,

    init: function(ani_obj) {
        var self = this;
        self.ani = ani_obj;

        self.setEffType(self.ani.effect_type);
        self.setStart(self.ani.start_time);
        self.setDuration(self.ani.duration);
        self.setLoop(self.ani.loop);
        self.setSelChk(self.ani.sel);
        self.setTween(self.ani.tween_type);
        self.setXYZ();
        self.setXYZChk();
        self.focusSettings();
        self.addEvents();
    },

    setEffType: function(effect_type) {
        var self = this;

        self.effect_type = effect_type;
        objById('effect_name').value = AREditor.effectLabelByType(self.effect_type);
    },

    setStart: function(start) {
        var self = this;

        self.start = Number(start/1000).toFixed(2);
        objById('start_time').value = self.start;
    },

    setDuration: function(duration) {
        var self = this;

        self.duration = Number(duration/1000).toFixed(2);
        objById('duration').value = self.duration;
    },

    setLoop: function(loop) {
        var self = this;

        self.loop = loop;
        objById('loop').checked = self.loop;
    },

    setSelChk: function(sel) {
        var self = this;

        self.sel = sel;
    },

    setTween: function(tween_type) {
        var self = this;

        self.tween_type = tween_type;
        var methods = objById('easing_method');
        for(var i=0; i<methods.options.length; i++) {
            if(methods.options[i].value == self.tween_type) {
                methods.selectedIndex = i;
            }
        }
    },

    setXYZ: function() {
        var self = this;

        var target = null;
        switch (self.effect_type) {
            case 'fadein':
                target = self.ani.target.opacity;
                break;
            case 'fadeout':
                target = self.ani.target.opacity;
                break;
            case 'move':
                target = self.ani.target.position;
                self.setMove(target);
                break;
            case 'rotate':
                target = self.ani.target.rotation;
                self.setRotate(target);
                break;
            case 'scale':
                target = self.ani.target.scale;
                self.setScale(target);
                break;
        }

        return target;
    },

    focusSettings: function() {
        var self = this;

        $('#effect_settings').show();
        $('#effect_setting').show();
        $('#est_indicator')[0].className = 'ett_show';
        $('#effect_widget').hide();
        $('#ett_indicator')[0].className = 'ett_hide';
        $('#effect_name').html(AREditor.effectLabelByType(self.effect_type));

        var obj = objById('axes_setting');
        var type = self.effect_type;

        switch (type) {
            case 'fadein':
                $(obj).hide();
                break;
            case 'fadeout':
                $(obj).hide();
                break;
            case 'scale':
                $(obj).show();
                break;
            case 'rotate':
                $(obj).show();
                break;
            case 'move':
                $(obj).show();
                break;
        }
    },

    setMove: function(target) {
        var self = this;
        //var target = self.targetXYZ();

        objById('x_num').value = self.filterXYZ(target.x);
        objById('y_num').value = self.filterXYZ(target.y);
        objById('z_num').value = self.filterXYZ(target.z);

        objById('x_num').step = "0.1";
        objById('y_num').step = "0.1";
        objById('z_num').step = "0.1";

        objById('x_num').min = -Infinity;
        objById('y_num').min = -Infinity;
        objById('z_num').min = -Infinity;
    },

    setScale: function(target) {
        var self = this;
        //var target = self.targetXYZ();

        objById('x_num').value = self.filterXYZ(target.x);
        objById('y_num').value = self.filterXYZ(target.y);
        objById('z_num').value = self.filterXYZ(target.z);

        objById('x_num').step = "0.1";
        objById('y_num').step = "0.1";
        objById('z_num').step = "0.1";

        objById('x_num').min = "0.1";
        objById('y_num').min = "0.1";
        objById('z_num').min = "0.1";
    },

    setRotate: function(target) {
        var self = this;
        //var target = self.targetXYZ();

        objById('x_num').value = self.filterXYZ(Number(target.x)/Math.PI * 180);
        objById('y_num').value = self.filterXYZ(Number(target.y)/Math.PI * 180);
        objById('z_num').value = self.filterXYZ(Number(target.z)/Math.PI * 180);

        objById('x_num').step = "1";
        objById('y_num').step = "1";
        objById('z_num').step = "1";

        objById('x_num').min = -Infinity;
        objById('y_num').min = -Infinity;
        objById('z_num').min = -Infinity;
    },

    setXYZChk: function() {
        var self = this;

        if(self.sel.x === false) {
            objById('x_chk').checked = false;
            objById('x_num').disabled = true;
        } else {
            objById('x_chk').checked = true;
            objById('x_num').disabled = false;
        }
        if(self.sel.y === false) {
            objById('y_chk').checked = false;
            objById('y_num').disabled = true;
        } else {
            objById('y_chk').checked = true;
            objById('y_num').disabled = false;
        }
        if(self.sel.z === false) {
            objById('z_chk').checked = false;
            objById('z_num').disabled = true;
        } else {
            objById('z_chk').checked = true;
            objById('z_num').disabled = false;
        }
    },

    hideSettings: function() {
        var self = this;

        $('#effect_settings').hide();
        $('#est_indicator')[0].className = 'ett_hide';
    },

    addEvents: function() {
        var self = this;

        objById('start_time').onchange = function(e) {
            //if(e.keyCode == 13) {
                var value = Number(e.target.value);
                if(value<0) value = 0;
                e.target.value = value.toFixed(2);
                AREditor.ee.trigger('setting_start', [self.ani, value]);
            //}
        };
        objById('duration').onchange = function(e) {
            //if(e.keyCode == 13) {
                var value = Number(e.target.value);
                if(value<0) value = 0;
                e.target.value = value.toFixed(2);
                AREditor.ee.trigger('setting_duration', [self.ani, value]);
            //}
        };
        objById('loop').onchange = function(e) {
            var loop = e.target.checked? true : false;
            AREditor.ee.trigger('setting_loop', [self.ani, loop]);
        };
        objById('easing_method').onchange = function(e) {
            var method = e.target.value;
            AREditor.ee.trigger('setting_easing', [self.ani, method]);
        };
        objById('x_chk').onchange = function(e) {
            var chk = e.target.checked? true : false;
            objById('x_num').disabled = chk? false : true;
            AREditor.ee.trigger('setting_x_chk', [self.ani, chk]);
        };
        objById('y_chk').onchange = function(e) {
            var chk = e.target.checked? true : false;
            objById('y_num').disabled = chk? false : true;
            AREditor.ee.trigger('setting_y_chk', [self.ani, chk]);
        };
        objById('z_chk').onchange = function(e) {
            var chk = e.target.checked? true : false;
            objById('z_num').disabled = chk? false : true;
            AREditor.ee.trigger('setting_z_chk', [self.ani, chk]);
        };
        objById('x_num').onchange = function(e) {
            if(e.target.disabled) return;
            //if(e.keyCode == 13) {
                var value = self.filterXYZ(e.target.value);
                e.target.value = value;
                AREditor.ee.trigger('setting_x', [self.ani, self.xyz()]);
            //}
        };
        objById('y_num').onchange = function(e) {
            if(e.target.disabled) return;
            //if(e.keyCode == 13) {
                var value = self.filterXYZ(e.target.value);
                e.target.value = value;
                AREditor.ee.trigger('setting_y', [self.ani, self.xyz()]);
            //}
        };
        objById('z_num').onchange = function(e) {
            if(e.target.disabled) return;
            //if(e.keyCode == 13) {
                var value = self.filterXYZ(e.target.value);
                e.target.value = value;
                AREditor.ee.trigger('setting_z', [self.ani, self.xyz()]);
            //}
        };
    },

    filterXYZ: function(val) {
        var self = this;
        var effect_type = self.ani.effect_type;
        var value = Number(val);
        switch (effect_type) {
            case 'scale':
                value = value<=0? 0.1 : value;
                break;
            case 'rotate':
                value = parseInt(value);
                break;
            case 'move':
                value = value;
                break;
        }
        return value;
    },

    xyz: function() {
        var self = this;
        var effect_type = self.ani.effect_type;
        var x_num = self.filterXYZ(objById('x_num').value);
        var y_num = self.filterXYZ(objById('y_num').value);
        var z_num = self.filterXYZ(objById('z_num').value);
        var obj = {};

        switch (effect_type) {
            case 'scale':
                obj = {
                    x: x_num,
                    y: y_num,
                    z: z_num
                };
                break;
            case 'rotate':
                obj = {
                    x: x_num/180 * Math.PI,
                    y: y_num/180 * Math.PI,
                    z: z_num/180 * Math.PI
                };
                break;
            case 'move':
                obj = {
                    x: x_num,
                    y: y_num,
                    z: z_num
                };
                break;
        }
        return obj;
    },

};