function EffectSlider(ee, ar) {
    var self = this;
    self.ee = ee;
    self.ar = ar;
    self.uid = null; // Note: effect slider uid
    self.targetuid = null; // ARObject uid = parent uid
    self.color = null;
    self.effect_range = null;
    self.effect_type = null;
    self.step = 0.1;

    self.container = null;
    self.loop = false;

    self.slider = null;

    self.htmlids = null;

    self.isSelected = false;

    return self;
}
EffectSlider.prototype.create = function(data, insertTarget) {
    var self = this;
    self.color = data.color;
    self.uid = data.uid;
    self.effect_range = {
        min: Number(data.effect_range.min).toFixed(2),
        max: Number(data.effect_range.max).toFixed(2)
    };
    self.effect_type = data.effect_type;

    self.targetuid = data.targetuid;
    self.loop = data.loop;

    self.setHtmlIds();

    self.createHTML(insertTarget);
    self.createSlider();

    self.addSomeEvent();

    self.addExternalEvent();

    var effect_root_id = self.htmlids.effect_root_id;
    $('#'+effect_root_id).hide().fadeIn();

    self.focusSlider();

    self.ee.trigger('created_effect_slider', [self.getData()]);

    return self;
};
EffectSlider.prototype.setData = function(data) {
    var self = this;

    self.color = data.color;
    self.uid = data.uid;
    self.effect_range = {
        min: Number(data.effect_range.min).toFixed(2),
        max: Number(data.effect_range.max).toFixed(2)
    };
    self.effect_type = data.effect_type;

    self.targetuid = data.targetuid;
    self.loop = data.loop;
};
EffectSlider.prototype.getData = function() {
    var self = this;
    var data = {
        uid: self.uid,
        targetuid: self.targetuid,
        effect_range: self.effect_range,
        effect_type: self.effect_type,
        loop: self.loop,
        color: self.color,
    };
    return data;
};

EffectSlider.prototype.addExternalEvent = function() {
    var self = this;

    self.bind_update_main_range = function() {
        self.updateSlider(AREditor.mainslider.getRange());
    }.bind(self);
    self.ee.on('update_main_range', self.bind_update_main_range);
};
EffectSlider.prototype.removeExternalEvent = function() {
    var self = this;
    self.ee.off('update_main_range',    self.bind_update_main_range);
};

EffectSlider.prototype.objectUID = function(obj) {
    var self = this;
    if(obj) {
        return obj.userData.uid;
    }

    return self.targetuid;
};

EffectSlider.prototype.setHtmlIds = function() {
    var self = this;
    var uid = self.uid;
    var targetuid = self.targetuid;

    var effect_root_id      = 'effect_root_' + uid;
    var effect_type_id      = 'effect_type_' + uid;
    var effect_range_id     = 'effect_range_' + uid;
    var effect_drop_id      = 'effect_drop_' + uid;
    var effect_slider_id    = 'effect_slider_' + uid;
    var effect_loop_id      = 'effect_loop_' + uid;

    var item_root_id    = 'item_root_' + targetuid;
    var item_title_id   = 'item_title_' + targetuid;
    var item_slider_id  = 'item_slider_' + targetuid;

    var main_slider_id = 'main_slider';

    var obj = {
        effect_root_id: effect_root_id,
        effect_type_id: effect_type_id,
        effect_range_id: effect_range_id,
        effect_drop_id: effect_drop_id,
        effect_loop_id: effect_loop_id,
        effect_slider_id: effect_slider_id,

        main_slider_id: main_slider_id,

        item_root_id: item_root_id,
        item_title_id: item_title_id,
        item_slider_id: item_slider_id
    };
    return self.htmlids = obj;
};
EffectSlider.prototype.createHTML = function(insertTarget) {
    var self = this;
    var uid = self.uid;
    var effect_type = self.effect_type;
    var color = self.color;
    var htmlids = self.htmlids;

    var item_slider_id      = htmlids.item_slider_id;
    var effect_root_id      = htmlids.effect_root_id;
    var effect_type_id      = htmlids.effect_type_id;
    var effect_range_id     = htmlids.effect_range_id;
    var effect_slider_id    = htmlids.effect_slider_id;
    var effect_drop_id      = htmlids.effect_drop_id;
    var effect_loop_id      = htmlids.effect_loop_id;
    
    var root = document.createElement('div');
    root.id = effect_root_id;
    root.className = 'item_effect';
    root.innerHTML = "<div class='item_effect_info'>"
                        +"<div class='item_effect_info_set' style='background-color: "+self.color+";'>"
                            +"<span id='"+effect_type_id+"' class='item_effect_type item_text_shadow'>"+effect_type+"</span>"
                            +"<img id='"+effect_loop_id+"' class='item_effect_loop'>"
                            +"<img id='"+effect_drop_id+"' class='item_effect_drop'>"
                        +"</div>"
                        +"<div>"
                            +"<span id='"+effect_range_id+"' class='item_effect_range item_text_shadow' style='background-color: "+self.color+";'></span>"
                        +"</div>"                             
                    +"</div>"
                    +"<div id='"+effect_slider_id+"'></div>";

    if(!insertTarget || insertTarget.target == 'self') {
        document.getElementById(item_slider_id).appendChild(root);
    } else {
        var insert_targetid = 'effect_root_' + insertTarget.uid;
        if(insertTarget.target == 'after') {
            $(root).insertAfter('#'+insert_targetid);
        }
        if(insertTarget.target == 'before') {
            $(root).insertBefore('#'+insert_targetid);
        }
    }

    self.refreshLoopIcon();
};
EffectSlider.prototype.refreshLoopIcon = function() {
    var self = this;
    var htmlids = self.htmlids;
    var effect_loop_id = htmlids.effect_loop_id;
    if(self.loop === true) {
        document.getElementById(effect_loop_id).style.display = 'inline-block';
    } else {
        document.getElementById(effect_loop_id).style.display = 'none';
    }
};
EffectSlider.prototype.remove = function() {
    var self = this;
    var uid = self.uid;
    var htmlids = self.htmlids;
    var item_slider_id      = htmlids.item_slider_id;
    var effect_root_id      = htmlids.effect_root_id;
    
    //if(!window.confirm('Are you sure to remove the effect?')) return;

    self.unfocusSlider();

    self.removeExternalEvent();

    self.slider.noUiSlider.destroy();

    document.getElementById(effect_root_id).remove();

    //return;
    Object.keys(self).forEach(function (key) {
        delete self[key];
    });
    self.__proto__ = {};
};
EffectSlider.prototype.updateSlider = function(range) {
    var self = this;
    var slider = self.slider;
    
    slider.noUiSlider.destroy();

    self.createSlider(range);
    return self;
};
EffectSlider.prototype.onSetSlider = function(e) {
    var self = this;
    if(e.length == 1) {
        self.effect_range = {min: Number(e[0]), max: self.effect_range.max};
    }
    if(e.length == 2) {
        self.effect_range = {min: Number(e[0]), max: Number(e[1])};
        if(Number(e[1]) - Number(e[0]) < 0.1) {
            self.effect_range.max = Number(self.effect_range.min + 0.1).toFixed(2);
            self.slider.noUiSlider.setHandle(1, self.effect_range.max, false);
        }
    }
    self.refreshEffectRange();

    self.focusSlider();
    self.ee.trigger('change_effect_range', [self.uid, self.targetuid, self.effect_range]);
    self.ee.trigger('select_effect_slider', [self.targetuid, self.uid]);
};
EffectSlider.prototype.createSlider = function(range) {
    var self = this;
    var uid = self.uid;
    var color = self.color;
    var effect_range = self.effect_range;

    var main_range = null;
    if(range) {
        main_range = range;
    } else {
        var main_slider = self.getMainSlider();
        if(!main_slider) {
            console.error('Cannot find main slider.');
            return;
        }

        var main_options = main_slider.noUiSlider.options;
        main_range = main_options.range;
    }

    
    var start = null;
    var connect = null;
    var disable = false;

    if(effect_range.max > main_range.max) {
        if(effect_range.min > main_range.max) {
            disable = true;
        }
        start = [effect_range.min];
        connect = [false, true];
    } else {
        start = [effect_range.min, effect_range.max];
        connect = [false, true, false];
    }                        
    
    var slider_id = self.htmlids.effect_slider_id;
    var slider = document.getElementById(slider_id);
    noUiSlider.create(slider, {
        range: main_range,
        step: self.step,
        start: start,
        animate: true,
        // Move handle on tap, bars are draggable
        behaviour: 'tap-drag',
        tooltips: disable? false : true,
        connect: connect,
    });

    slider.onclick = function(e) {
        console.log('clicked: ', slider);
    };

    
    slider.noUiSlider.on('set', function(e) {
        var effect_old = copyObj(self.getData());
        self.onSetSlider(e);
        var effect_new = copyObj(self.getData());

        if(effect_new.effect_range.min == effect_old.effect_range.min && effect_new.effect_range.max == effect_old.effect_range.max) {
            return;
        }

        AREditor.history.add({
            undo: function() {
                AREditor.ee.trigger('setting_effect_log', [effect_old]);
            },
            redo: function() {
                AREditor.ee.trigger('setting_effect_log', [effect_new]);
            }
        });
    });
    
    if(disable === true) {               
        slider.setAttribute('disabled', disable);
        $('#'+slider_id+' .noUi-handle').css('display', 'none');
    } else {
        slider.removeAttribute('disabled');
        $('#'+slider_id+' .noUi-handle').css('display', 'block');
    }

    $('#'+slider_id+' .noUi-tooltip').css({
        'bottom': '0',
        'padding': '0'
    });
    $('#'+slider_id+' .noUi-handle').css('height', '18px');
    $('#'+slider_id).css('height', '10px');
    $('#'+slider_id+' .noUi-connect').css('background', color);

    self.slider = slider;

    self.refreshEffectRange();

    self.addSomeEvent();
};
EffectSlider.prototype.addSomeEvent = function() {
    var self = this;
    self.addClickEffectRangeInfo();
    self.addEventEffectRoot();
    self.addDropEffect();
};
EffectSlider.prototype.addEventEffectRoot = function() {
    var self = this;
    var uid = self.uid;
    var effect_root_id = self.htmlids.effect_root_id;
    var obj = document.getElementById(effect_root_id);
    obj.onmouseover = function(e) {
        return;
        obj.style.borderBottom = 'solid 2px #3FB8AF';
        obj.style.borderTop = 'solid 2px #3FB8AF';
    };
    obj.onmouseout = function(e) {
        return;
        obj.style.border = 'none';
    };
    obj.onclick = function(e) {
        self.focusSlider();
        self.ee.trigger('select_effect_slider', [self.targetuid, self.uid]);
    };
};
EffectSlider.prototype.unfocusSlider = function() {
    var self = this;
    var htmlids = self.htmlids;
    var effect_root_id = htmlids.effect_root_id;
    var effect_root = document.getElementById(effect_root_id);

    if(effect_root) {
        effect_root.style.backgroundColor = 'unset';
    }

    self.isSelected = false;
};
EffectSlider.prototype.focusSlider = function() {
    var self = this;
    var htmlids = self.htmlids;

    var effect_root_id = htmlids.effect_root_id;

    var ooo = document.getElementsByClassName('item_effect');
    for(var i=0; i<ooo.length; i++) {
        ooo[i].style.backgroundColor = 'unset';
    }

    var effect_root = document.getElementById(effect_root_id);
    if(effect_root) {
        effect_root.style.backgroundColor = '#D9D9D9';
        var topPos = effect_root.offsetTop;
        effect_root.scrollTop = topPos;
        effect_root.scrollIntoView();
    }

    self.isSelected = true;
};
EffectSlider.prototype.addClickEffectRangeInfo = function() {
    var self = this;
    var effect_range_id  = self.htmlids.effect_range_id;
    var obj = document.getElementById(effect_range_id);
    obj.onclick = function() {
        self.ee.trigger('clicked_effect_range', [self.getData()]);
    };
};
EffectSlider.prototype.addDropEffect = function() {
    var self = this;
    var effect_drop_id = self.htmlids.effect_drop_id;
    document.getElementById(effect_drop_id).onclick = function(e) {
        e.stopPropagation();
        self.ee.trigger('remove_effect', [self.getData()]);
    };
};
EffectSlider.prototype.refreshEffectRange = function() {
    var self = this;
    var slider = self.slider;
    var options = slider.noUiSlider.options;
    var uid = self.uid;
    var range = self.effect_range;
    var ele = self.htmlids.effect_range_id;
    var html = range.min + ' - ' + range.max;
    document.getElementById(ele).innerHTML = html;
};
EffectSlider.prototype.getMainSlider = function() {
    var obj = AREditor.mainslider.getSlider();
    if(obj && obj.noUiSlider) {
        return obj;
    }
    return null;
};
EffectSlider.prototype.getMainPos = function() {
    var pos = AREditor.mainslider.getPos();
    return pos;
};