var LightSettings = {
    data: null,

    init: function(data) {
        var self = this;
        self.data = data;

        self.setUID();
        self.setType();
        self.setLabel();
        self.setPosition();
        self.setColor();
        self.setVisible();
        self.setIntensity();
        /*
        self.setShadow();*/
        self.setDistance();
        self.setAngle();
        self.setGroundColor();

        self.addEvents();

        self.show();

        self.initShow();
    },

    show: function() {
        $('.settings_root2').show();
        $('.settings_root3').hide();
        $('.settings_root1').hide();
        $('.settings_root0').hide();
    },
    hide: function() {
        $('.settings_root3').hide();
        $('.settings_root2').hide();
        $('.settings_root1').hide();
        $('.settings_root0').show();
    },

    initShow: function() {
        var self = this;
        var conf = getObjConf(self.type);

        $('#light_distance_root').hide();
        $('#light_groundcolor_root').hide();
        $('#light_angle_root').hide();
        $('#light_shadow_root').hide();

        if(conf.type == 'directional') {
            //$('#light_shadow_root').show();
        }
        if(conf.type == 'point') {
            //$('#light_shadow_root').show();
            $('#light_distance_root').show();
        }
        if(conf.type == 'spot') {
            //$('#light_shadow_root').show();
            $('#light_distance_root').show();
            $('#light_angle_root').show();
        }
        if(conf.type == 'hemisphere') {
            $('#light_groundcolor_root').show();
        }
    },

    setUID: function(val) {
        var self = this;
        var uid = val? val : self.data.uid;
        self.uid = uid;
    },
    setVisible: function(val) {
        var self = this;
        var visible = val? val : self.data.visible;
        self.visible = visible;
        objById('light_visible').checked = self.visible;
    },

    setType: function(val) {
        var self = this;
        var type = val? val : self.data.type;
        self.type = type;
    },

    setLabel: function(val) {
        var self = this;
        var label = val? val : self.data.label;
        self.label = label;

        objById('light_label').value = self.label;
    },

    setColor: function(val) {
        var self = this;
        var color = val? val : self.data.color;
        self.color = color;

        if(objById('light_color').jscolor) {
            objById('light_color').value = self.color;
            objById('light_color').jscolor.fromString(self.color);
        } else {
            setTimeout(function() {
                self.setColor(val);
            }, 500);
        }
    },

    setGroundColor: function(val) {
        var self = this;
        var color = val? val : self.data.groundColor;
        self.groundColor = color;

        if(objById('light_groundcolor').jscolor) {
            objById('light_groundcolor').value = self.groundColor;
            objById('light_groundcolor').jscolor.fromString(self.groundColor);
        } else {
            setTimeout(function() {
                self.setColor(val);
            }, 500);
        }
    },

    setPosition: function(val) {
        var self = this;
        var position = val? val : self.data.position;
        self.position = position;

        objById('light_position_x').value = Number(self.position.x);
        objById('light_position_y').value = Number(self.position.y);
        objById('light_position_z').value = Number(self.position.z);
    },

    setIntensity: function(val) {
        var self = this;
        var intensity = typeof val === 'undefined'? self.data.intensity : val;
        self.intensity = Number(intensity) < 0 ? 0 : Number(intensity);

        objById('light_intensity').value = self.intensity;
    },

    setDistance: function(val) {
        var self = this;
        var data = typeof val === 'undefined'? self.data.distance : val;
        self.distance = Number(data) < 0 ? 0 : Number(data);

        objById('light_distance').value = self.distance;
    },

    setAngle: function(val) {
        var self = this;
        var data = val? val : self.data.angle;
        self.angle = data>Math.PI/2? Math.PI/2 : (data<0? 0 : data);
        var t = Number(self.angle/Math.PI * 180);

        objById('light_angle').value = t;
    },

    addEvents: function() {
        var self = this;

        objById('delete_light_btn').onclick = self.onDeleteObject.bind(self);

        objById('light_label').onchange = self.onUpdateLabel.bind(self);

        objById('light_color').onchange = self.onUpdateColor.bind(self);

        objById('light_position_x').onchange = self.onUpdatePosition.bind(self);
        objById('light_position_y').onchange = self.onUpdatePosition.bind(self);
        objById('light_position_z').onchange = self.onUpdatePosition.bind(self);

        objById('light_visible').onchange = self.onUpdateVisible.bind(self);

        objById('light_intensity').onchange = self.onUpdateIntensity.bind(self);

        objById('light_distance').onchange = self.onUpdateDistance.bind(self);
        objById('light_groundcolor').onchange = self.onUpdateGroundColor.bind(self);
        objById('light_angle').onchange = self.onUpdateAngle.bind(self);
    },

    onDeleteObject: function(e ) {
        var self = this;
        AREditor.ee.trigger('delete_object_ws', [self.data]);
    },

    onUpdateLabel: function(e) {
        var self = this;
        var label = objById('light_label').value;
        if(label != self.label) {
            self.label = label;
            AREditor.ee.trigger('set_label', [self.uid, self.label]);
        }
    },

    onUpdateColor: function(e) {
        var self = this;
        var color = objById('light_color').jscolor.toHEXString();
        objById('light_color').value = color;
        if(color != self.color) {
            self.color = color;
            AREditor.ee.trigger('set_color', [self.uid, self.color]);
        }
    },
    onUpdateGroundColor: function(e) {
        var self = this;
        var color = objById('light_groundcolor').jscolor.toHEXString();
        objById('light_groundcolor').value = color;
        if(color != self.groundColor) {
            self.groundColor = color;
            AREditor.ee.trigger('set_groundColor', [self.uid, self.groundColor]);
        }
    },

    onUpdatePosition: function(e) {
        var self = this;
        var x = Number(objById('light_position_x').value);
        var y = Number(objById('light_position_y').value);
        var z = Number(objById('light_position_z').value);

        if(self.position.x != x || self.position.y != y || self.position.z != z) {
            self.position.x = x;
            self.position.y = y;
            self.position.z = z;
            AREditor.ee.trigger('set_position', [self.uid, self.position]);
        }
    },

    onUpdateVisible: function(e) {
        var self = this;
        self.visible = e.target.checked;

        AREditor.ee.trigger('set_visible', [self.uid, self.visible]);
    },

    onUpdateIntensity: function(e) {
        var self = this;
        var t = Number(e.target.value);
        self.intensity = t < 0? 0 : t;
        objById('light_intensity').value = self.intensity;
        AREditor.ee.trigger('set_intensity', [self.uid, self.intensity]);
    },

    onUpdateDistance: function(e) {
        var self = this;
        var t = Number(e.target.value);
        self.distance = t < 0? 0 : t;
        objById('light_distance').value = self.distance;
        AREditor.ee.trigger('set_distance', [self.uid, self.distance]);
    },

    onUpdateAngle: function(e) {
        var self = this;
        var t = Number(e.target.value);
        objById('light_angle').value = t < 0? 0 : (t>90? 90 : t);
        self.angle = Number(objById('light_angle').value)/180 * Math.PI;
        AREditor.ee.trigger('set_angle', [self.uid, self.angle]);
    },
};