function MainSlider(ee) {
    var self = this;
    self.ee = ee;
    self.htmlids = null;
    self.slider = null;
    self.isPlaying = false;
    self.timer = null;
    self.seekpos = 0;

    self.setHtmlIds();

    self.addExternalEvent();

    return self;            
}

MainSlider.prototype.addExternalEvent = function() {
    var self = this;
    self.ee.on('clicked_effect_range', function(slider) {
        var main_slider = self.getSlider();
        var effect_range = slider.effect_range;
        var max = Number(effect_range.max);
        var main_max = Math.floor(max) * 2;
        main_max = main_max - main_max%10;

        main_slider.noUiSlider.updateOptions({
            range: {
                'min': 0,
                'max': main_max > 10 ? main_max : 10
            }                 
        }, false);
        main_slider.noUiSlider.setHandle(0, effect_range.min, false);

        self.refreshEffectSliders();
    });

    self.ee.on('get_main_pos', function(callback) {
        var pos = self.getPos();
        return callback(pos);
    });
};

MainSlider.prototype.setHtmlIds = function() {
    var self = this;
    self.htmlids = {
        root_id: 'main_slider_root',
        info_id: 'main_slider_info',
        timenow_id: 'main_slider_timenow',
        container_id: 'main_slider_container',
        slider_id: 'main_slider',
        locator_id: 'main_slider_locator'
    };
};
MainSlider.prototype.create = function() {
    var self = this;
    var htmlids = self.htmlids;
    var slider_id = htmlids.slider_id;
    
    self.slider = document.getElementById(slider_id);
    noUiSlider.create(self.slider, {

        range: {
            'min': 0,
            'max': 10
        },

        step: 0.1,

        start: 0,

        animate: true,
        animationDuration: 0,

        keyboardSupport: false,

        // Move handle on tap, bars are draggable
        behaviour: 'tap-drag',
        tooltips: true,
        
        connect: [true, false],

        // Show a scale with the slider
        pips: {
            mode: 'positions',
            values: [0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
            stepped: true,
            //density: 4
        }
    });

    self.slider.noUiSlider.on('update', function(e) {
        self.updateLocator();

        self.updateTimenow(e[0]);


        self.ee.trigger('update_seek', [self.getPos()]);
    });
    self.slider.noUiSlider.on('change', function(e) {
        self.updateRange();
    });
    self.slider.noUiSlider.on('set', function(e) {
        self.updateRange();
    });
    self.slider.noUiSlider.on('slide', function(e) {
        self.ee.trigger('slide_seek', [self.getPos()]);
    });
    self.slider.noUiSlider.on('start', function(e) {
        self.pause1();
    });
    self.slider.noUiSlider.on('end', function(e) {
        if(self.isPlaying === true) {
            self.play();
        }
    });

    self.addSomeEvent();

    return self;
};
MainSlider.prototype.getPos = function() {
    var self = this;
    var slider = self.getSlider();
    return Number(slider.noUiSlider.get());
};
MainSlider.prototype.addSomeEvent = function() {
    var self = this;
    self.addMouseWheel();
};
MainSlider.prototype.getSlider = function() {
    var self = this;
    return self.slider;
};
MainSlider.prototype.updateLocator = function() {
    var self = this;
    var locator_id = self.htmlids.locator_id;
    var slider = self.getSlider();
    var max = slider.noUiSlider.options.range.max;
    var pos = slider.noUiSlider.get();
    var per = (pos/max)*100;
    document.getElementById(locator_id).style.left = per + '%';
};
MainSlider.prototype.updateTimenow = function(pos) {
    var self = this;
    var timenow_id = self.htmlids.timenow_id;
    document.getElementById(timenow_id).innerHTML = seconds2hms(pos).hms;
};
MainSlider.prototype.updateRange = function() {
    var self = this;

    var slider = self.getSlider();
    var max = slider.noUiSlider.options.range.max;
    var pos = slider.noUiSlider.get();
    var per = (pos/max)*100;

    if(per >= 90) {
        max = (max - max%10)*2;
        slider.noUiSlider.updateOptions({
            range: {
                'min': 0,
                'max': max
            },
            start: pos
        }, false);

        self.refreshEffectSliders();
    }
    if(per <= 10 && max > 10) {       
        max = max/2;
        max = max - max%10;
        slider.noUiSlider.updateOptions({
            range: {
                'min': 0,
                'max': max
            },
            start: pos
        }, false);

        self.refreshEffectSliders();
    }
};

MainSlider.prototype.refreshEffectSliders = function() {
    var self = this;
    self.ee.trigger('update_main_range');
};

MainSlider.prototype.historyControlState = function() {
    var self = this;
    if(self.isPlaying) {
        objById('undo').disabled = true;
        objById('redo').disabled = true;
    } else {
        objById('undo').disabled = false;
        objById('redo').disabled = false;
    }
};
MainSlider.prototype.play = function() {
    var self = this;
    if(self.timer) {
        return;
    }
    self.isPlaying = true;
    self.historyControlState();
    self.ee.trigger('play_seek', [self.getPos()]);
    self.timer = setInterval(function() { self.seek(); }, 100);
};
MainSlider.prototype.seek = function() {
    var self = this;
    var slider = self.getSlider();
    var pos = slider.noUiSlider.get();
    self.seekpos = Number(Number(pos) + 0.1).toFixed(2);
    slider.noUiSlider.set(self.seekpos);
};
MainSlider.prototype.stop = function() {
    var self = this;
    clearInterval(self.timer);
    self.timer = null;
    self.isPlaying = false;
    self.historyControlState();
    self.seekpos = 0;
    var slider = self.getSlider();
    slider.noUiSlider.set(self.seekpos);
};
MainSlider.prototype.pause = function() {
    var self = this;
    clearInterval(self.timer);
    self.timer = null;
    self.isPlaying = false;
    self.historyControlState();
};
MainSlider.prototype.pause1 = function() {
    var self = this;
    clearInterval(self.timer);
    self.timer = null;
};
MainSlider.prototype.addMouseWheel = function() {
    var self = this;
    var container_id = self.htmlids.container_id;
    document.getElementById(container_id).onmousewheel = function(e) {
        self.zoom(e);
    };
};
MainSlider.prototype.zoom = function(e) {
    var self = this;
    if(e.deltaY > 0) {
        self.updateRangePlus();
    } else {
        self.updateRangeMinus();
    }
};
MainSlider.prototype.updateRangePlus = function() {
    var self = this;
    var slider = self.getSlider();
    var range = slider.noUiSlider.options.range;
    max = Number(range.max) + 10;
    slider.noUiSlider.updateOptions({
        range: {
            'min': 0,
            'max': max
        }
    }, false);

    self.refreshEffectSliders();
};
MainSlider.prototype.updateRangeMinus = function() {
    var self = this;
    var slider = self.getSlider();
    var range = slider.noUiSlider.options.range;
    max = Number(range.max) - 10;
    slider.noUiSlider.updateOptions({
        range: {
            'min': 0,
            'max': max > 10? max : 10
        }
    }, false);

    self.refreshEffectSliders();
};
MainSlider.prototype.getRange = function() {
    var self = this;
    var slider = self.getSlider();
    var range = slider.noUiSlider.options.range;
    return range;
};