var ObjectSettings = {
    data: null,

    init: function(data) {
        var self = this;
        self.data = data;

        self.setUID();
        self.setType();
        self.setLabel();
        self.setColor();
        self.setOpacity();
        self.setScale();
        self.setPosition();
        self.setRotation();

        self.setTexture();
        self.setResource();
        self.setLink();

        self.setShadow();

        self.setVisible();

        self.setMaterial();

        self.addEvents();

        self.show();
    },

    show: function() {
        $('.settings_root3').hide();
        $('.settings_root2').hide();
        $('.settings_root1').show();
        $('.settings_root0').hide();
    },
    hide: function() {
        $('.settings_root3').hide();
        $('.settings_root2').hide();
        $('.settings_root1').hide();
        $('.settings_root0').show();
    },

    setUID: function(val) {
        var self = this;
        var uid = val? val : self.data.uid;
        self.uid = uid;
    },
    setVisible: function(val) {
        var self = this;
        var visible = val? val : self.data.visible;
        self.visible = visible;
        objById('visible').checked = self.visible;
    },
    setShadow: function(val) {
        var self = this;
        var shadow = val? val : self.data.shadow;
        self.shadow = shadow;
        if(self.shadow && self.shadow.hasOwnProperty('cast')) {
            objById('shadow_cast').checked = self.shadow.cast;
        } else {
            objById('shadow_cast').checked = false;
        }

        if(self.shadow && self.shadow.hasOwnProperty('receive')) {
            objById('shadow_receive').checked = self.shadow.receive;
        } else {
            objById('shadow_receive').checked = false;
        }

    },
    setMaterial: function(val) {
        var self = this;
        var conf = getObjConf(self.type);
        if(conf.ctg == 'model') {
            $('#material_root').hide();
            return;
        } else {
            $('#material_root').show();
        }
        var material = val? val : self.data.material;
        self.material = material;
        var materials = objById('material');
        for(var i=0; i<materials.options.length; i++) {
            if(materials.options[i].value == self.material) {
                materials.selectedIndex = i;
            }
        }
    },

    setType: function(val) {
        var self = this;
        var type = val? val : self.data.type;
        self.type = type;
    },

    setLabel: function(val) {
        var self = this;
        var label = val? val : self.data.label;
        self.label = label;

        objById('label').value = self.label;
    },

    setColor: function(val) {
        var self = this;
        var color = val? val : self.data.color;
        self.color = color;

        if(objById('color').jscolor) {
            objById('color').value = self.color;
            objById('color').jscolor.fromString(self.color);
        } else {
            setTimeout(function() {
                self.setColor(val);
            }, 500);
        }
    },

    setOpacity: function(val) {
        var self = this;
        var opacity = typeof val === 'undefined'? self.data.opacity : val;
        self.opacity = Number(opacity);

        objById('opacity').value = self.opacity*100;
    },

    setScale: function(val) {
        var self = this;
        var scale = val? val : self.data.scale;
        self.scale = scale;

        objById('scale_x').value = Number(self.scale.x);
        objById('scale_y').value = Number(self.scale.y);
        objById('scale_z').value = Number(self.scale.z);
    },

    setPosition: function(val) {
        var self = this;
        var position = val? val : self.data.position;
        self.position = position;

        objById('position_x').value = Number(self.position.x);
        objById('position_y').value = Number(self.position.y);
        objById('position_z').value = Number(self.position.z);
    },

    setRotation: function(val) {
        var self = this;
        var rotation = val? val : self.data.rotation;
        self.rotation = rotation;

        objById('rotation_x').value = Number(self.rotation.x/Math.PI * 180);
        objById('rotation_y').value = Number(self.rotation.y/Math.PI * 180);
        objById('rotation_z').value = Number(self.rotation.z/Math.PI * 180);
    },

    setTexture: function(val) {
        var self = this;
        var texture = val? val : self.data.texture;
        self.texture = copyObj(texture);
        var objConf = getObjConf(self.data.type);
        if(objConf.ctg == 'element' || objConf.ctg == 'model') {
            $('#texture_root').show();

            if(self.texture && self.texture.hasOwnProperty('url')) {
                objById('select_texture').value = 'Replace';
                objById('texture_thumb').style.visibility = 'visible';
                objById('texture_thumb').src = self.texture.url;
                $('#remove_texture').show();
            } else {
                objById('select_texture').value = 'Add';
                objById('texture_thumb').style.visibility = 'hidden';
                $('#remove_texture').hide();
            }
        } else {
            $('#texture_root').hide();
        }
    },

    setResource: function(val) {
        var self = this;
        var resource = val? val : self.data.resource;
        self.resource = resource;
        var conf = getObjConf(self.data.type);

        $('#resource_root').hide();
        $('#media_setting').hide();
        $('#input_resource').hide();
        $('#file_resource').hide();
        $('#model_resource').hide();
        $('#gallery_resource').hide();

        $('#groupWidget_file').hide();
        $('#groupWidget_link').hide();

        $('#remove_resource').hide();
        $('#select_resource').hide();

        if(conf.type == 'gallery') {
            $('#gallery_resource').show();
        }

        if(conf.ctg != 'element') {
            $('#resource_root').show();
        }
        if(conf.type == 'text') {
            $('#input_resource').show();
        }
        if(conf.ctg == 'group_widget') {
            $('#input_resource').show();
            if(conf.subctg == 'file') { // select resource by adding file
                $('#groupWidget_file').show();
            }
            if(conf.subctg == 'link') { // select resource by adding link
                $('#groupWidget_link').show();
            }
        }
        if(conf.type == '360img' || conf.type == 'image') {
            $('#file_resource').show();
        }
        if(conf.type == 'audio') {
            if(self.resource && self.resource.hasOwnProperty('url')) {
                $('#media_setting').show();

                objById('resource_loop').checked = self.resource.loop? self.resource.loop : false;
                objById('resource_autoplay').checked = self.resource.autoplay? self.resource.autoplay : false;
            } else {
                $('#media_setting').hide();
            }
        }
        if(conf.type == 'video') {
            $('#file_resource').show();
            if(self.resource && self.resource.hasOwnProperty('url')) {
                $('#media_setting').show();

                objById('resource_loop').checked = self.resource.loop? self.resource.loop : false;
                objById('resource_autoplay').checked = self.resource.autoplay? self.resource.autoplay : false;
            } else {
                $('#media_setting').hide();
            }
        }
        if(conf.ctg == 'model') {
            $('#model_resource').show();
            if(self.resource && self.resource.hasOwnProperty('url')) {
                objById('model_url').innerHTML = getFileName(self.resource.url);
                objById('model_loop').checked = self.resource.loop? self.resource.loop : false;
                var totalTime = Number(AREditor.workspace.getAnimationDuration(self.uid).toFixed(2));

                var slider = self.getModelSlider();
                if(slider) { slider.destroy(); }
                $("#model_slider").ionRangeSlider({
                    type: "double",
                    step: 0.1,
                    grid: true,
                    grid_num: 4,
                    min: 0,
                    max: totalTime,
                    from: 0,
                    to: totalTime,
                    drag_interval: false,
                    min_interval: null,
                    max_interval: null,
                    onStart: function (data) {
                        // fired then range slider is ready
                        //console.log('onStart: ', data);
                    },
                    onChange: function (data) {
                        // fired on every range slider update
                        //console.log('onChange: ', data);
                        self.onChangeModelSliderRange(data);
                    },
                    onFinish: function (data) {
                        // fired on pointer release
                        //console.log('onFinish: ', data);
                        self.onFinishModelSliderRange(data);
                    },
                    onUpdate: function (data) {
                        // fired on changing slider with Update method
                        //console.log('onUpdate: ', data);
                    }
                });
                self.refreshModelActions();
                if(self.uid != objById('model_id').value) {
                    self.selectModelAction(0);
                }
            }
        }


        if(self.type == 'video') {
            if(self.resource && self.resource.url) {
                var videoToThumb = new VideoToThumb.default(self.resource.url);
                videoToThumb
                    .load()
                    .type('base64')
                    //.size([320, 240])
                    //.positions([1])
                    //.xy([0,0])
                    .error(function(err) {
                        console.warn(err);
                    })
                    .done(function(imgs) {
                        imgs.forEach(function(img, index) {
                            objById('resource_thumb').src = img;
                        });
                    });
                objById('select_resource').value = 'Replace';
                objById('resource_thumb').style.visibility = 'visible';
                $('#remove_resource').show();
            } else {
                objById('select_resource').value = 'Add';
                objById('resource_thumb').style.visibility = 'hidden';
                $('#select_resource').show();
            }
        }
        /*if(self.type == 'audio') {
            if(self.resource.url) {
                objById('select_resource').value = 'Replace';
                objById('resource_thumb').style.visibility = 'hidden';
                $('#remove_resource').show();
            } else {
                objById('select_resource').value = 'Add';
                objById('resource_thumb').style.visibility = 'hidden';
                $('#select_resource').show();
            }
        }*/

        if(self.type == '360img') {
            if(self.resource && self.resource.url) {
                objById('resource_thumb').src = self.resource.url;

                objById('select_resource').value = 'Replace';
                objById('resource_thumb').style.visibility = 'visible';
                $('#remove_resource').show();
            } else {
                objById('select_resource').value = 'Add';
                objById('resource_thumb').style.visibility = 'hidden';
                $('#select_resource').show();
            }
        }

        if(self.type == 'image') {
            if(self.resource && self.resource.url) {
                objById('resource_thumb').src = self.resource.url;
                objById('resource_thumb').style.visibility = 'visible';
            }
        }

        if(self.type == 'text') {
            if(self.resource) {
                objById('resource_title').value = self.resource.title;
            }
        }

        if(conf.ctg == 'group_widget') {
            if(self.resource) {
                objById('resource_title').value = self.resource.title;
                if(conf.subctg == 'file') {
                    objById('groupWidget_file_name').innerHTML = self.resource.url? getFileName(self.resource.url) : '';
                    objById('groupWidget_file_name').title = self.resource.url? getFileName(self.resource.url) : '';
                    //objById('groupWidget_file_thumb').src = self.resource.url;
                    if(self.resource.url) {
                        objById('select_groupWidget_file').value = 'Replace';
                    } else {
                        objById('select_groupWidget_file').value = 'Add';
                    }
                }
                if(conf.subctg == 'link') {
                    objById('select_groupWidget_link').value = self.resource.url? self.resource.url : '';
                }
            }
        }

        if(conf.type == 'gallery') {
            objById('gallery_list').innerHTML = '';
            if(self.resource && Array.isArray(self.resource.urls)) {
                var html = '';
                self.resource.urls.forEach(function(item, index) {
                    html += '<div class="gallery_item" data-index="'+index+'" data-url="'+item+'"' +
                        ' style="background-image: url('+item+');"><span class="remove_gallery"' +
                        ' onclick="ObjectSettings.removeGalleryItem('+index+')">x</span></div>';
                });
                objById('gallery_list').innerHTML = html;

                $('.gallery_item').arrangeable();
            }

            objById('gallery_autoplay').checked = self.resource.autoplay? self.resource.autoplay : false;
        }
    },

    removeGalleryItem: function(idx) {
        var self = this;
        var arr = self.resource.urls.filter(function(item, index) {
            if(index == idx) return false;
            return true;
        });
        self.resource.urls = arr;

        AREditor.ee.trigger('set_resource', [self.uid, self.resource]);
    },

    setLink: function(val) {
        var self = this;
        var link = val? val : self.data.link;
        self.link = copyObj(link);

        var conf = getObjConf(self.type);

        if(conf.ctg == 'element' || conf.ctg == 'model' || conf.ctg == 'misc') {
            $('#link_root').show();
        } else {
            $('#link_root').hide();
        }

        if(self.link && self.link.hasOwnProperty('url')) {
            objById('link').value = self.link.url;
        } else {
            objById('link').value = '';
        }
    },

    addEvents: function() {
        var self = this;

        objById('delete_object_btn').onclick = self.onDeleteObject.bind(self);

        objById('label').onchange = self.onUpdateLabel.bind(self);

        objById('color').onchange = self.onUpdateColor.bind(self);

        objById('opacity').onchange = self.onUpdateOpacity.bind(self);

        objById('scale_x').onchange = self.onUpdateScale.bind(self);
        objById('scale_y').onchange = self.onUpdateScale.bind(self);
        objById('scale_z').onchange = self.onUpdateScale.bind(self);

        objById('position_x').onchange = self.onUpdatePosition.bind(self);
        objById('position_y').onchange = self.onUpdatePosition.bind(self);
        objById('position_z').onchange = self.onUpdatePosition.bind(self);

        objById('rotation_x').onchange = self.onUpdateRotation.bind(self);
        objById('rotation_y').onchange = self.onUpdateRotation.bind(self);
        objById('rotation_z').onchange = self.onUpdateRotation.bind(self);

        objById('select_texture').onclick = self.onSelectTexture.bind(self);
        objById('remove_texture').onclick = self.onRemoveTexture.bind(self);

        objById('select_resource').onclick = self.onSelectResource.bind(self);
        objById('remove_resource').onclick = self.onRemoveResource.bind(self);

        objById('resource_title').onchange = self.onUpdateResourceTitle.bind(self);

        objById('link').onchange = self.onUpdateLink.bind(self);
        objById('link_test').onclick = self.onTestLink.bind(self);

        objById('visible').onchange = self.onUpdateVisible.bind(self);

        objById('shadow_cast').onchange = self.onUpdateShadowCast.bind(self);
        objById('shadow_receive').onchange = self.onUpdateShadowReceive.bind(self);

        objById('select_groupWidget_file').onclick = self.onSelectResource.bind(self);
        objById('select_groupWidget_link').onchange = self.onUpdateGroupWidgetLink.bind(self);
        objById('test_groupWidget_link').onclick = self.onClickTestWidgetLink.bind(self);

        objById('resource_loop').onchange = function(e) {
            var chk = e.target.checked? true : false;
            if(self.resource) {
                self.resource.loop = chk;
                AREditor.ee.trigger('set_resource', [self.uid, self.resource]);
            }
        };
        objById('resource_autoplay').onchange = function(e) {
            var chk = e.target.checked? true : false;
            if(self.resource) {
                self.resource.autoplay = chk;
                AREditor.ee.trigger('set_resource', [self.uid, self.resource]);
            }
        };

        objById('model_loop').onchange = function(e) {
            var chk = e.target.checked? true : false;
            if(self.resource && self.resource.hasOwnProperty('url')) {
                self.resource.loop = chk;
                AREditor.ee.trigger('set_resource', [self.uid, self.resource]);
            }
        };
        objById('model_action_add').onclick = self.onAddModelAction.bind(self);
        /*
        objById('model_start').onchange = function(e) {
            var total = AREditor.workspace.getAnimationDuration(self.uid);
            if(self.resource && self.resource.hasOwnProperty('url')) {
                var val = Number(e.target.value);
                val = val>0? val : 0;
                val = val>total? total : val;

                e.target.value = Number(val).toFixed(2);
                self.resource.start = val;
                AREditor.ee.trigger('set_resource', [self.uid, self.resource]);
            }
        };
        objById('model_end').onchange = function(e) {
            var total = AREditor.workspace.getAnimationDuration(self.uid);
            if(self.resource && self.resource.hasOwnProperty('url')) {
                var val = Number(e.target.value);
                val = val>0? val : total;
                val = val>total? total : val;
                e.target.value = Number(val).toFixed(2);
                self.resource.end = val;
                AREditor.ee.trigger('set_resource', [self.uid, self.resource]);
            }
        };
        */

        objById('material').onchange = function(e) {
            var material = e.target.value;
            AREditor.ee.trigger('set_material', [self.uid, material]);
        };

        $('#gallery_list').off('drag.end.arrangeable');
        $('#gallery_list').on('drag.end.arrangeable', function() {
            self.onUpdateGalleryResource();
        });
        objById('gallery_file').onclick = self.onAddGallery.bind(self);
        objById('gallery_autoplay').onchange = function(e) {
            var chk = e.target.checked? true : false;
            if(self.resource) {
                self.resource.autoplay = chk;
                AREditor.ee.trigger('set_resource', [self.uid, self.resource]);
            }
        };
    },

    onDeleteObject: function(e ) {
        var self = this;
        AREditor.ee.trigger('delete_object_ws', [self.data]);
    },

    onUpdateLabel: function(e) {
        var self = this;
        var label = objById('label').value;
        if(label != self.label) {
            self.label = label;
            AREditor.ee.trigger('set_label', [self.uid, self.label]);
        }
    },

    onUpdateColor: function(e) {
        var self = this;
        var color = objById('color').jscolor.toHEXString();
        objById('color').value = color;
        if(color != self.color) {
            self.color = color;
            AREditor.ee.trigger('set_color', [self.uid, self.color]);
        }
    },

    onUpdateOpacity: function(e) {
        var self = this;
        //if(e.keyCode == 13) {
            var value = Number(objById('opacity').value);
            if(value<0) {
                value = 0;
            }
            if(value>100) {
                value = 100;
            }

            if(Number(self.opacity)*100 != value) {
                objById('opacity').value = value;
                self.opacity = value/100;
                AREditor.ee.trigger('set_opacity', [self.uid, self.opacity]);
            }
        //}
    },

    onUpdateScale: function(e) {
        var self = this;

        //if(e.keyCode == 13) {
            var x = Number(objById('scale_x').value);
            var y = Number(objById('scale_y').value);
            var z = Number(objById('scale_z').value);

            if(x <= 0) {
                x = 0.1;
                objById('scale_x').value = x;
            }
            if(y <= 0) {
                y = 0.1;
                objById('scale_y').value = y;
            }
            if(z <= 0) {
                z = 0.1;
                objById('scale_x').value = z;
            }

            if(self.scale.x != x || self.scale.y != y || self.scale.z != z) {
                self.scale.x = x;
                self.scale.y = y;
                self.scale.z = z;
                AREditor.ee.trigger('set_scale', [self.uid, self.scale]);
            }
        //}
    },

    onUpdatePosition: function(e) {
        var self = this;

        //if(e.keyCode == 13) {
            var x = Number(objById('position_x').value);
            var y = Number(objById('position_y').value);
            var z = Number(objById('position_z').value);

            if(self.position.x != x || self.position.y != y || self.position.z != z) {
                self.position.x = x;
                self.position.y = y;
                self.position.z = z;
                AREditor.ee.trigger('set_position', [self.uid, self.position]);
            }
        //}
    },

    onUpdateRotation: function(e) {
        var self = this;

        //if(e.keyCode == 13) {
            var x = Number(objById('rotation_x').value)/180 * Math.PI;
            var y = Number(objById('rotation_y').value)/180 * Math.PI;
            var z = Number(objById('rotation_z').value)/180 * Math.PI;

            if(self.rotation.x != x || self.rotation.y != y || self.rotation.z != z) {
                self.rotation.x = x;
                self.rotation.y = y;
                self.rotation.z = z;
                AREditor.ee.trigger('set_rotation', [self.uid, self.rotation]);
            }
        //}
    },

    getResource: function(resource_type, callback) {
        var self = this;
        var url = './upload/resource_list.php'; // Need to modify in the future
        var data = {
            user_id: 66,
            object_id: self.uid,
            type: self.type,
            ctg: getObjConf(self.type).ctg,
            resource_type: resource_type,
        };
        $.ajax({
            url: url,
            type: "POST",
            data: data,
            success: function (result) {
                return callback(null, result);
            },
            error: function (jqXHR, textStatus, errorThrown) {
                console.warn('Error in ' + url, jqXHR, textStatus, errorThrown);
                return callback(textStatus);
            }
        });
    },

    onSelectTexture: function(e) {
        var self = this;

        self.getResource('image', function(err, result) {
            if(err) {
                console.warn(err);
                return;
            }

            self.showSelectTexture(result);
        });
    },

    showSelectTexture: function(val) {
        var temp = JSON.parse(val);
        if(temp.hasOwnProperty('error')) {
            alert('Error: \r\n' + temp.error);
            return;
        }

        var html = '';
        var data = temp.data;
        data.forEach(function(item) {
            html += '<img src="'+item.url+'" width="80px" height="80px" class="texture_thumb" ondblclick="ObjectSettings.selectTexture(\''+item.id+'\', \''+item.url+'\')">';
        });

        var box = new Box({
            contentsType: 'html',
            boxContents: html,
            boxEffect: 'fade',
            isShowCloseBtn: true,
            boxWidth: 500,
            boxHeight: 500,
            isHideByClick: true});
        box.loadBox();
    },
    selectTexture: function(id, url) {
        var self = this;
        if(!self.texture) {
            self.texture = {};
        }
        self.texture.id = id;
        self.texture.url = url;

        AREditor.ee.trigger('set_texture', [self.uid, self.texture]);
    },
    onRemoveTexture: function(e) {
        var self = this;
        //if(!window.confirm('Are you sure to remove the texture')) return;

        self.texture = null;
        AREditor.ee.trigger('set_texture', [self.uid, copyObj(self.texture)]);
    },

    onSelectResource: function(e) {
        var self = this;
        var conf = getObjConf(self.type);
        var resource_type = '';

        if(self.type == 'video') {
            resource_type = 'video';
        }
        if(self.type == 'audio') {
            resource_type = 'audio';
        }
        if(self.type == '360img' || self.type == 'image') {
            resource_type = 'image';
        }
        if(self.type == 'pdf') {
            resource_type = 'pdf';
        }
        self.getResource(resource_type, function(err, result) {
            if(err) {
                console.warn(err);
                return;
            }

            if(resource_type == 'image') {
                self.showImageResource(result);
            }

            if(resource_type == 'audio') {
                self.showAudioResource(result);
            }

            if(resource_type == 'video') {
                self.showVideoResource(result);
            }

            if(resource_type == 'pdf') {
                self.showPDFResource(result);
            }
        });
    },

    showImageResource: function(val) {
        var temp = JSON.parse(val);
        if(temp.hasOwnProperty('error')) {
            alert('Error: \r\n' + temp.error);
            return;
        }

        var html = '';
        var data = temp.data;

        data.forEach(function(item) {
            html += '<img src="'+item.url+'" style="width: 80px; height: 80px; " class="resource_thumb" ondblclick="ObjectSettings.selectResource(\''+item.id+'\', \''+item.url+'\')">';
        });

        var box = new Box({
            contentsType: 'html',
            boxContents: html,
            boxEffect: 'fade',
            isShowCloseBtn: true,
            boxWidth: 500,
            boxHeight: 500,
            isHideByClick: true});
        box.loadBox();
    },

    showVideoResource: function(val) {
        var temp = JSON.parse(val);
        if(temp.hasOwnProperty('error')) {
            alert('Error: \r\n' + temp.error);
            return;
        }

        var html = '';
        var data = temp.data;
        data.forEach(function(item) {
            var videoToThumb = new VideoToThumb.default(item.url);
            videoToThumb
                .load()
                .type('base64')
                //.size([320, 240])
                //.positions([1])
                //.xy([0,0])
                .error(function(err) {
                    console.warn(err);
                })
                .done(function(imgs) {
                    imgs.forEach(function(img, index) {
                        html += '<img src="'+img+'" style="width: 160px; height: 120px; " class="resource_thumb" ondblclick="ObjectSettings.selectResource(\''+item.id+'\', \''+item.url+'\')">';

                        var box = new Box({
                            contentsType: 'html',
                            boxContents: html,
                            boxEffect: 'fade',
                            isShowCloseBtn: true,
                            boxWidth: 500,
                            boxHeight: 500,
                            isHideByClick: true});
                        box.loadBox();
                    });
                });
        });
    },
    selectResource: function(id, url) {
        var self = this;
        if(!self.resource) {
            self.resource = {};
        }
        self.resource.id = id;
        self.resource.url = url;
        self.resource.type = self.data.type;

        AREditor.ee.trigger('set_resource', [self.uid, self.resource]);
    },
    onRemoveResource: function(e) {
        var self = this;
        //if(!window.confirm('Are you sure to remove the resource')) return;

        self.resource = null;
        AREditor.ee.trigger('set_resource', [self.uid, self.resource]);
    },

    onUpdateResourceTitle: function(e) {
        var self = this;
        var text = objById('resource_title').value;
        if(self.resource.title != text) {
            self.resource.title = text;
            self.resource.type = self.data.type;
            AREditor.ee.trigger('set_resource', [self.uid, self.resource]);
        }
    },
    onUpdateVisible: function(e) {
        var self = this;
        self.visible = e.target.checked;

        AREditor.ee.trigger('set_visible', [self.uid, self.visible]);
    },

    onUpdateShadowCast: function(e) {
        var self = this;
        if(!self.shadow) {
            self.shadow = {};
        }
        self.shadow.cast = e.target.checked;

        AREditor.ee.trigger('set_shadow', [self.uid, self.shadow]);
    },
    onUpdateShadowReceive: function(e) {
        var self = this;
        var self = this;
        if(!self.shadow) {
            self.shadow = {};
        }
        self.shadow.receive = e.target.checked;

        AREditor.ee.trigger('set_shadow', [self.uid, self.shadow]);
    },

    onUpdateLink: function(e) {
        var self = this;

        if(!self.link) {
            self.link = {};
        }
        var link = objById('link').value;
        if(self.link.url != link) {
            self.link.url = link;
            self.link.type = self.data.type;

            AREditor.ee.trigger('set_link', [self.uid, self.link]);
        }
    },
    onTestLink: function(e) {
        var url = document.getElementById('link').value;
        if(url) {
            window.open(url, '_blank')
        }
    },

    showAudioResource: function(val) {
        var temp = JSON.parse(val);
        if(temp.hasOwnProperty('error')) {
            alert('Error: \r\n' + temp.error);
            return;
        }

        var html = '';
        var data = temp.data;

        data.forEach(function(item) {
            html += '<div style="color: #fafad2; background-color: #3e999f;cursor: pointer;margin: 10px; padding: 5px;" ' +
                'ondblclick="ObjectSettings.selectResource(\''+item.id+'\', \''+item.url+'\')">' + getFileName(item.url) + '</div>';
        });

        var box = new Box({
            contentsType: 'html',
            boxContents: html,
            boxEffect: 'fade',
            isShowCloseBtn: true,
            boxWidth: 500,
            boxHeight: 500,
            isHideByClick: true});
        box.loadBox();
    },

    showPDFResource: function(val) {
        var temp = JSON.parse(val);
        if(temp.hasOwnProperty('error')) {
            alert('Error: \r\n' + temp.error);
            return;
        }

        var html = '';
        var data = temp.data;

        data.forEach(function(item) {
            html += '<div style="color: #fafad2; background-color: #3e999f;cursor: pointer;margin: 10px; padding: 5px;" ' +
                'ondblclick="ObjectSettings.selectResource(\''+item.id+'\', \''+item.url+'\')">' + getFileName(item.url) + '</div>';
        });

        var box = new Box({
            contentsType: 'html',
            boxContents: html,
            boxEffect: 'fade',
            isShowCloseBtn: true,
            boxWidth: 500,
            boxHeight: 500,
            isHideByClick: true});
        box.loadBox();
    },

    onUpdateGroupWidgetLink: function(e) {
        var self = this;
        var url = objById('select_groupWidget_link').value;
        if(url) {
            self.selectResource(null, url);
        }
    },
    onClickTestWidgetLink: function(e) {
        var self = this;
        var url = objById('select_groupWidget_link').value;
        if(url) {
            if(self.type == 'youtube') {
                if(url.indexOf("/") != -1) {
                    url = url;
                } else {
                    url = "https://www.youtube.com/embed/" + url
                        + "?autoplay=1&autohide=1&border=0&wmode=opaque&enablejsapi=1";
                }
                var html = '<embed src="'+url+'" width="100%" height="100%">';
                var box = new Box({
                    contentsType: 'html',
                    boxContents: html,
                    boxEffect: 'fade',
                    isShowCloseBtn: false,
                    isShowLoading: true,
                    boxWidth: 500,
                    boxHeight: 300,
                    isHideByClick: true,
                });
                box.loadBox();
                return;
            }
            window.open(url, '_blank');
        }
    },

    onUpdateGalleryResource: function() {
        var self = this;
        var urls = [];
        $('.gallery_item').each(function(item, obj) {
            var url = $(obj).attr('data-url');
            urls.push(url);
        });

        self.resource.urls = urls;

        AREditor.ee.trigger('set_resource', [self.uid, self.resource]);
    },
    onAddGallery: function() {
        var self = this;
        self.getResource('image', function(err, result) {
            if(err) {
                console.warn(err);
                return;
            }

            self.showGalleryImages(result);
        });
    },
    showGalleryImages: function(val) {
        var temp = JSON.parse(val);
        if(temp.hasOwnProperty('error')) {
            alert('Error: \r\n' + temp.error);
            return;
        }

        var html = '';
        var data = temp.data;
        data.forEach(function(item) {
            html += '<img src="'+item.url+'" width="80px" height="80px" class="texture_thumb" ondblclick="ObjectSettings.selectGallery(\''+item.id+'\', \''+item.url+'\')">';
        });

        var box = new Box({
            contentsType: 'html',
            boxContents: html,
            boxEffect: 'fade',
            isShowCloseBtn: true,
            boxWidth: 500,
            boxHeight: 500,
            isHideByClick: true});
        box.loadBox();
    },
    selectGallery: function(id, url) {
        var self = this;
        if(!Array.isArray(self.resource.urls)) {
            self.resource.urls = [];
        }

        self.resource.urls.push(url);

        AREditor.ee.trigger('set_resource', [self.uid, self.resource]);
    },

    /**
     * specified model action
     */
    getModelSlider: function() {
        var slider = $("#model_slider").data("ionRangeSlider");
        return slider;
    },
    onAddModelAction: function() {
        var self = this;
        var slider = self.getModelSlider();
        
        if(!self.resource.actionList || !self.resource.actionList.length) { return; }
        
        var from = 0;//slider.result.from;
        var to = Number(Number(AREditor.workspace.getAnimationDuration(self.uid).toFixed(2)));//slider.result.to;
        var name = 'clip'+self.resource.actionList.length;

        var action = {
            name: name,
            start: from,
            end: to,
        };
        self.resource.actionList.push(action);
        self.refreshModelActions();

        setTimeout(function() {
            self.selectModelAction(self.resource.actionList.length-1);
        }, 100);

        AREditor.ee.trigger('set_resource', [self.uid, self.resource]);
    },
    refreshModelActions: function() {
        var self = this;
        var actionList = self.resource.actionList;
        if(!actionList || !Array.isArray(actionList)) {
            objById('model_action_list').innerHTML = '';
            return;
        }

        var html = '';
        actionList.forEach(function(item, index) {
            var start = Number(item.start).toFixed(2);
            var end = Number(item.end).toFixed(2);
            var name = item.name;

            html += '<div style="display: flex;" id="model_action_'+index+'" onclick="ObjectSettings.selectModelAction('+index+')">';
            html += '<div style="width: 30%;">';
            html += '<input type="text" class="model_action_val" value="'+name+'" ' +
                'id="model_action_name_'+index+'" onchange="ObjectSettings.onUpdateModelAction(event, '+index+')">';
            html += '</div>';
            html += '<div style="width: 30%;">';
            html += '<input type="number" min="0" step="0.1" class="model_action_val" value="'+start+'" ' +
                'id="model_action_start_'+index+'" onchange="ObjectSettings.onUpdateModelAction(event, '+index+')">';
            html += '</div>';
            html += '<div style="width: 30%;">';
            html += '<input type="number" min="0" step="0.1" class="model_action_val" value="'+end+'" ' +
                'id="model_action_end_'+index+'" onchange="ObjectSettings.onUpdateModelAction(event, '+index+')">';
            html += '</div>';
            html += '<div style="width: 10%;">';
            html += '<button id="model_action_del_'+index+'" onclick="ObjectSettings.onDeleteModelAction(event, '+index+')">x</button>';
            html += '</div>';
            html += '</div>';
        });
        objById('model_action_list').innerHTML = html;
    },
    getModelActionParam: function(index) {
        try {
            var start = objById('model_action_start_' + index).value;
            var end = objById('model_action_end_' + index).value;
            var name = objById('model_action_name_' + index).value;

            return {name: name, start: start, end: end};
        } catch (e) {
            return null;
        }
    },
    selectModelAction: function(index) {
        var self = this;
        var conf = getObjConf(self.type);
        var data = self.getModelActionParam(index);

        if(!data || conf.type == 'obj') {
            objById('model_action_index').value = index;
            objById('model_action_name').value = '';
            objById('model_id').value = self.uid;
            objById('model_action_from').value = 0;
            objById('model_action_to').value = 0;
            return;
        }

        var slider = self.getModelSlider();
        slider.update({
            from: data.start,
            to: data.end
        });

        objById('model_action_index').value = index;
        objById('model_action_name').value = data.name;
        objById('model_id').value = self.uid;
        objById('model_action_from').value = Number(data.start);
        objById('model_action_to').value = Number(data.end);
    },
    onUpdateModelAction: function(e, idx) {
        var self = this;
        var data = self.getModelActionParam(idx);
        if(!data) { return; }

        self.resource.actionList.forEach(function(item, index) {
            if(index == idx) {
                item.name = data.name;
                item.start = Number(data.start);
                item.end = Number(data.end);
            }
        });

        AREditor.ee.trigger('set_resource', [self.uid, self.resource]);
    },
    onDeleteModelAction: function(e, idx) {
        var self = this;
        var arr = self.resource.actionList.filter(function(item, index) {
            if(index == idx) {
                return false;
            }
            return true;
        });
        self.resource.actionList = arr;
        self.refreshModelActions();

        self.selectModelAction(0);

        AREditor.ee.trigger('set_resource', [self.uid, self.resource]);
    },
    onChangeModelSliderRange: function(result) {
        var self = this;
        var from = Number(objById('model_action_from').value);
        var to = Number(objById('model_action_to').value);
        var focus = null;
        if(result.from != from) {
            focus = result.from;
        } else if(result.to != to) {
            focus = result.to;
        }
        if(focus == null) { return; }
        AREditor.workspace.previewModelOnSlide(self.uid, focus);
    },
    onFinishModelSliderRange: function(result) {
        var self = this;

        var start = result.from;
        var end = result.to;
        var idx = objById('model_action_index').value;

        self.resource.actionList.forEach(function(item, index) {
            if(index == idx) {
                item.start = Number(start);
                item.end = Number(end);
            }
        });

        objById('model_action_from').value = Number(start);
        objById('model_action_to').value = Number(end);

        AREditor.ee.trigger('set_resource', [self.uid, self.resource]);
    },
    // End for specified model action.
};