var SceneSettings = {
    data: null,

    init: function(data) {
        var self = this;
        self.data = data;

        self.setUID();
        self.setMarker();
        self.setLabel();
        self.setColor();
        self.setBGImage();
        self.setBG360Image();

        self.addEvents();

        self.show();
    },

    show: function() {
        $('.settings_root3').hide();
        $('.settings_root2').hide();
        $('.settings_root1').hide();
        $('.settings_root0').show();
    },
    hide: function() {
        $('.settings_root3').hide();
        $('.settings_root2').hide();
        $('.settings_root1').show();
        $('.settings_root0').hide();
    },

    setUID: function(val) {
        var self = this;
        var uid = val? val : self.data.uid;
        self.uid = uid;
    },

    setMarker: function(val) {
        var self = this;
        var data = val? val : self.data.marker;
        self.marker = data;

        if(self.marker && self.marker.url) {
            objById('marker_image').src = self.marker.url;
            objById('marker_add_btn').innerHTML = 'Replace';
            $('#marker_image').show();
            $('#marker_del_btn').show();
        } else {
            objById('marker_image').src = "";
            $('#marker_image').hide();
            objById('marker_add_btn').innerHTML = 'Add';
            $('#marker_del_btn').hide();
        }
    },

    setLabel: function(val) {
        var self = this;
        var label = val? val : self.data.label;
        self.label = label;

        objById('scene_label').value = self.label;
    },

    setColor: function(val) {
        var self = this;
        var color = val? val : self.data.color;

        self.color = color;
        if(objById('scene_color').jscolor) {
            objById('scene_color').value = self.color;
            objById('scene_color').jscolor.fromString(self.color);
        } else {
            setTimeout(function() {
                self.setColor(val);
            }, 500);
        }
    },

    setBGImage: function(val) {
        var self = this;
        var data = val? val : self.data.bgImage;
        self.bgImage = data;

        if(self.bgImage && self.bgImage.hasOwnProperty('url')) {
            objById('bgimg_thumb').src = self.bgImage.url;
        } else {
            objById('bgimg_thumb').src = '';
        }

        if(self.bgImage && self.bgImage.hasOwnProperty('enable')) {
            if(self.bgImage.enable) {
                $('#bgimg_root').show();
            } else {
                $('#bgimg_root').hide();
            }
            objById('bgimg_enable').checked = self.bgImage.enable;
        } else {
            $('#bgimg_root').hide();
            objById('bgimg_enable').checked = false;
        }

        objById('is_panorama').checked = self.bgImage && self.bgImage.isPanorama? true : false;
    },

    setBG360Image: function(val) {
        var self = this;
        var data = val? val : self.data.bg360Image;
        self.bg360Image = data;
        // Cube panorama
        if(self.bg360Image && self.bg360Image.hasOwnProperty('urls')) {
            objById('bg360img_left_thumb').src = self.bg360Image.urls.left;
            objById('bg360img_right_thumb').src = self.bg360Image.urls.right;
            objById('bg360img_top_thumb').src = self.bg360Image.urls.top;
            objById('bg360img_bottom_thumb').src = self.bg360Image.urls.bottom;
            objById('bg360img_front_thumb').src = self.bg360Image.urls.front;
            objById('bg360img_back_thumb').src = self.bg360Image.urls.back;
        } else {
            objById('bg360img_left_thumb').src = '';
            objById('bg360img_right_thumb').src = '';
            objById('bg360img_top_thumb').src = '';
            objById('bg360img_bottom_thumb').src = '';
            objById('bg360img_front_thumb').src = '';
            objById('bg360img_back_thumb').src = '';
        }

        // Sphere panorama
        if(self.bg360Image && self.bg360Image.hasOwnProperty('url')) {
            objById('bg360img_thumb').src = self.bg360Image.url;
        } else {
            objById('bg360img_thumb').src = '';
        }

        if(self.bg360Image && self.bg360Image.hasOwnProperty('enable')) {
            if(self.bg360Image.enable) {
                $('#bg360img_root').show();
            } else {
                $('#bg360img_root').hide();
            }
            objById('bg360img_enable').checked = self.bg360Image.enable;
        } else {
            $('#bg360img_root').hide();
            objById('bg360img_enable').checked = false;
        }
    },

    addEvents: function() {
        var self = this;

        objById('marker_add_btn').onclick = self.onUpdateMarker.bind(self);
        objById('marker_del_btn').onclick = self.onDeleteMarker.bind(self);

        objById('scene_label').onchange = self.onUpdateLabel.bind(self);

        objById('scene_color').onchange = self.onUpdateColor.bind(self);

        objById('bgimg_enable').onchange = self.onCheckBGImage.bind(self);

        objById('bg360img_enable').onchange = self.onCheckBG360Image.bind(self);

        objById('bgimg_btn').onclick = self.onSelectBGImage.bind(self);
        objById('is_panorama').onchange = self.onCheckPanorama.bind(self);

        objById('bg360img_btn').onclick = self.onSelectBG360Image.bind(self);
        objById('bg360img_left_btn').onclick = self.onSelectBG360ImageLeft.bind(self);
        objById('bg360img_right_btn').onclick = self.onSelectBG360ImageRight.bind(self);
        objById('bg360img_top_btn').onclick = self.onSelectBG360ImageTop.bind(self);
        objById('bg360img_bottom_btn').onclick = self.onSelectBG360ImageBottom.bind(self);
        objById('bg360img_front_btn').onclick = self.onSelectBG360ImageFront.bind(self);
        objById('bg360img_back_btn').onclick = self.onSelectBG360ImageBack.bind(self);
    },

    onUpdateLabel: function(e) {
        var self = this;
        var label = objById('scene_label').value;
        if(label != self.label) {
            self.label = label;
            AREditor.ee.trigger('set_scene_label', [self.uid, self.label]);
        }
    },

    onUpdateColor: function(e) {
        var self = this;
        var color = objById('scene_color').jscolor.toHEXString();
        objById('scene_color').value = color;
        if(color != self.color) {
            self.color = color;
            AREditor.ee.trigger('set_scene_color', [self.uid, self.color]);
        }
    },

    onCheckBGImage: function(e) {
        var self = this;
        if(!self.bgImage) {
            self.bgImage = {};
        }

        self.bgImage.enable = objById('bgimg_enable').checked;

        if(self.bgImage.enable) {
            $('#bgimg_root').show();
        } else {
            $('#bgimg_root').hide();
        }

        AREditor.ee.trigger('set_scene_bgimage', [self.uid, copyObj(self.bgImage)]);
    },
    onSelectBGImage: function(e) {
        var self = this;
        self.getResource('image:video', function(err, result) {
            if(err) {
                console.warn(err);
                return;
            }

            self.showSelectImage(result);
        });
    },
    onCheckPanorama: function(e) {
        var self = this;
        var chk = e.target.checked;
        self.bgImage.isPanorama = chk;

        AREditor.ee.trigger('set_scene_bgimage', [self.uid, copyObj(self.bgImage)]);
    },

    showPopupBoxResource: function(html) {
        var self = this;

        var box = new Box({
            contentsType: 'html',
            boxContents: html,
            boxEffect: 'fade',
            isShowCloseBtn: true,
            boxWidth: 500,
            boxHeight: 500,
            isHideByClick: true});
        box.loadBox();
    },

    getResource: function(resource_type, callback) {
        var self = this;
        var url = './upload/resource_list.php'; // Need to modify in the future
        var data = {
            user_id: 66,
            object_id: self.uid,
            resource_type: resource_type,
        };
        $.ajax({
            url: url,
            type: "POST",
            data: data,
            success: function (result) {
                return callback(null, result);
            },
            error: function (jqXHR, textStatus, errorThrown) {
                console.warn('Error in ' + url, jqXHR, textStatus, errorThrown);
                return callback(textStatus);
            }
        });
    },
    showSelectImage: function(val) {
        var self = this;
        var temp = JSON.parse(val);
        if(temp.hasOwnProperty('error')) {
            alert('Error: \r\n' + temp.error);
            return;
        }

        var html = '';
        var data = temp.data;
        data.forEach(function(item) {
            html += '<img src="'+item.url+'" width="80px" height="80px" class="texture_thumb wordwrap" alt="'+item.url+'"' +
                'ondblclick="SceneSettings.selectBGImage(\''+item.id+'\', \''+item.url+'\')">';
        });

        self.showPopupBoxResource(html);
    },
    selectBGImage: function(id, url) {
        var self = this;
        var data = {
            id: id,
            url: url,
            enable: objById('bgimg_enable').checked,
            isPanorama: objById('is_panorama').checked,
        };
        self.setBGImage(data);

        AREditor.ee.trigger('set_scene_bgimage', [self.uid, copyObj(self.bgImage)]);
    },

    onCheckBG360Image: function(e) {
        var self = this;
        if(!self.bg360Image) {
            self.bg360Image = {};
        }

        self.bg360Image.enable = objById('bg360img_enable').checked;

        if(self.bg360Image.enable) {
            $('#bg360img_root').show();
        } else {
            $('#bg360img_root').hide();
        }

        AREditor.ee.trigger('set_scene_bg360image', [self.uid, copyObj(self.bg360Image)]);
    },
    onSelectBG360Image: function(e) {
        var self = this;
        self.getResource('image', function(err, result) {
            if(err) {
                console.warn(err);
                return;
            }

            self.showSelect360Image1(result);
        });
    },
    onSelectBG360ImageLeft: function(e) {
        var self = this;
        self.getResource('image', function(err, result) {
            if(err) {
                console.warn(err);
                return;
            }

            self.showSelect360Image(result, 'left');
        });
    },
    onSelectBG360ImageRight: function(e) {
        var self = this;
        self.getResource('image', function(err, result) {
            if(err) {
                console.warn(err);
                return;
            }

            self.showSelect360Image(result, 'right');
        });
    },
    onSelectBG360ImageTop: function(e) {
        var self = this;
        self.getResource('image', function(err, result) {
            if(err) {
                console.warn(err);
                return;
            }

            self.showSelect360Image(result, 'top');
        });
    },
    onSelectBG360ImageBottom: function(e) {
        var self = this;
        self.getResource('image', function(err, result) {
            if(err) {
                console.warn(err);
                return;
            }

            self.showSelect360Image(result, 'bottom');
        });
    },
    onSelectBG360ImageFront: function(e) {
        var self = this;
        self.getResource('image', function(err, result) {
            if(err) {
                console.warn(err);
                return;
            }

            self.showSelect360Image(result, 'front');
        });
    },
    onSelectBG360ImageBack: function(e) {
        var self = this;
        self.getResource('image', function(err, result) {
            if(err) {
                console.warn(err);
                return;
            }

            self.showSelect360Image(result, 'back');
        });
    },
    showSelect360Image: function(val, direction) {
        var self = this;
        var temp = JSON.parse(val);
        if(temp.hasOwnProperty('error')) {
            alert('Error: \r\n' + temp.error);
            return;
        }

        var html = '';
        var data = temp.data;
        data.forEach(function(item) {
            html += '<img src="'+item.url+'" width="80px" height="80px" class="texture_thumb" ondblclick="SceneSettings.selectBG360Image(\''+item.id+'\', \''+item.url+'\', \''+direction+'\')">';
        });

        self.showPopupBoxResource(html);
    },
    showSelect360Image1: function(val) {
        var self = this;
        var temp = JSON.parse(val);
        if(temp.hasOwnProperty('error')) {
            alert('Error: \r\n' + temp.error);
            return;
        }

        var html = '';
        var data = temp.data;
        data.forEach(function(item) {
            html += '<img src="'+item.url+'" width="80px" height="80px" class="texture_thumb" ondblclick="SceneSettings.selectBG360Image(\''+item.id+'\', \''+item.url+'\')">';
        });

        self.showPopupBoxResource(html);
    },

    selectBG360Image: function(id, url, direction) {
        var self = this;

        var data = {
            id: id,
            urls: (self.bg360Image && self.bg360Image.hasOwnProperty('urls'))? copyObj(self.bg360Image.urls) : {},
            url: (self.bg360Image && self.bg360Image.hasOwnProperty('url'))? copyObj(self.bg360Image.url) : {},
            enable: objById('bg360img_enable').checked,
        };
        if(!direction) {
            data.url = url;
        }


        if(direction == 'left') data.urls.left = url;
        if(direction == 'right') data.urls.right = url;
        if(direction == 'top') data.urls.top = url;
        if(direction == 'bottom') data.urls.bottom = url;
        if(direction == 'front') data.urls.front = url;
        if(direction == 'back') data.urls.back = url;

        self.setBG360Image(data);

        AREditor.ee.trigger('set_scene_bg360image', [self.uid, copyObj(self.bg360Image)]);
    },

    onDeleteMarker: function(e) {
        var self = this;
        self.marker = {};
        self.setMarker(self.marker);
        AREditor.ee.trigger('set_scene_marker', [self.uid, copyObj(self.marker)]);
    },
    onUpdateMarker: function(e) {
        var self = this;
        self.getResource('image', function(err, result) {
            if(err) {
                console.warn(err);
                return;
            }

            self.showMarkerImage(result);
        });
    },
    showMarkerImage: function(val) {
        var self = this;
        var temp = JSON.parse(val);
        if(temp.hasOwnProperty('error')) {
            alert('Error: \r\n' + temp.error);
            return;
        }

        var html = '';
        var data = temp.data;
        data.forEach(function(item) {
            html += '<img src="'+item.url+'" width="80px" height="80px" class="texture_thumb wordwrap" alt="'+item.url+'"' +
                'ondblclick="SceneSettings.selectMarkerImage(\''+item.id+'\', \''+item.url+'\')">';
        });

        self.showPopupBoxResource(html);
    },
    selectMarkerImage: function(id, url) {
        var self = this;
        var data = {
            id: id,
            url: url,
        };
        self.setMarker(data);

        AREditor.ee.trigger('set_scene_marker', [self.uid, copyObj(self.marker)]);
    },
};