function seconds2hms(val) {
    var obj = {};
    var sec_num = val; // don't forget the second param
    var hours   = Math.floor(sec_num / 3600);
    var minutes = Math.floor((sec_num - (hours * 3600)) / 60);
    var seconds = sec_num - (hours * 3600) - (minutes * 60);
    seconds = Number(seconds).toFixed(2);

    obj.original = val;
    obj.h = hours;
    obj.m = minutes;
    obj.s = seconds;

    if (hours   < 10) {
        hours   = "0"+hours;
    }
    if (minutes < 10) {minutes = "0"+minutes;}            
    if (seconds < 10) {seconds = "0"+seconds;}
    obj.hms = hours+':'+minutes+':'+seconds;
    return obj;
}

function randomColor() {
    var letters = '0123456789ABCDEF';
    var color = '#';
    for (var i = 0; i < 6; i++) {
        color += letters[Math.floor(Math.random() * 16)];
    }
    if(color == '#D9D9D9') return randomColor();
    return color;
}
/**
 * Get color (black/white) depending on bgColor so it would be clearly seen.
 * @param bgColor
 * @returns {string} black|white
 */
function getColorByBgColor(bgColor) {
    if (!bgColor) { return ''; }
    return (parseInt(bgColor.replace('#', ''), 16) > 0xffffff / 2) ? 'black' : 'white';
}
/**
 * Get color (black/white) depending on bgColor so it would be clearly seen.
 * @param bgColor
 * @returns {string}
 */
function determinColor(bgColor){
    var hexcolor = bgColor.replace('#', '');
    var r = parseInt(hexcolor.substr(0,2),16);
    var g = parseInt(hexcolor.substr(2,2),16);
    var b = parseInt(hexcolor.substr(4,2),16);
    var yiq = ((r*299)+(g*587)+(b*114))/1000;
    return (yiq >= 128) ? 'black' : 'white';
}


function generateUUID() { // Public Domain/MIT
    var d = new Date().getTime();
    if (typeof performance !== 'undefined' && typeof performance.now === 'function'){
        d += performance.now(); //use high-precision timer if available
    }
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
        var r = (d + Math.random() * 16) % 16 | 0;
        d = Math.floor(d / 16);
        return (c === 'x' ? r : (r & 0x3 | 0x8)).toString(16);
    });
}

function uuidv4() {
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
        var r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
        return v.toString(16);
    });
}

function random(min, max) {
    return Math.floor(Math.random() * (1 + max - min)) + min;
}

function copyObj(obj) {
    if(!obj) return obj;
    //return $.extend({}, obj); // shallow copy
    return $.extend(true, {}, obj); // deep copy, but slow
    //return JSON.parse(JSON.stringify(obj)); // faster jquery
}

function createGraph( t, f, c ) {

    var div = document.createElement( 'div' );
    div.style.display = 'inline-block';
    div.style.width = '200px';
    div.style.height = '120px';

    var canvas = document.createElement( 'canvas' );
    canvas.width = 180;
    canvas.height = 100;
    canvas.style.border = 'solid 1px darkgray';

    var context = canvas.getContext( '2d' );
    context.fillStyle = "rgb(250,250,250)";
    context.fillRect( 0, 0, 180, 100 );

    context.lineWidth = 0.5;
    context.strokeStyle = "rgb(230,230,230)";

    context.beginPath();
    context.moveTo( 0, 20 );
    context.lineTo( 180, 20 );
    context.moveTo( 0, 80 );
    context.lineTo( 180, 80 );
    context.closePath();
    context.stroke();

    context.lineWidth = 2;
    context.strokeStyle = "rgb(255,127,127)";

    var position = { x: 5, y: 80 };
    var position_old = { x: 5, y: 80 };

    var g1 = new TWEEN.Tween( position ).to( { x: 175 }, 2000 ).easing( TWEEN.Easing.Linear.None ).start(0);
    var g2 = new TWEEN.Tween( position ).to( { y: 20 }, 2000 ).easing( f ).onUpdate( function () {

        context.beginPath();
        context.moveTo( position_old.x, position_old.y );
        context.lineTo( position.x, position.y );
        context.closePath();
        context.stroke();

        position_old.x = position.x;
        position_old.y = position.y;

    }).start(0);

    //div.appendChild( document.createTextNode( t ) );
    //div.appendChild( document.createElement( 'br' ) );
    div.appendChild( canvas );

    c.innerHTML = '';
    c.appendChild(div);

    var timer = null;
    var now = 0;
    function looper() {
        now += 10;
        if(now > 2000) {
            clearInterval(timer);
            return;
        }
        g1.update(now);
        g2.update(now);
    }
    timer = setInterval(function() {
        looper();
    }, 10);
    //return div;
}

function tweenMethod(name) {
    var method = TWEEN.Easing.Linear.None;
    switch (name) {
        case 'linear.none':
            method = TWEEN.Easing.Linear.None;
            break;
        case 'back.in':
            method = TWEEN.Easing.Back.In;
            break;
        case 'back.out':
            method = TWEEN.Easing.Back.Out;
            break;
        case 'back.inout':
            method = TWEEN.Easing.Back.InOut;
            break;
        case 'bounce.in':
            method = TWEEN.Easing.Bounce.In;
            break;
        case 'bounce.out':
            method = TWEEN.Easing.Bounce.Out;
            break;
        case 'bounce.inout':
            method = TWEEN.Easing.Bounce.InOut;
            break;
        case 'circular.in':
            method = TWEEN.Easing.Circular.In;
            break;
        case 'circular.out':
            method = TWEEN.Easing.Circular.Out;
            break;
        case 'circular.inout':
            method = TWEEN.Easing.Circular.InOut;
            break;
        case 'cubic.in':
            method = TWEEN.Easing.Cubic.In;
            break;
        case 'cubic.out':
            method = TWEEN.Easing.Cubic.Out;
            break;
        case 'cubic.inout':
            method = TWEEN.Easing.Cubic.InOut;
            break;
        case 'elastic.in':
            method = TWEEN.Easing.Elastic.In;
            break;
        case 'elastic.out':
            method = TWEEN.Easing.Elastic.Out;
            break;
        case 'elastic.inout':
            method = TWEEN.Easing.Elastic.InOut;
            break;
        case 'exponential.in':
            method = TWEEN.Easing.Exponential.In;
            break;
        case 'exponential.out':
            method = TWEEN.Easing.Exponential.Out;
            break;
        case 'exponential.inout':
            method = TWEEN.Easing.Exponential.InOut;
            break;
        case 'quadratic.in':
            method = TWEEN.Easing.Quadratic.In;
            break;
        case 'quadratic.out':
            method = TWEEN.Easing.Quadratic.Out;
            break;
        case 'quadratic.inout':
            method = TWEEN.Easing.Quadratic.InOut;
            break;
        case 'quartic.in':
            method = TWEEN.Easing.Quartic.In;
            break;
        case 'quartic.out':
            method = TWEEN.Easing.Quartic.Out;
            break;
        case 'quartic.inout':
            method = TWEEN.Easing.Quartic.InOut;
            break;
        case 'quintic.in':
            method = TWEEN.Easing.Quintic.In;
            break;
        case 'quintic.out':
            method = TWEEN.Easing.Quintic.Out;
            break;
        case 'quintic.inout':
            method = TWEEN.Easing.Quintic.InOut;
            break;
        case 'sinusoidal.in':
            method = TWEEN.Easing.Sinusoidal.In;
            break;
        case 'sinusoidal.out':
            method = TWEEN.Easing.Sinusoidal.Out;
            break;
        case 'sinusoidal.inout':
            method = TWEEN.Easing.Sinusoidal.InOut;
            break;
    }
    return method;
}

function objById(id) {
    return document.getElementById(id);
}

function getTextSize(text, font) {
    // if given, use cached canvas for better performance
    // else, create new canvas
    var canvas = document.createElement("canvas");
    var context = canvas.getContext("2d");
    context.font = font;
    var metrics = context.measureText(text);
    var width = metrics.width;
    var height = parseInt(context.font)*1.4;
    return {width: width, height: height};
}

function roundRect(ctx, x, y, w, h, r)
{
    ctx.beginPath();
    ctx.moveTo(x+r, y);
    ctx.lineTo(x+w-r, y);
    ctx.quadraticCurveTo(x+w, y, x+w, y+r);
    ctx.lineTo(x+w, y+h-r);
    ctx.quadraticCurveTo(x+w, y+h, x+w-r, y+h);
    ctx.lineTo(x+r, y+h);
    ctx.quadraticCurveTo(x, y+h, x, y+h-r);
    ctx.lineTo(x, y+r);
    ctx.quadraticCurveTo(x, y, x+r, y);
    ctx.closePath();
    ctx.fill();
    ctx.stroke();
}

function stringCleanup(string) {
    return string = (string = (string = (string = (string = (string = (string = string.replace("<br>", "\n")).replace("<left>", " ")).replace("<right>", " ")).replace("<just>", " ")).replace("<valigntop>", " ")).replace("<valignbottom>", " ")).replace("<b>", " ");
}

function getObjConf(type) {
    var obj = null;
    g_object.forEach(function(item) {
        if(item.type == type) {
            obj = item;
        }
    });
    return obj;
}
function defaultLabel(type) {
    var label = getObjConf(type).label;

    return label;
}

function measureTextMultiLine (text, fontSize, lineHeight) {
    var canvas = document.createElement('canvas');
    var context = canvas.getContext('2d');
    context.font = "Normal "+fontSize+"px Arial";
    //align text horizontally center
    context.textAlign = "center";
    //align text vertically center
    context.textBaseline = "middle";
    //context.fillStyle = "#000000";

    for (var w = 0, h = 0, lines = text.split("\n"), lineCount = lines.length, i = 0; i < lineCount; i++) {
        var line = lines[i],
            metrics = context.measureText(line);
        w = Math.max(w, metrics.width), (line.length > 0 || i < lineCount - 1 || 1 === lineCount) && (h += lineHeight);
    }
    return {
        width: w,
        height: h
    };
}
function fillTextMultiLine (context, text, lineHeight, x, y) {
    for (var lines = text.split("\n"), i = 0; i < lines.length; i++) {
        var line = lines[i];
        context.fillText(line, x, y), y += lineHeight;
    }
}
function getFileExtension(str) {
    return str.split('.').pop();
}
function getFileName(str) {
    //return str.substring(str.lastIndexOf('/')+1);
    return str.split('\\').pop().split('/').pop();
}
function isImage(url) {
    var ext = getFileExtension(url);
    var arr = ['jpg', 'png', 'gif', 'jpeg', 'bmp'];
    var flag = false;
    arr.forEach(function(item) {
        if(item == ext) {
            flag = true;
        }
    });
    return flag;
}
function isVideo(url) {
    var ext = getFileExtension(url);
    var arr = ['mp4', 'ogg', 'mov', 'webm'];
    var flag = false;
    arr.forEach(function(item) {
        if(item == ext) {
            flag = true;
        }
    });
    return flag;
}
function jsonBeautify(json) {
    if (typeof json != 'string') {
        json = JSON.stringify(json, undefined, 2);
    }
    json = json.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;');
    return json.replace(/("(\\u[a-zA-Z0-9]{4}|\\[^u]|[^\\"])*"(\s*:)?|\b(true|false|null)\b|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?)/g, function (match) {
        var cls = 'number';
        if (/^"/.test(match)) {
            if (/:$/.test(match)) {
                cls = 'key';
            } else {
                cls = 'string';
            }
        } else if (/true|false/.test(match)) {
            cls = 'boolean';
        } else if (/null/.test(match)) {
            cls = 'null';
        }
        return '<span class="' + cls + '">' + match + '</span>';
    });
}

/**
 * Use to convert urls safely.
 * e.g. amperoctoplus(encodeURI('http://test.com/test 89_sdf/')
 * @param s
 * @return {*}
 */
function amperoctoplus(s) {
    s = s.replace(/&/g, '%26');
    s = s.replace(/#/g, '%23');
    s = s.replace(/\+/g, '%2B');
    s = s.replace(/@/g, '%40');
    s = s.replace(/:/g, '%3A');
    return s;
}
