function Workspace(ee, container) {
    var self = this;
    self.ee = ee; // EventEmitter

    self.glScene = null;
    self.cssScene = null;

    self.glRenderer = null;
    self.cssRenderer = null;

    self.camera3d = null;
    self.camera2d = null;

    self.orbitControl = null; // self.orbitControl
    self.transControl = null; // self.transControl
    self.dragControl = null; // self.dragControl

    self.INTERSECTED = null; // Selected Object
    self.raycaster = null;

    self.gridHelper = null;

    self.clock = null;

    self.container = container; // canvas Workspace container

    self.objects = [];

    self.ctrlkey = false;
    self.shiftkey = false;

    self.animationList = [];

    self.init();
}

Workspace.prototype.init = function() {
    var self = this;


    self.clock = new THREE.Clock();

    self.createScene();
    self.createCamera();
    self.createRenderer();
    self.createGridHelper();
    self.createOrbitControls();
    self.createTransformControls();
    self.createOutBox();
    self.createRayCaster();
    self.addEventResize();

    self.addKeyboardEvent();

    self.animate();
    self.render();

    self.addExternalEvent();

    self.zoom = 100;

    self.addChromakeyVideo({uid: 'chromakey'});
};
Workspace.prototype.addExternalEvent = function() {
    var self = this;
    // Force zoom
    window.addEventListener( 'wheel', onDocumentMouseWheel, false );
    function onDocumentMouseWheel(event ) {
        if(event.deltaY > 0) {
            self.zoom = self.zoom *0.95;
        } else {
            self.zoom = self.zoom /0.95;
        }
        self.reviewZoom();
        //console.log(Math.floor(self.zoom));
        return;
        var fov = self.activeCamera.fov + event.deltaY * 0.05;
        self.activeCamera.fov = THREE.Math.clamp( fov, 10, 90 );
        self.activeCamera.updateProjectionMatrix();
    }
};
Workspace.prototype.size = function() {
    var self = this;
    var width = self.container.clientWidth;
    var height = self.container.clientHeight;
    var aspect = width/height;
    return {
        width: width,
        height: height,
        aspect: aspect
    };
};
Workspace.prototype.createScene = function() {
    var self = this;
    self.glScene = new THREE.Scene();
};
Workspace.prototype.setBackgroundColor = function(data) {
    var self = this;

    self.glRenderer.autoClear = false;
    self.glRenderer.setClearColor(new THREE.Color( data ).getHex(), 1);
};
Workspace.prototype.setBackgroundImage = function(data) {
    var self = this;
    var url = data.url;

    self.unsetBackgroundImage();

    if(data.isPanorama) {
        self.setBackground360Image(data);
        return;
    }

    if(isImage(url)) {
        self.imageTexture(url, function(err, result) {
            if(err) {
                console.log(err);
                return;
            }

            var texture = result;
            self.glScene.background = texture;
            return;
        });
    }

    if(isVideo(url)) {
        self.videoTexture('video_background', url, true, true, function(err, result) {
            if(err) {
                console.warn(err);
            } else {
                var texture = result;
                console.log(texture);
                self.glScene.background = texture;
            }
        });
    }
};
Workspace.prototype.unsetBackgroundImage = function() {
    var self = this;
    self.glScene.background = null;
    self.removeAnimationByType('particle_effect');
    self.removeHTMLResource('video_background');
};
Workspace.prototype.setBackground360Image = function(data) {
    var self = this;
    /*var left = data.urls.left? data.urls.left : './assets/images/background.png';
    var right = data.urls.right? data.urls.right : './assets/images/background.png';
    var top = data.urls.top? data.urls.top : './assets/images/background.png';
    var bottom = data.urls.bottom? data.urls.bottom : './assets/images/background.png';
    var front = data.urls.front? data.urls.front : './assets/images/background.png';
    var back = data.urls.back? data.urls.back : './assets/images/background.png';

    var urls = [
        left, right,
        top, bottom,
        front, back
    ];

    var reflectionCube = new THREE.CubeTextureLoader().load( urls );
    reflectionCube.format = THREE.RGBFormat;

    self.glScene.background = reflectionCube;*/

    var url = data.url;

    if(isVideo(url)) {
        self.videoTexture('video_background', url, true, true, function(err, result) {
            if(err) {
                console.warn(err);
            } else {
                var texture = result;
                texture.mapping = THREE.UVMapping;
                var options = {
                    resolution: 1024,
                    generateMipmaps: true,
                    minFilter: THREE.LinearMipMapLinearFilter,
                    magFilter: THREE.LinearFilter
                };
                self.glScene.background = new THREE.CubemapGenerator(self.glRenderer).fromEquirectangular(texture, options);
            }
        });
    }


    if(isImage(url)) {
        self.imageTexture(url, function (err, result) {
            if (err) {
                console.log(err);
                return;
            }

            var texture = result;
            texture.mapping = THREE.UVMapping;
            var options = {
                resolution: 1024,
                generateMipmaps: true,
                minFilter: THREE.LinearMipMapLinearFilter,
                magFilter: THREE.LinearFilter
            };
            self.glScene.background = new THREE.CubemapGenerator(self.glRenderer).fromEquirectangular(texture, options);
        });
    }
};
Workspace.prototype.unsetBackground360Image = function() {
    var self = this;
    self.glScene.background = null;
};
Workspace.prototype.createCamera = function(angle, aspect, near, far, position) {
    var self = this;
    var z = 100;
    var angle = angle? angle : 50;
    var aspect = aspect? aspect : self.size().aspect;
    var near = near? near : 0.01;
    var far = far? far : 1000;
    var position = position? position : {x: 0, y: 50, z: z};


    self.createCamera3d(angle, aspect, near, far, position);

    self.createCamera2d(angle, aspect, near, far, position);


    if(!self.activeCamera) {
        self.activeCamera = self.camera3d;
        self.activeCamera.active_type = '3d';
    }

    self.reviewZoom();
};
Workspace.prototype.createCamera3d = function(angle, aspect, near, far, position) {
    var self = this;

    self.camera3d = new THREE.PerspectiveCamera(angle, aspect, near, far); // 카메라 생성
    self.camera3d.position.x = position.x;
    self.camera3d.position.y = position.y;
    self.camera3d.position.z = position.z;
    //self.camera3d.lookAt( self.glScene.position );
};
Workspace.prototype.createCamera2d = function(angle, aspect, near, far, position) {
    var self = this;
    var z = position.z;
    var depht_s = Math.tan(angle/2.0 * Math.PI/180.0) * 2.0;
    var size_y = depht_s * z;
    var size_x = depht_s * z * aspect;
    var left = -size_x/2;
    var right = size_x/2;
    var top = size_y/2;
    var bottom = -size_y/2;
    /*
    var left = -self.size().width/2;
    var right = self.size().width/2;
    var top = self.size().height/2;
    var bottom = -self.size().height/2;*/

    self.camera2d = new THREE.OrthographicCamera(left, right, top, bottom, near, far); // 카메라 생성
    self.camera2d.position.x = position.x;
    self.camera2d.position.y = position.y;
    self.camera2d.position.z = position.z;
    //self.camera2d.lookAt( self.glScene.position );
};
Workspace.prototype.refreshCamera = function() {
    var self = this;
    var aspect = self.size().aspect;

    self.camera3d.aspect = aspect; // 카메라 종횡비 조정
    self.camera3d.updateProjectionMatrix(); // 카메라 현시 조정
    //self.camera3d.lookAt(self.glScene.position);

    var angle = self.camera3d.fov;
    var z = self.camera3d.position.z;
    var aspect = self.size().aspect;
    var depht_s = Math.tan(angle/2.0 * Math.PI/180.0) * 2.0;
    var size_y = depht_s * z;
    var size_x = depht_s * z * aspect;
    var left = -size_x/2;
    var right = size_x/2;
    var top = size_y/2;
    var bottom = -size_y/2;
    self.camera2d.left = left;
    self.camera2d.right = right;
    self.camera2d.top = top;
    self.camera2d.bottom = bottom;
    self.camera2d.updateProjectionMatrix(); // 카메라 현시 조정
    //self.camera2d.lookAt(self.glScene.position);
};
Workspace.prototype.reviewZoom = function() {
    var self = this;
    if(self.orbitControl && self.orbitControl.autoRotate) { return; }
    var zoom = 1;
    if(self.activeCamera.active_type == '3d') {
        zoom = 100/self.activeCamera.position.z;
    } else {
        zoom = self.activeCamera.zoom;
    }
    zoom = self.zoom? Math.floor(self.zoom) : 100;
    objById('zoom').innerHTML = (zoom) + '%';
};
Workspace.prototype.setViewMode = function() {
    var self = this;
    self.createCamera();
    var zoom = 1;
    self.zoom = 100;
    if(self.activeCamera.active_type == '3d') {
        zoom = 100/self.activeCamera.position.z;
        self.activeCamera = self.camera2d;
        self.activeCamera.active_type = '2d';
        //self.activeCamera.zoom = zoom;
    } else {
        zoom = 100/self.activeCamera.zoom;
        self.activeCamera = self.camera3d;
        self.activeCamera.active_type = '3d';
        //self.activeCamera.position.z = zoom;
    }
    //return;
    self.createGridHelper();
    self.createOrbitControls();
    self.unselectObject();
    self.createTransformControls();
    self.createRayCaster();

    if(self.activeCamera.active_type == '3d') {
        self.transControl.showZ = true;
    } else {
        self.transControl.showZ = false;
    }

    self.reviewZoom();
    /*
    self.camera3d.aspect = self.size().aspect; // 카메라 종횡비 조정
    self.camera3d.updateProjectionMatrix(); // 카메라 현시 조정
    self.camera2d.updateProjectionMatrix(); // 카메라 현시 조정*/
};
Workspace.prototype.selectTool = function(type) {
    var self = this;

    if(type == 'translate') {
        self.transControl.setMode( "translate" );
    }
    if(type == 'rotation') {
        self.transControl.setMode( "rotate" );
    }
    if(type == 'scale') {
        self.transControl.setMode( "scale" );
    }
};
Workspace.prototype.createRenderer = function() {
    var self = this;
    var width = self.size().width;
    var height = self.size().height;
    self.glRenderer = new THREE.WebGLRenderer({alpha: true, antialias: true, preserveDrawingBuffer: true}); // 렌더러 생성, antialias: 오브젝트의 테두리가 매출하게 설정
    self.glRenderer.setPixelRatio( window.devicePixelRatio );
    self.glRenderer.setSize(width, height);
    self.glRenderer.autoClear = false;
    self.glRenderer.shadowMap.enabled = true;
    self.glRenderer.shadowMap.type = THREE.PCFSoftShadowMap;
    //self.glRenderer.gammaInput = true;
    //self.glRenderer.gammaOutput = true;
    //self.glRenderer.sortObjects = false; // if set as false then, the objects will render in the order they are added to the scene. Ignore rendering by renderOrder value of object. Use always show top transform controller.
    self.container.appendChild(self.glRenderer.domElement);
    self.glRenderer.domElement.addEventListener('webglcontextlost', function(e) {
        console.warn(e);
    });

};
Workspace.prototype.createGridHelper = function() {
    var self = this;

    if(self.gridHelper && self.axesHelper) {
        return;
        self.glScene.remove( self.gridHelper );
        self.gridHelper = null;
    }

    self.gridHelper = new THREE.GridHelper( 300, 30 );
    self.gridHelper.name = 'main_board';
    self.gridHelper.color1 = 0x000000;
    self.gridHelper.color2 = 0x000000;
    self.gridHelper.material.opacity = 0.66;
    self.gridHelper.material.depthWrite = false;
    self.gridHelper.material.transparent = true;
    self.gridHelper.renderOrder = -100;
    //self.gridHelper.lookAt(new THREE.Vector3(0,90,0));
    self.glScene.add( self.gridHelper );


    self.axesHelper = new THREE.AxesHelper( 1000 );
    self.axesHelper.name = 'axis_helper';
    self.axesHelper.renderOrder = -101;
    self.glScene.add(self.axesHelper);
};
Workspace.prototype.createOrbitControls = function() {
    var self = this;

    if(self.orbitControl) {
        self.orbitControl.enabled = false;
        self.orbitControl = null;
    }

    self.orbitControl = new THREE.OrbitControls(self.activeCamera, self.glRenderer.domElement);
    //self.orbitControl.target = new THREE.Vector3(0,0,0);
    //self.orbitControl.maxDistance = 150;
    self.orbitControl.autoRotate = false;
    self.orbitControl.enableDamping = true;
    self.orbitControl.dampingFactor =  1.5;
    self.orbitControl.enableZoom = true;
    self.orbitControl.enablePan = true;
    self.orbitControl.enableKeys = false;
    self.orbitControl.enableRotate = self.activeCamera.active_type=='2d'? false : true;
    self.orbitControl.screenSpacePanning = true;
    self.orbitControl.update();
    self.orbitControl.addEventListener( 'change', function(e) {

    } );

};
Workspace.prototype.createTransformControls = function() {
    var self = this;

    self.transControl = null;
    self.transControl = new THREE.TransformControls( self.activeCamera, self.glRenderer.domElement );
    /*self.transControl.traverse(function(obj) {
        obj.isTransformControls = true;
    });*/
    self.transControl.addEventListener( 'change', function(e) {
        var object = self.transControl.object;

        if(object) {
            self.selectionBox.setFromObject( object );
        }
    });
    self.transControl.addEventListener( 'objectChange', function(e) {
        //self.fireObjectChange(self.INTERSECTED);
    } );
    self.transControl.addEventListener( 'mouseDown', function(e) {
        //console.log('mouseDown', self.INTERSECTED.position);
        self.oldObjData = self.getObjectData(self.INTERSECTED);
    } );
    self.transControl.addEventListener( 'mouseUp', function(e) {
        //console.log('mouseUp', self.INTERSECTED.position);
        self.newObjData = self.getObjectData(self.INTERSECTED);
    } );
    self.transControl.addEventListener( 'dragging-changed', function ( e ) {
        self.orbitControl.enabled = ! e.value;

        if(e.value === false) { // End dragging
            self.addHistory();
            self.fireObjectChange(self.getObjectData(self.INTERSECTED));
        }
    } );



    self.dragControl = new THREE.DragControls( self.objects, self.activeCamera, self.glRenderer.domElement );
    self.dragControl.addEventListener( 'dragstart', function (e) {
        e.target.enabled = false;
        var conf = getObjConf(e.object.userData.type);
        if(!conf) return;

        if(AREditor.isPreview === true) {
            return;
        }

        if(e.object !== self.INTERSECTED) {
            return;
        }

        if(self.transControl.getMode() == 'translate') {
            e.target.enabled = true;
            self.orbitControl.enabled = false;
            self.oldObjData = self.getObjectData(e.object);
        }
    } );
    self.dragControl.addEventListener( 'dragend', function (e) {
        e.target.enabled = false;
        var conf = getObjConf(e.object.userData.type);
        if(!conf) return;

        if(AREditor.isPreview === true) {
            return;
        }
        if(e.object !== self.INTERSECTED) {
            return;
        }
        if(self.transControl.getMode() == 'translate') {
            e.target.enabled = true;
            self.orbitControl.enabled = true;
            self.newObjData = self.getObjectData(e.object);

            self.addHistory();
            self.fireObjectChange(self.getObjectData(e.object));
        }
    } );
    self.dragControl.addEventListener( 'drag', function (e) {
        e.target.enabled = false;
        var conf = getObjConf(e.object.userData.type);
        if(!conf) return;

        if(AREditor.isPreview === true) {
            return;
        }
        if(self.transControl.getMode() == 'translate') {
            e.target.enabled = true;
            var object = e.object;

            if (object) {
                self.selectionBox.setFromObject(object);
            }
        }
    } );

};
Workspace.prototype.addHistory = function(oldData, newData) {
    var self = this;
    var oldObjData = oldData? oldData : self.oldObjData;
    var newObjData = newData? newData : self.newObjData;

    if(oldObjData && newObjData && oldObjData.uuid == newObjData.uuid) {
        AREditor.history.add({
            undo: function() {
                self.resetObject(oldObjData);
                self.fireObjectChange(oldObjData);
            },
            redo: function() {
                self.resetObject(newObjData);
                self.fireObjectChange(newObjData);
            }
        });
    }
};
Workspace.prototype.fireObjectChange = function(obj) {
    var self = this;
    self.ee.trigger('update_object_workspace', [obj]);
};
Workspace.prototype.setObjectData = function(uid, data) {
    var self = this;
};
Workspace.prototype.getObjectData = function(obj) {
    var self = this;
    var data = {
        uuid: obj.uuid,
        userData: copyObj(obj.userData),
        position: copyObj({x: obj.position.x, y: obj.position.y, z: obj.position.z}),
        rotation: copyObj({x: obj.rotation.x, y: obj.rotation.y, z: obj.rotation.z}),
        scale: copyObj({x: obj.scale.x, y: obj.scale.y, z: obj.scale.z}),
        opacity: Number(obj.userData.opacity),
    };
    return data;
};
Workspace.prototype.resetObject = function(obj) {
    var self = this;
    /**
     * Used when object is changed by transform control.
     * Reset position, scale, rotate as obj parameter.
     */
    var uid = obj.userData.uid;
    var nowObj = self.getObjectByUID(uid);
    self.focusObjectByUID(uid);

    nowObj.position.x = obj.position.x;
    nowObj.position.y = obj.position.y;
    nowObj.position.z = obj.position.z;

    nowObj.rotation.x = obj.rotation.x;
    nowObj.rotation.y = obj.rotation.y;
    nowObj.rotation.z = obj.rotation.z;

    nowObj.scale.x = obj.scale.x;
    nowObj.scale.y = obj.scale.y;
    nowObj.scale.z = obj.scale.z;

    //nowObj.material.opacity = obj.opacity;
};

Workspace.prototype.focusedTag = function() {
    return document.activeElement.tagName;
};
Workspace.prototype.addKeyboardEvent = function() {
    var self = this;

    window.addEventListener( 'keydown', function ( event ) {
        console.log(event.keyCode, self.focusedTag());

        var moveDistance = 10 * 0.1; // 200 pixels per second
        var rotateAngle = Math.PI / 2 * 0.1;   // pi/2 radians (90 degrees) per second


        switch ( event.keyCode ) {
            case 81: // Q
                self.transControl.setSpace( self.transControl.space === "local" ? "world" : "local" );
                break;
            case 16: // Shift
                self.shiftkey = true;
                break;
            case 17: // Ctrl
                self.transControl.setTranslationSnap( 100 );
                self.transControl.setRotationSnap( THREE.Math.degToRad( 15 ) );
                self.ctrlkey = true;
                break;
                /*
            case 87: // W
                self.transControl.setMode( "translate" );
                break;
            case 69: // E
                self.transControl.setMode( "rotate" );
                break;
            case 82: // R
                self.transControl.setMode( "scale" );
                break;
                */
            case 187:
            case 107: // +, =, num+
                self.transControl.setSize( self.transControl.size + 0.1 );
                break;
            case 189:
            case 109: // -, _, num-
                self.transControl.setSize( Math.max( self.transControl.size - 0.1, 0.1 ) );
                break;
                /*
            case 88: // X
                if(self.ctrlkey === false) self.transControl.showX = ! self.transControl.showX;
                break;
            case 89: // Y
                if(self.ctrlkey === false) self.transControl.showY = ! self.transControl.showY;
                break;
            case 90: // Z
                if(self.ctrlkey === false) self.transControl.showZ = ! self.transControl.showZ;
                break;
                */
            case 32: // Spacebar
                self.transControl.enabled = ! self.transControl.enabled;
                break;
            case 37: // arrow-left
                if(self.INTERSECTED && self.focusedTag() != 'INPUT') {
                    if(self.ctrlkey && self.shiftkey) {
                        self.INTERSECTED.translateX( -moveDistance );
                    } else if(self.ctrlkey && !self.shiftkey) {
                        self.INTERSECTED.rotateOnAxis( new THREE.Vector3(0,1,0), rotateAngle);
                    } else {
                        self.INTERSECTED.position.x -= moveDistance;
                    }
                    self.fireObjectChange(self.INTERSECTED);
                }
                break;
            case 38: // arrow-up
                if(self.INTERSECTED && self.focusedTag() != 'INPUT') {
                    if(self.ctrlkey && self.shiftkey) {
                        self.INTERSECTED.translateZ( -moveDistance );
                    } else if(self.ctrlkey && !self.shiftkey) {
                        self.INTERSECTED.rotateOnAxis( new THREE.Vector3(1,0,0), rotateAngle);
                    } else {
                        self.INTERSECTED.position.z -= moveDistance;
                    }
                    self.fireObjectChange(self.INTERSECTED);
                }
                break;
            case 39: // arrow-right
                if(self.INTERSECTED && self.focusedTag() != 'INPUT') {
                    if(self.ctrlkey && self.shiftkey) {
                        self.INTERSECTED.translateX( moveDistance );
                    } else if(self.ctrlkey && !self.shiftkey) {
                        self.INTERSECTED.rotateOnAxis( new THREE.Vector3(0,1,0), -rotateAngle);
                    } else {
                        self.INTERSECTED.position.x += moveDistance;
                    }
                    self.fireObjectChange(self.INTERSECTED);
                }
                break;
            case 40: // arrow-down
                if(self.INTERSECTED && self.focusedTag() != 'INPUT') {
                    if(self.ctrlkey && self.shiftkey) {
                        self.INTERSECTED.translateZ( moveDistance );
                    } else if(self.ctrlkey && !self.shiftkey) {
                        self.INTERSECTED.rotateOnAxis( new THREE.Vector3(1,0,0), -rotateAngle);
                    } else {
                        self.INTERSECTED.position.z += moveDistance;
                    }
                    self.fireObjectChange(self.INTERSECTED);
                }
                break;
        }
    } );
    window.addEventListener( 'keyup', function ( event ) {
        switch ( event.keyCode ) {
            case 17: // Ctrl
                self.transControl.setTranslationSnap( null );
                self.transControl.setRotationSnap( null );
                self.ctrlkey = false;
                self.shiftkey = false;
                break;
            case 46: // Del
                if(self.INTERSECTED && self.focusedTag() != 'INPUT' && AREditor.isPreview === false) {
                    self.ee.trigger('delete_object_ws', [self.INTERSECTED.userData]);
                }
                //self.removeObject();
                break;
        }
    } );
};
Workspace.prototype.getMousePos3D = function(event) {
    var self = this;
    var vec = new THREE.Vector3(); // create once and reuse
    var pos = new THREE.Vector3(); // create once and reuse

    vec.set(
        ( event.clientX / window.innerWidth ) * 2 - 1,
        - ( event.clientY / window.innerHeight ) * 2 + 1,
        0.5);

    vec.unproject( self.activeCamera );

    var z = vec.z/10;

    vec.sub( self.activeCamera.position ).normalize();

    var distance = - self.activeCamera.position.z / vec.z;

    pos.copy( self.activeCamera.position ).add( vec.multiplyScalar( distance ) );
    pos.z = z;
    return pos;
};
Workspace.prototype.createRayCaster = function() {
    var self = this;

    self.raycaster = new THREE.Raycaster();


    self.container.onmousedown = function(e) {
        e.preventDefault();

        var mouse = new THREE.Vector2();
        /*
        mouse.x = ( e.clientX / window.innerWidth ) * 2 - 1;
        mouse.y = - ( e.clientY / window.innerHeight ) * 2 + 1;
        */

        mouse.x = 2 * (e.offsetX / self.size().width) - 1;
        mouse.y = 1 - 2 * ( e.offsetY / self.size().height );


        self.raycaster.setFromCamera( mouse, self.activeCamera );

        var intersects = self.raycaster.intersectObjects( self.objects, true); // true: for picking group object like fbx, obj.
        if ( intersects.length > 0 ) {
            //self.orbitControl.enabled = false;

            //self.outlinePass.selectedObjects = [intersects[0].object];
            /*
            if(intersects[0].object.name == 'main_board') {
                return;
            }
            if(intersects[0].object.name == 'axis_helper') {
                return;
            }
            if(intersects[0].object.name == 'obj_outline') {
                return;
            }
            if(intersects[0].object.type != 'Mesh') {
                return;
            }
            */
            var obj = intersects[0].object;

            if(obj.parent.type != 'Scene') { // It is model or group...
                obj = obj.parent;
            }
            var conf = getObjConf(obj.userData.type);
            if(!conf) {
                return;
            }

            if(conf.type == 'gallery' && typeof obj.onclick === 'function') {
                obj.onclick(intersects[0].point.x);
            }

            if(conf.ctg == 'light' && !obj.isLight) { // It is light helper.
                obj = self.getObjectByUID(obj.userData.uid);
            }


            if (self.INTERSECTED != obj) {

                self.INTERSECTED = obj;

                self.selectObject(self.INTERSECTED);
                self.focusObject(self.INTERSECTED);

                self.ee.trigger('selected_object', [self.INTERSECTED]);
                //console.log('selected_object', self.INTERSECTED);
            }
        } else {
            return;
        }
    };

    self.container.ondblclick = function(e) {
        e.preventDefault();
        var mouse = new THREE.Vector2();
        mouse.x = 2 * (e.offsetX / self.size().width) - 1;
        mouse.y = 1 - 2 * ( e.offsetY / self.size().height );
        self.raycaster.setFromCamera( mouse, self.activeCamera );
        var intersects = self.raycaster.intersectObjects( self.objects, true); // true: for picking group object like fbx, obj.
        if ( intersects.length > 0 ) {
            if(self.INTERSECTED) {
                if(typeof self.INTERSECTED.ondblclick === 'function') {
                    self.INTERSECTED.ondblclick();
                }
            }
        } else {
            self.orbitControl.enabled = true;

            self.unselectObject();

            self.ee.trigger('unselected_object');
        }
    };
};
Workspace.prototype.addEventResize = function() {
    var self = this;
    function onResize(e) {
        var w = self.size().width;
        var h = self.size().height;
        var a = self.size().aspect;

        self.refreshCamera();


        self.glRenderer.setSize(w, h); //렌더러 크기 조정
    }
    window.addEventListener('resize', onResize);
    addResizeListener(document.getElementById('workspace_container'), onResize);
};
Workspace.prototype.animate = function() {
    var self = this;
    requestAnimationFrame(function() { self.animate(); }); // 애니매션 반복 호출

    //self.activeCamera.updateProjectionMatrix();

    //if(self.orbitControl.autoRotate) {
        self.orbitControl.update();
    //}

    self.refreshCamera();
    /**
     * Important!!! for multiple model animation...
     * If not specified in here, and set in go() function... then only played one model, not played others.
     */
    var delta = self.getClock();
    self.animationList.forEach(function(item) {
        item.go(delta);
    });

    self.render();
};
Workspace.prototype.render = function() {
    var self = this;
    //self.activeCamera.lookAt( self.glScene.position );
    self.glRenderer.clear();
    self.glRenderer.render(self.glScene, self.activeCamera);
};
Workspace.prototype.getClock = function() {
    return this.clock.getDelta();
};

Workspace.prototype.getObjectByUID = function(uid) {
    var self = this;
    var obj = null;
    for(var i=0; i<self.objects.length; i++) {
        var t = self.objects[i];
        var conf = getObjConf(t.userData.type);
        if(t.userData.uid == uid) {
            if(conf.ctg == 'light' && !t.isLight)  {
                continue;
            }
            obj = self.objects[i];
            break;
        }
    }
    return obj;
};
Workspace.prototype.unselectObject = function() {
    var self = this;
    self.glScene.remove(self.transControl);
    self.transControl.detach();
    self.INTERSECTED = null;
    self.selectionBox.visible = false;
};
Workspace.prototype.selectObject = function(obj) {
    var self = this;
    self.INTERSECTED = obj;
};
Workspace.prototype.focusObjectByUID = function(uid) {
    var self = this;
    var obj = self.getObjectByUID(uid);
    if(obj) {
        return self.focusObject(obj);
    }
    return null;
};
Workspace.prototype.focusObject = function(obj) {
    var self = this;
    self.transControl.detach();
    self.glScene.remove(self.transControl);

    self.INTERSECTED = obj;
    if(AREditor.isPreview === false) {
        self.transControl.attach(self.INTERSECTED);
        var conf = getObjConf(self.INTERSECTED.userData.type);
        if(conf.ctg === 'model') {
            var box = new THREE.Box3().setFromObject( self.INTERSECTED );
            /*var center = new THREE.Vector3();
            box.getCenter( center );
            self.INTERSECTED.position.sub( center ); // center the model*/
            self.transControl.position.y = box.getSize(new THREE.Vector3()).y/2;
        } else {
            self.transControl.position.y = 0;
        }

        self.glScene.add( self.transControl );

        self.addOutline(self.INTERSECTED);
    }

    return self.INTERSECTED;
};
Workspace.prototype.createOutBox = function() {
    var self = this;
    self.selectionBox = new THREE.BoxHelper();
    self.selectionBox.material.depthTest = false;
    self.selectionBox.material.transparent = true;
    self.selectionBox.material.color.set('#ff8c11');
    self.selectionBox.visible = false;
    self.glScene.add( self.selectionBox );
};
Workspace.prototype.addOutline = function(obj) {
    var self = this;

    var targetObj = obj? obj : self.INTERSECTED;

    if(!targetObj) {
        return;
    }

    var conf = getObjConf(targetObj.userData.type);

    self.selectionBox.visible = false;

    if(conf.ctg == 'light') { return; }
    if(conf.ctg == 'particle') { return; }

    if(AREditor.isPreview === true) { return; }

    var box = new THREE.Box3();
    box.setFromObject( targetObj );
    if ( box.isEmpty() === false ) {
        self.selectionBox.setFromObject( targetObj );
        self.selectionBox.visible = true;
    }
};

Workspace.prototype.addObject = function(obj) {
    var self = this;
    self.objects.push(obj);

    self.selectObject(obj);
    self.focusObject(obj);
};
Workspace.prototype.removeObject = function(obj) { // obj: three.js object, it is not an instance of ARObject.
    var self = this;
    var target = obj;
    if(!obj) {
        target = self.INTERSECTED;
    }

    if(target) {
        var uid = target.userData.uid;

        self.removeAnimationByUID(uid);
        self.removeHTMLResource(uid);

        var aaa = self.objects.filter(function(item) {
            if(target.userData.uid == item.userData.uid) return false;
            return true;
        });
        self.objects = aaa;


        self.unselectObject();

        //self.removeDepth(target);
        self.glScene.remove(target);
        self.removeDepth(target);

        self.selectionBox.visible = false;

        //self.ee.trigger('remove_object', [target]);
        self.ee.trigger('unselected_object');
    }
};

Workspace.prototype.removeDepth = function(node) {
    if(!node.isMesh) {
        node.children.forEach(function(item) {
            node.remove(item);
            removeNode(node);
        });
    } else {
        removeNode(node);
    }
    function removeNode(node) {
        if (node instanceof THREE.Mesh) {
            if (node.geometry) {
                node.geometry.dispose();
                node.geometry = undefined; // fixed problem
            }

            if (node.material) {
                if (node.material instanceof THREE.MeshFaceMaterial || node.material instanceof THREE.MultiMaterial) {
                    node.material.materials.forEach( function(mtrl, idx) {
                        if (mtrl.map) mtrl.map.dispose();
                        if (mtrl.lightMap) mtrl.lightMap.dispose();
                        if (mtrl.bumpMap) mtrl.bumpMap.dispose();
                        if (mtrl.normalMap) mtrl.normalMap.dispose();
                        if (mtrl.specularMap) mtrl.specularMap.dispose();
                        if (mtrl.envMap) mtrl.envMap.dispose();

                        mtrl.dispose();
                        mtrl = undefined; // fixed problem
                    } );
                }
                else {
                    if (node.material.map) node.material.map.dispose();
                    if (node.material.lightMap) node.material.lightMap.dispose();
                    if (node.material.bumpMap) node.material.bumpMap.dispose();
                    if (node.material.normalMap) node.material.normalMap.dispose();
                    if (node.material.specularMap) node.material.specularMap.dispose();
                    if (node.material.envMap) node.material.envMap.dispose();

                    node.material.dispose();
                    node.material = undefined; // fixed problem
                }
            }
        }
        console.log('node before removal: ', node);
        return node;
    }
    //self.glScene.remove( node );
    //renderer.dispose(); // ***EDIT*** improved even memory more original scene heap is 12.4 MB; add objects increases to 116 MB or 250 MB (different models), clearing always brings down to 13.3 MB ... there still might be some artifacts.
    //node = undefined; // unnecessary
};
Workspace.prototype.createObject = function(data, callback) {
    var self = this;
    var type = data.type;
    var obj = null;

    switch (type) {
        case 'plane':
            self.addPlane(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'circle':
            self.addCircle(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'cube':
            self.addCube(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'text':
            self.addText(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'video':
            self.addVideo(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'audio':
            self.addAudio(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'sphere':
            self.addSphere(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'cylinder':
            self.addCylinder(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'pyramid':
            self.addPyramid(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'cone':
            self.addCone(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'tube':
            self.addTube(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'torus':
            self.addTorus(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case '360img':
            self.add360(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'fbx':
            self.addFBX(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'obj':
            self.addOBJ(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'pdf':
            self.addPDF(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'website':
            self.addWebsite(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'youtube':
            self.addYoutube(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'facebook':
            self.addFacebook(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'linkedin':
            self.addLinkedin(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'twitter':
            self.addTwitter(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'image':
            self.addImage(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'directional':
            self.addDirectionalLight(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'spot':
            self.addSpotLight(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'point':
            self.addPointLight(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'hemisphere':
            self.addHemisphereLight(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'ambient':
            self.addAmbientLight(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'gallery':
            self.addGallery(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'bubbling':
            self.addBubbling(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'spawning':
            self.addSpawning(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'fire_smoke':
            self.addFireSmoke(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'spark':
            self.addSpark(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'water':
            self.addWater(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'cloud_fog':
            self.addCloudFog(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'fire':
            self.addFire(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'flame':
            self.addFlame(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'snow':
            self.addSnow(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        case 'rain':
            self.addRain(data, function(result) {
                obj = result;
                return callback(obj);
            });
            break;
        default:
            return callback(null);
            break;
    }
    return;
    //return obj;
};
Workspace.prototype.setPosition = function(obj, data) {
    var conf = getObjConf(obj.userData.type);

    obj.position.x = data.x;
    obj.position.y = data.y;
    obj.position.z = data.z;
};
Workspace.prototype.setRotation = function(obj, data) {
    var conf = getObjConf(obj.userData.type);

    obj.rotation.x = data.x;
    obj.rotation.y = data.y;
    obj.rotation.z = data.z;
};
Workspace.prototype.setScale = function(obj, data) {
    var conf = getObjConf(obj.userData.type);

    obj.scale.x = data.x;
    obj.scale.y = data.y;
    obj.scale.z = data.z;
};
Workspace.prototype.setColor = function(obj, data) {
    var self = this;
    var conf = getObjConf(obj.userData.type);

    if(!obj.isMesh) {
        obj.traverse( function ( child ) {
            if(child.isMesh ) {
                if(conf.ctg == 'group_widget') {
                    if(child.name == 'group_bg') {
                        child.material.color.set(data);
                    }
                } else {
                    child.material.color.set(data);
                }
            }
        });
    } else {
        obj.material.color.set(data);
        obj.material.needsUpdate = true;
    }

    /*
    if(conf.ctg == 'model') {
        obj.traverse( function ( child ) {
            if(child.isMesh ) {
                child.material.color.set(data);
            }
        });
        return;
    }
    if(conf.ctg == 'group_widget') {
        obj.children.forEach(function(child) {
            if(child.isMesh) {
                if(child.name == 'group_bg') {
                    child.material.color.set(data);
                }
            }
        });
        return;
    }

    obj.material.color.set(data);
    obj.material.needsUpdate = true;
    */

};
Workspace.prototype.setOpacity = function(obj, data) {
    var conf = getObjConf(obj.userData.type);

    if(!obj.isMesh) {
        obj.traverse( function ( child ) {
            if(child.isMesh ) {
                child.material.opacity = data;
                //child.material.transparent = false;
            }
        });
    } else {
        obj.material.opacity = data;
    }
    /*
    if(conf.ctg == 'model' || conf.ctg == 'group_widget') {
        obj.traverse( function ( child ) {
            if(child.isMesh ) {
                child.material.opacity = data;
                // /child.material.transparent = false;
            }
        });
        return;
    }
    obj.material.opacity = data;
    */
};
Workspace.prototype.setTransparent = function(obj, data) {
    var conf = getObjConf(obj.userData.type);
    if(!obj.isMesh) {
        obj.traverse( function ( child ) {
            if(child.isMesh ) {
                child.material.transparent = data;
            }
        });
    } else {
        obj.material.transparent = data;
    }
    /*
    if(conf.ctg == 'model' || conf.ctg == 'group_widget') {
        obj.traverse( function ( child ) {
            if(child.isMesh ) {
                child.material.transparent = data;
            }
        });
        return;
    }
    obj.material.transparent = data;
    */
};
Workspace.prototype.setImageTexture = function(obj, data) {
    var self = this;
    var conf = getObjConf(obj.userData.type);

    self.imageTexture(data.texture.url, function (err, result) {
        if (err) {
            console.log(err);
        } else {
            var texture = result;

            if(conf.ctg == 'model') {
                obj.traverse( function ( child ) {
                    if(child.isMesh ) {
                        if(!child.material) {
                            console.warn('Removed object.', child);
                            return;
                        }
                        child.material.map = texture;
                        child.material.map.needsUpdate = true;
                        child.material.needsUpdate = true;
                    }
                });
                return;
            }

            if(!obj.material) {
                console.warn('Removed object.', obj);
                return;
            }

            obj.material.map = texture;
            obj.material.map.needsUpdate = true;
            obj.material.needsUpdate = true;
        }
    });
};
Workspace.prototype.unsetImageTexture = function(obj, data) {
    var self = this;
    var conf = getObjConf(obj.userData.type);

    if(conf.ctg == 'model') {
        obj.traverse( function ( child ) {
            if(child.isMesh ) {
                var material = child.material;
                if(material.map) {
                    material.map.dispose();
                    material.map.needsUpdate = true;
                }
                material.map = null;
                material.needsUpdate = true;
            }
        });
        return;
    }

    var material = obj.material;
    if(material.map) {
        material.map.dispose();
        material.map.needsUpdate = true;
    }
    material.map = null;
    material.needsUpdate = true;
};
Workspace.prototype.setVideoResource = function(obj, data) {
    var self = this;
    var uid = data.uid;
    var url = data.resource.url;
    var autoplay = data.resource.autoplay? true : false;
    var loop = data.resource.loop? true : false;
    self.videoTexture(uid, url, autoplay, loop, function(err, result) {
        if(err) {
            console.warn(err);
        } else {
            var map = result;
            if(!obj.material) {
                console.warn('Removed object.', obj);
                return;
            }
            obj.material.map = map;
            obj.material.map.needsUpdate = true;
            obj.material.needsUpdate = true;
        }
    });
};
Workspace.prototype.setGalleryResource = function(obj, data) {
    var self = this;
    var urls = Array.isArray(data.resource.urls)? [].concat(data.resource.urls) : [];

    clearTimeout(obj.autoplay_timer);
    clearTimeout(obj.zoom_timer);

    for(var i = obj.children.length - 1; i >= 0; i--) {
        obj.remove(obj.children[i]);
    }

    timeout(obj);

    if(data.resource.autoplay) {
        autoplay();
    }
    function autoplay() {
        obj.onclick(1); // + value: left->right, - value: right->left
        obj.autoplay_timer = setTimeout(autoplay, 4000);
    }

    function timeout(obj) {
        if(obj.children.length !== 0) {
            console.log(obj.children.length);
            setTimeout(function() { timeout(obj); }, 50);
            return;
        }

        obj.curr = 0;
        var count = urls.length - 1;
        var curr = obj.curr;
        var next = (curr+1) > count? 0 : curr+1;
        var prev = (curr-1) < 0? count : curr-1;
        /*
        urls.forEach(function(item, index) {
            obj.createImage(item, index, function(result) {
                if(result) {
                    obj.add(result);
                }
            });
        });
        */

        var index = -1;
        var item = null;
        function looper() {
            if(!(urls.length > 0)) {
                return;
            }

            item = urls.shift();
            index++;
            obj.createImage(item, index, function(result) {
                if(result) {
                    obj.add(result);

                    looper();
                }
            });
        }
        looper();
    }
};
Workspace.prototype.setImageResource = function(obj, data) {
    var self = this;
    var conf = getObjConf(obj.userData.type);

    self.imageTexture(data.resource.url, function (err, result) {
        if (err) {
            console.log(err);
        } else {
            var texture = result;
            if(conf.ctg == 'model') {
                obj.traverse( function ( child ) {
                    if(child.isMesh ) {
                        if(!child.material) {
                            console.warn('Removed object.', obj);
                            return;
                        }
                        child.material.map = texture;
                        child.material.map.needsUpdate = true;
                        child.material.needsUpdate = true;
                    }
                });
                return;
            }
            if(!obj.material) {
                console.warn('Removed object.', obj);
                return;
            }
            obj.material.map = texture;
            obj.material.map.needsUpdate = true;
            obj.material.needsUpdate = true;
        }
    });
};
Workspace.prototype.setTextResource = function(obj, data) {
    var self = this;
    var conf = getObjConf(data.type);
    var fontColor = conf.type == 'text'? (data.color? data.color : '#000000') : (conf.fontColor? conf.fontColor : '#000000');

    var t = self.textTexture(data.resource.title, fontColor);
    var texture = t.texture;
    var size = t.size;
    var x = 10*(size.width/size.height);
    var y = 10;
    var geometry = new THREE.PlaneGeometry(x, y);
    obj.geometry = geometry;

    var map = texture;
    obj.material.map = map;
    obj.material.map.needsUpdate = true;
    obj.material.needsUpdate = true;

    return {w: x, h: y};
};
Workspace.prototype.unsetResource = function(obj, data) {
    var self = this;
    var conf = getObjConf(obj.userData.type);
    if(!obj.isMesh) {
        obj.traverse( function ( child ) {
            if(child.isMesh ) {
                var material = child.material;
                if(material.map) {
                    material.map.dispose();
                    material.map.needsUpdate = true;
                }
                material.map = null;
                material.needsUpdate = true;
            }
        });
    } else {
        var material = obj.material;
        if(material.map) {
            material.map.dispose();
            material.map.needsUpdate = true;
        }
        material.map = null;
        material.needsUpdate = true;
    }

    self.removeHTMLResource(data.uid);

    /*
    if(conf.ctg == 'model') {
        obj.traverse( function ( child ) {
            if(child.isMesh ) {
                var material = child.material;
                if(material.map) {
                    material.map.dispose();
                    material.map.needsUpdate = true;
                }
                material.map = null;
                material.needsUpdate = true;
            }
        });
        return;
    }

    var material = obj.material;
    if(material.map) {
        material.map.dispose();
        material.map.needsUpdate = true;
    }
    material.map = null;
    material.needsUpdate = true;

    self.removeHTMLResource(data.uid);
    */
};
Workspace.prototype.setGroupWidgetResource = function(obj, data) {
    var self = this;
    var titleObj = null;
    var iconObj = null;
    var bgObj = null;
    obj.traverse(function(child) {
        if(child.isMesh && child.name == 'title') {
            titleObj = child;
        }
        if(child.isMesh && child.name == 'icon') {
            iconObj = child;
        }
        if(child.isMesh && child.name == 'group_bg') {
            bgObj = child;
        }
    });

    if(!(bgObj && iconObj && titleObj)) {
        console.warn('Invalid group widget: ', bgObj, iconObj, titleObj);
        return;
    }


    var size = self.setTextResource(titleObj, data);
    var base_w = 10;
    var w = Number(size.w);
    var h = Number(size.h);

    var totalWidth = w + base_w;// + 2; // 4 is between space
    iconObj.position.x = -(totalWidth/2 - (base_w/2));//+1;
    titleObj.position.x = (totalWidth)/2 - w/2;//-1;

    bgObj.scale.x = totalWidth/(bgObj.geometry.parameters.width);

    if(data.type == 'audio' && data.resource && data.resource.hasOwnProperty('url')) {
        obj.onChangeResource(data);
    }


    titleObj.renderOrder = Infinity;
    iconObj.renderOrder = Infinity;
    bgObj.renderOrder = 1;
    //titleObj.material.depthTest = false;
    //iconObj.material.depthTest = false;
    bgObj.material.depthWrite = false;
};

Workspace.prototype.createAudioHTML = function(url, uid, autoplay, loop) {
    var self = this;
    self.removeHTMLResource(uid);
    var audio = document.createElement('audio');
    audio.src = url;
    audio.autoplay = autoplay;
    audio.controls = true;
    audio.loop = loop;
    audio.style.display = 'none';
    audio.id = 'audio_' + uid;
    document.body.appendChild(audio);
    return audio;
};
Workspace.prototype.createVideoHTML = function(url, uid, autoplay, loop) {
    var self = this;
    self.removeHTMLResource(uid);
    var video = document.createElement('video');
    video.src = url;
    video.autoplay = autoplay;
    video.controls = true;
    video.loop = loop;
    video.style.display = 'none';
    video.id = 'video_' + uid;
    video.setAttribute( 'webkit-playsinline', 'webkit-playsinline' );
    document.body.appendChild(video);
    return video;
};
Workspace.prototype.setVisible = function(obj, data) {
    var self = this;
    obj.visible = data;
};
Workspace.prototype.setShadow = function(obj, data) {
    obj.castShadow = false;
    obj.receiveShadow = false;

    if(data && data.cast) {
        obj.castShadow = true;
    }
    if(data && data.receive) {
        obj.receiveShadow = true;
    }

    if(!obj.isMesh) {
        obj.traverse(function (child) {
            if(child.isMesh) {
                child.castShadow = false;
                child.receiveShadow = false;

                if(data && data.cast) {
                    child.castShadow = true;
                }
                if(data && data.receive) {
                    child.receiveShadow = true;
                }
            }
        });
    }
};

Workspace.prototype.setData = function(obj, data) {
    var self = this;
    var mesh = obj;
    mesh.userData = data;

    //mesh.renderOrder = -1;
    //mesh.renderOrder = 999;
    //mesh.onBeforeRender = function(  ) { self.glRenderer.clearDepth(); };
    var objConf = getObjConf(data.type);

    self.setVisible(obj, data.visible);

    self.setShadow(obj, data.shadow);

    if(data.color) {
        self.setColor(obj, data.color);
    }

    if(data.position) {
        self.setPosition(obj, data.position);
    }
    if(data.rotation) {
        self.setRotation(obj, data.rotation);
    }
    if(data.scale) {
        self.setScale(obj, data.scale);
    }

    if(typeof data.opacity === 'number') {
        self.setOpacity(obj, data.opacity);
    }
    if(data.hasOwnProperty('transparent')) {
        self.setTransparent(obj, data.transparent);
    }

    if(objConf.ctg=='element') {
        if(data.texture && data.texture.hasOwnProperty('url')) {
            self.setImageTexture(obj, data);

        } else {
            self.unsetImageTexture(obj, data);
        }
    }

    if(objConf.ctg=='widget') {
        if(data.resource) {
            if(data.type == 'video') {
                self.setVideoResource(obj, data);
            }
            if(data.type == '360img') {
                self.setImageResource(obj, data);
            }
            if(data.type == 'gallery') {
                self.setGalleryResource(obj, data);
            }
        } else {
            self.unsetResource(obj, data);
        }
    }

    if(objConf.ctg=='group_widget') {
        if(data.resource) {
            self.setGroupWidgetResource(obj, data);
        }
    }

    if(objConf.ctg=='misc') {
        if(data.type == 'text') {
            self.setTextResource(obj, data);
        }
        if(data.type == 'image') {
            self.setImageResource(obj, data);
        }
    }

    if(objConf.ctg == 'model') {
        if(data.texture && data.texture.hasOwnProperty('url')) {
            self.setImageTexture(obj, data);
        } else {
            self.unsetImageTexture(obj, data);
        }
        if(data.resource && data.resource.hasOwnProperty('url')) {

            if(objConf.type == 'fbx') {
                self.setAnimation(obj, data);
            }
        }
    }

    self.fireObjectChange(self.getObjectData(mesh));
    //return mesh;
};
Workspace.prototype.removeHTMLResource = function(uid, resourceType) {
    var video = 'video_' + uid;
    var audio = 'audio_' + uid;

    if(objById(video)) {
        objById(video).remove();
    }
    if(objById(audio)) {
        objById(audio).remove();
    }
};
Workspace.prototype.imageTexture = function(url, callback) {
    if(!url) {
        return null;
    }
    var loader = new THREE.TextureLoader();
    loader.crossOrigin = '';
    loader.load(
        // resource URL
        url,
        // Function when resource is loaded
        function ( texture ) {
            texture.needsUpdate = true;
            if(callback) {
                return callback(null, texture);
            }
        },
        // Function called when download progresses
        function ( xhr ) {
            console.log( (xhr.loaded / xhr.total * 100) + '% loaded' );
        },
        // Function called when download errors
        function ( xhr ) {
            console.log( 'An error happened' );
            return callback('An error happened');
        }
    );
};
Workspace.prototype.decideMaterial = function(data, options) {
    var options = options? options : {};
    var material = null;
    var material_type = data.material;
    switch (material_type) {
        case 'basic':
            material = new THREE.MeshBasicMaterial(options);
            break;
        case 'lambert':
            material = new THREE.MeshLambertMaterial(options);
            break;
        case 'standard':
            material = new THREE.MeshStandardMaterial(options);
            break;
        case 'phong':
            material = new THREE.MeshPhongMaterial(options);
            break;
        case 'shadow':
            material = new THREE.ShadowMaterial(options);
            break;
    }
    return material;
};
Workspace.prototype.addCube = function(data, callback) {
    var self = this;
    var geometry = new THREE.BoxGeometry(10, 10, 10);
    var material = self.decideMaterial(data);
    if(!material) {
        return callback(null);
    }
    var mesh = new THREE.Mesh(geometry, material);
    self.setData(mesh, data);

    self.glScene.add(mesh); // 씬에 메시 추가
    self.addObject(mesh);
    mesh.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.link && objData.link.hasOwnProperty('url')) {
            window.open(objData.link.url, '_blank');
        }
    };
    return callback(mesh);
};
Workspace.prototype.addImage = function(data, callback) {
    var self = this;

    var image = new Image();
    image.src= data.resource.url;
    image.onload = function() {
        var geometry = new THREE.PlaneGeometry(image.width/10, image.height/10);
        //var material = new THREE.MeshLambertMaterial( {side: THREE.DoubleSide} );
        var material = self.decideMaterial(data, {side: THREE.DoubleSide});
        if(!material) {
            return callback(null);
        }
        var mesh = new THREE.Mesh( geometry, material );

        self.setData(mesh, data);
        self.glScene.add(mesh);
        self.addObject(mesh);

        mesh.ondblclick = function() {
            var obj = self.getObjectByUID(data.uid);
            var objData = obj.userData;
            if(objData.link && objData.link.hasOwnProperty('url')) {
                window.open(objData.link.url, '_blank');
            }
        };
        return callback(mesh);
    };


};
Workspace.prototype.addChromakeyVideo = function(data, callback) {
    var self = this;

    var THREEx	= {};
    THREEx.ChromaKeyMaterial = function (url, keyColor) {
        console.log(this);
        THREE.ShaderMaterial.call(this);

        var video = null;
        video = document.createElement('video');
        video.src = url;
        video.load();
        video.autoplay = true;

        var keyColorObject = new THREE.Color(keyColor);

        var videoTexture = new THREE.VideoTexture(video);
        videoTexture.minFilter = THREE.LinearFilter;
        videoTexture.magFilter = THREE.LinearFilter;

        this.startVideo = function() {
            video.play();
        };

        this.stopVideo = function() {
            video.pause();
            video.src = "";
        };

        this.update = function () {
            if (video.readyState === video.HAVE_ENOUGH_DATA) {
                // videoImageContext.drawImage(video, 0, 0);
                if (videoTexture) {
                    videoTexture.needsUpdate = true;
                }
            }
        };

        this.setValues({

            uniforms: {
                texture: {
                    type: "t",
                    value: videoTexture
                },
                color: {
                    type: "c",
                    value: keyColorObject
                }
            },
            vertexShader:
                "varying vec2 vUv;\n" +
                "void main()\n" +
                "\t{\n" +
                "\t\tvUv = uv;\n" +
                "\t\tvec4 mvPosition = modelViewMatrix * vec4( position, 1.0 );\n" +
                "\t\tgl_Position = projectionMatrix * mvPosition;\n" +
                "\t}",
            fragmentShader:
                "uniform sampler2D texture;\n" +
                "\tuniform vec3 color;\n" +
                "\tvarying vec2 vUv;\n" +
                "\tvoid main()\n" +
                "\t{\n" +
                "\t\tvec3 tColor = texture2D( texture, vUv ).rgb;\n" +
                "\t\tfloat a = (length(tColor - color) - 0.5) * 7.0;\n" +
                "\t\t\n" +
                "\t\tgl_FragColor = vec4(a, a, a, 1.0);\n" +
                "\t\tgl_FragColor = vec4(tColor, a);\n" +
                "\t}",
            transparent: true
        });
    };

    THREEx.ChromaKeyMaterial.prototype = Object.create(THREE.ShaderMaterial.prototype);

    //0xd432 is the green screen color, insert yours, if different, below
    var myGreenScreenMaterial = new THREEx.ChromaKeyMaterial("./upload/videos/66/chormakey4.webm", 0xff00);
    console.log(myGreenScreenMaterial);
    myGreenScreenMaterial.needsUpdate = true;
    myGreenScreenMaterial.uniformsNeedsUpdate = true;
    var myGeometry = new THREE.PlaneBufferGeometry( 30, 20);
    var myGreenScreenVideoObject = new THREE.Mesh( myGeometry, myGreenScreenMaterial );
    self.glScene.add( myGreenScreenVideoObject );

    myGreenScreenMaterial.update();
    //myGreenScreenMaterial.startVideo();

};
Workspace.prototype.addVideo = function(data, callback) {
    var self = this;
    //var texture = self.videoTexture(data);

    //var material = new THREE.MeshLambertMaterial({ side: THREE.DoubleSide });
    var material = self.decideMaterial(data, {side: THREE.DoubleSide});
    if(!material) {
        return callback(null);
    }
    material.transparent = true;
    var geometry = new THREE.PlaneGeometry( 50, 30 );
    var mesh = new THREE.Mesh(geometry, material);

    mesh.ondblclick = function() {
        var ele = objById('video_' + data.uid);
        if(ele) {
            if(ele.paused) {
                ele.play();
            } else {
                ele.pause();
            }
        }
    };

    self.setData(mesh, data);
    self.glScene.add(mesh);
    self.addObject(mesh);

    return callback(mesh);
};
Workspace.prototype.addAudio = function(data, callback) {
    var self = this;
    var conf = getObjConf(data.type);

    var w = 10;
    var h = 10;

    var iconGeometry = new THREE.PlaneGeometry(w, h);
    //var iconMaterial = new THREE.MeshBasicMaterial({ side: THREE.DoubleSide });
    var iconMaterial = self.decideMaterial(data, {side: THREE.DoubleSide});
    if(!iconMaterial) {
        return callback(null);
    }
    iconMaterial.transparent = true;

    var iconMesh = new THREE.Mesh(iconGeometry, iconMaterial);
    iconMesh.name = 'icon';
    var iconURL = data.resource.autoplay? './assets/images/stop.png' : './assets/images/play.png';
    self.imageTexture(iconURL, function(err, result) {
        if(err) {
            console.warn(err);
            return;
        }

        var map = result;
        iconMesh.material.map = map;
        iconMesh.material.map.needsUpdate = true;
        iconMesh.material.needsUpdate = true;
    });


    var t = self.textTexture(data.resource.title);
    var texture = t.texture;
    var size = t.size;
    var x = w*(size.width/size.height);
    var y = h;

    //var titleMaterial = new THREE.MeshBasicMaterial({ map: texture, side: THREE.DoubleSide });
    var titleMaterial = self.decideMaterial(data, { map: texture, side: THREE.DoubleSide });
    if(!titleMaterial) {
        return callback(null);
    }
    titleMaterial.transparent = true;
    titleMaterial.color.set('#000000');
    var titleGeometry = new THREE.PlaneGeometry(x, y);
    var titleMesh = new THREE.Mesh(titleGeometry, titleMaterial);
    titleMesh.name = 'title';


    var totalWidth = x + w + 2; // 4 is between space
    iconMesh.position.x = -(totalWidth/2 - (w/2))+1;
    titleMesh.position.x = (totalWidth)/2 - x/2-1;


    var bgGeometry = new THREE.PlaneGeometry(totalWidth, h);
    //var bgMaterial = new THREE.MeshBasicMaterial({ side: THREE.DoubleSide });
    var bgMaterial = self.decideMaterial(data, { side: THREE.DoubleSide });
    if(!bgMaterial) {
        return callback(null);
    }
    bgMaterial.transparent = true;
    //bgMaterial.depthTest = false; // Always behind order
    bgMaterial.color.set(data.color);
    var bgMesh = new THREE.Mesh(bgGeometry, bgMaterial);
    bgMesh.name = 'group_bg';

    var group = new THREE.Group();
    group.add(bgMesh);
    group.add(titleMesh);
    group.add(iconMesh);

    group.ondblclick = function() {
        var iconURL = './assets/images/audio.png';
        var ele = objById('audio_' + data.uid);
        if(ele) {
            if(ele.paused) {
                ele.play();
                iconURL = './assets/images/stop.png';
            } else {
                ele.pause();
                iconURL = './assets/images/play.png';
            }

            self.imageTexture(iconURL, function(err, result) {
                if(err) {
                    console.warn(err);
                    return;
                }

                var map = result;
                iconMesh.material.map = map;
                iconMesh.material.map.needsUpdate = true;
                iconMesh.material.needsUpdate = true;
            });
        }
    };

    group.onChangeResource = function(data) {
        //var data = self.getObjectByUID(data.uid).userData;
        var autoplay = data.resource.autoplay? true : false;
        var loop = data.resource.loop? true : false;
        var audio = self.createAudioHTML(data.resource.url, data.uid, autoplay, loop);
        function changeAudioIcon() {
            var iconURL = './assets/images/audio.png';
            var ele = objById('audio_' + data.uid);
            if(ele) {
                if (ele.paused) {
                    iconURL = './assets/images/play.png';
                } else {
                    iconURL = './assets/images/stop.png';
                }

                self.imageTexture(iconURL, function(err, result) {
                    if(err) {
                        console.warn(err);
                        return;
                    }

                    var map = result;
                    iconMesh.material.map = map;
                    iconMesh.material.map.needsUpdate = true;
                    iconMesh.material.needsUpdate = true;
                });
            }
        }
        audio.onended = function(e) {
            changeAudioIcon();
        };
        audio.onplay = function(e) {
            changeAudioIcon();
        };
    };

    self.setData(group, data);
    self.glScene.add(group);
    self.addObject(group);
    //titleMesh.renderOrder = 5;
    //iconMesh.renderOrder = 10;

    return callback(group);
};
Workspace.prototype.videoTexture = function(uid, url, autoplay, loop, callback) {
    var self = this;
    if(!(uid && url)) {
        return null;
    }

    var video = self.createVideoHTML(url, uid, autoplay, loop);

    video.oncanplay = function() {
        if(callback) {
            var texture = new THREE.VideoTexture( video );
            texture.minFilter = THREE.LinearFilter;
            texture.magFilter = THREE.LinearFilter;
            texture.format = THREE.RGBFormat;
            texture.generateMipmaps = false;
            texture.needsUpdate = true;

            return callback(null, texture);
        }
    };
};
Workspace.prototype.addText = function(data, callback) {
    var conf = getObjConf(data.type);
    var self = this;
    /*
    var title = data.resource.title;
    var color = data.color;
    var loader = new THREE.FontLoader();
    loader.load( './assets/threejs/fonts/helvetiker_regular.typeface.json', function ( font ) {
        var material = new THREE.MeshBasicMaterial( {
            color: color,
            transparent: true,
            opacity: 1,
            side: THREE.DoubleSide
        } );
        var shapes = font.generateShapes( title, 5);
        var geometry = new THREE.ShapeBufferGeometry( shapes );
        geometry.computeBoundingBox();
        var xMid = - 0.5 * ( geometry.boundingBox.max.x - geometry.boundingBox.min.x );
        geometry.translate( xMid, 0, 0 );
        // make shape ( N.B. edge view not visible )
        var mesh = new THREE.Mesh( geometry, material );
        self.glScene.add( mesh );
        self.addObject(mesh);
    });
    */

    var t = self.textTexture(data.resource.title);
    var texture = t.texture;
    var size = t.size;
    var x = 10*(size.width/size.height);
    var y = 10;

    //var material = new THREE.MeshBasicMaterial({ map: texture, side: THREE.DoubleSide });
    var material = self.decideMaterial(data, { map: texture, side: THREE.DoubleSide });
    if(!material) {
        return callback(null);
    }
    material.transparent = true;
    //material.depthWrite = false;

    var geometry = new THREE.PlaneGeometry(x, y);

    var mesh = new THREE.Mesh(geometry, material);
    mesh.renderOrder = Infinity;

    self.setData(mesh, data);

    self.glScene.add(mesh);

    self.addObject(mesh);
    mesh.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.link && objData.link.hasOwnProperty('url')) {
            window.open(objData.link.url, '_blank');
        }
    };
    return callback(mesh);
};
Workspace.prototype.textTexture = function(text, color) {

    var fontSize = 30;
    var lineHeight = 4 * fontSize / 3;

    var sz = measureTextMultiLine(stringCleanup(text), fontSize, lineHeight);

    var canvas = document.createElement('canvas');
    canvas.width = sz.width>0? sz.width : 1;
    canvas.height = sz.height;

    var context = canvas.getContext('2d');
    context.font = "Normal "+fontSize+"px Arial";
    //align text horizontally center
    context.textAlign = "center";
    //align text vertically center
    context.textBaseline = "middle";

    context.fillStyle = color? color : "#000000";

    //roundRect(context, 0, 0, sz.width, sz.height, 0);
    //fillTextMultiLine(context, stringCleanup(data.text), lineHeight, rect[0], rect[1] + 3 * lineHeight / 4);

    context.fillText(text, canvas.width/2, canvas.height/2);

    // canvas contents will be used for a texture
    var texture = new THREE.Texture(canvas);
    texture.needsUpdate = true;

    return {texture: texture, size: sz};
};
Workspace.prototype.addSphere = function(data, callback) {
    var self = this;

    var geometry = new THREE.SphereGeometry( 5, 64, 64 );
    //var material = new THREE.MeshLambertMaterial( );
    var material = self.decideMaterial(data);
    if(!material) {
        return callback(null);
    }
    var mesh = new THREE.Mesh( geometry, material );
    self.setData(mesh, data);

    self.glScene.add(mesh);
    self.addObject(mesh);
    mesh.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.link && objData.link.hasOwnProperty('url')) {
            window.open(objData.link.url, '_blank');
        }
    };
    return callback(mesh);
};
Workspace.prototype.addCylinder = function(data, callback) {
    var self = this;

    var geometry = new THREE.CylinderGeometry( 5, 5, 10, 32 );
    //var material = new THREE.MeshLambertMaterial( );
    var material = self.decideMaterial(data);
    if(!material) {
        return callback(null);
    }
    var mesh = new THREE.Mesh( geometry, material );
    self.setData(mesh, data);

    self.glScene.add(mesh);
    self.addObject(mesh);
    mesh.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.link && objData.link.hasOwnProperty('url')) {
            window.open(objData.link.url, '_blank');
        }
    };
    return callback(mesh);
};
Workspace.prototype.addPyramid = function(data, callback) {
    var self = this;

    var geometry = new THREE.CylinderGeometry( 5, 10, 10, 4 );
    //var material = new THREE.MeshLambertMaterial( );
    var material = self.decideMaterial(data);
    if(!material) {
        return callback(null);
    }
    var mesh = new THREE.Mesh( geometry, material );
    self.setData(mesh, data);

    self.glScene.add(mesh);
    self.addObject(mesh);
    mesh.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.link && objData.link.hasOwnProperty('url')) {
            window.open(objData.link.url, '_blank');
        }
    };
    return callback(mesh);
};
Workspace.prototype.addCone = function(data, callback) {
    var self = this;

    var geometry = new THREE.ConeGeometry( 5, 10, 32 );
    //var material = new THREE.MeshLambertMaterial( );
    var material = self.decideMaterial(data);
    if(!material) {
        return callback(null);
    }
    var mesh = new THREE.Mesh( geometry, material );
    self.setData(mesh, data);

    self.glScene.add(mesh);
    self.addObject(mesh);
    mesh.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.link && objData.link.hasOwnProperty('url')) {
            window.open(objData.link.url, '_blank');
        }
    };
    return callback(mesh);
};
Workspace.prototype.addTube = function(data, callback) {
    var self = this;

    function CustomSinCurve( scale ) {

        THREE.Curve.call( this );

        this.scale = ( scale === undefined ) ? 1 : scale;

    }

    CustomSinCurve.prototype = Object.create( THREE.Curve.prototype );
    CustomSinCurve.prototype.constructor = CustomSinCurve;

    CustomSinCurve.prototype.getPoint = function ( t ) {

        var tx = t * 3 - 1.5;
        var ty = Math.sin( 2 * Math.PI * t );
        var tz = 0;

        return new THREE.Vector3( tx, ty, tz ).multiplyScalar( this.scale );

    };

    var path = new CustomSinCurve( 10 );
    var geometry = new THREE.TubeGeometry( path, 64, 2.5, 32, false );
    //var material = new THREE.MeshLambertMaterial( );
    var material = self.decideMaterial(data);
    if(!material) {
        return callback(null);
    }
    var mesh = new THREE.Mesh( geometry, material );
    self.setData(mesh, data);

    self.glScene.add(mesh);
    self.addObject(mesh);
    mesh.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.link && objData.link.hasOwnProperty('url')) {
            window.open(objData.link.url, '_blank');
        }
    };
    return callback(mesh);
};
Workspace.prototype.addTorus = function(data, callback) {
    var self = this;

    var geometry = new THREE.TorusGeometry( 5, 2, 16, 100 );
    //var material = new THREE.MeshLambertMaterial( );
    var material = self.decideMaterial(data);
    if(!material) {
        return callback(null);
    }
    var mesh = new THREE.Mesh( geometry, material );
    self.setData(mesh, data);

    self.glScene.add(mesh);
    self.addObject(mesh);
    mesh.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.link && objData.link.hasOwnProperty('url')) {
            window.open(objData.link.url, '_blank');
        }
    };
    return callback(mesh);
};
Workspace.prototype.addPlane = function(data, callback) {
    var self = this;

    var geometry = new THREE.PlaneGeometry( 10, 10, 10 );
    //var material = new THREE.MeshLambertMaterial( {side: THREE.DoubleSide} );
    var material = self.decideMaterial(data, {side: THREE.DoubleSide});
    if(!material) {
        return callback(null);
    }
    var mesh = new THREE.Mesh( geometry, material );
    self.setData(mesh, data);
    mesh.receiveShadow = true;

    self.glScene.add(mesh);
    self.addObject(mesh);
    mesh.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.link && objData.link.hasOwnProperty('url')) {
            window.open(objData.link.url, '_blank');
        }
    };
    return callback(mesh);
};
Workspace.prototype.addCircle = function(data, callback) {
    var self = this;

    var geometry = new THREE.CircleGeometry( 5, 32 );
    //var material = new THREE.MeshLambertMaterial( {side: THREE.DoubleSide} );
    var material = self.decideMaterial(data, {side: THREE.DoubleSide});
    if(!material) {
        return callback(null);
    }
    var mesh = new THREE.Mesh( geometry, material );
    self.setData(mesh, data);

    self.glScene.add(mesh);
    self.addObject(mesh);
    mesh.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.link && objData.link.hasOwnProperty('url')) {
            window.open(objData.link.url, '_blank');
        }
    };
    return callback(mesh);
};
Workspace.prototype.add360 = function(data, callback) {
    var self = this;
    /**
     * Sphere 360
     *
     */

    // creation of a big sphere geometry
    var geometry = new THREE.SphereGeometry(20, 64, 32);
    geometry.applyMatrix(new THREE.Matrix4().makeScale(-1, 1, 1));

    // creation of the sphere material
    //var material = new THREE.MeshBasicMaterial();
    var material = self.decideMaterial(data);
    if(!material) {
        return callback(null);
    }
    material.transparent = true;
    //material.map = new THREE.TextureLoader().load(panoramasArray[panoramaNumber]);

    // geometry + material = mesh (actual object)
    var mesh = new THREE.Mesh(geometry, material);
    self.setData(mesh, data);

    self.glScene.add(mesh);
    self.addObject(mesh);

    mesh.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.link && objData.link.hasOwnProperty('url')) {
            window.open(objData.link.url, '_blank');
        }
    };

    return callback(mesh);
};
Workspace.prototype.addModel = function(data, callback) {
    var self = this;
    var url = data.resource.url;
    var fileExtension = getFileExtension(url);

    if(fileExtension == 'obj') {
        self.addOBJ(data, function(result) {
            return callback(result);
        });
    }

    if(fileExtension == 'fbx') {
        self.addFBX(data, function(result) {
            return callback(result);
        });
    }

};
Workspace.prototype.getAnimationDuration = function(uid) {
    var self = this;
    var obj = self.getObjectByUID(uid);
    try{
        return obj.animations[ 0 ].duration;
    } catch (e) {
        return 0;
    }
};
Workspace.prototype.getFirstAction = function(obj) {
    var self = this;
    var action = null;
    if(obj && Array.isArray(obj.animations) && obj.animations.length > 0) {
        var animation = obj.animations[ 0 ];
        var totalTime = animation.duration;
        action = {name: animation.name, start: 0, end: totalTime};
    }
    return action;
};
Workspace.prototype.setAnimation = function(obj, data) {
    var self = this;
    var resource = data.resource;
    if(resource && Array.isArray(obj.animations) && obj.animations.length > 0) {
        self.removeAnimationByUID(data.uid);

        var animation = obj.animations[ 0 ];
        var totalTime = animation.duration;

        var mixer = new THREE.AnimationMixer( obj );
        var action = mixer.clipAction( animation );

        var actionList = Array.isArray(resource.actionList)? resource.actionList : [];
        if(actionList.length === 0) {
            actionList.push(
                {name: animation.name, start: 0, end: totalTime}
            );
        }

        data.resource.actionList = actionList;

        var ani = {
            uid: data.uid,
            object: obj,
            mixer: mixer,
            action: action,
            loop: resource.loop? THREE.LoopRepeat : THREE.LoopOnce,
            loopIndex: 0,
            actionList: actionList,
            actionIndex: 0,
        };

        ani.action.play();
        ani.action.time = ani.actionList[0].start;
        ani.actionIndex = 0;

        ani.remove = function() {
            //this.mixer.stopAllAction();
            this.mixer = null;
            this.action = null;
            Object.keys(this).forEach(function (key) {
                delete this[key];
            });
            this.__proto__ = {};
        };

        ani.go = function(delta) {
            //var delta = self.getClock(); // Important!!! for multiple model animation....

            if(!this.mixer || !this.action) {
                return;
            }

            if(this.actionIndex >= this.actionList.length) {
                console.log('End action1: ', this.actionIndex);
                this.mixer = null;
                this.action = null;
                return;
            }

            var currAction = this.actionList[this.actionIndex];

            if(this.mixer.time + delta < currAction.end-currAction.start) {
                this.mixer.update(delta);
            } else {
                this.actionIndex++;

                if(this.actionIndex >= this.actionList.length) { // finish all action one time.
                    this.loopIndex++;
                    if(this.loop === THREE.LoopOnce && this.loopIndex == 1) {
                        //this.action.stop();
                        //this.mixer.stopAllAction();
                        this.mixer = null;
                        this.action = null;
                        return;
                    }

                    this.actionIndex = 0; // move to first action.
                }

                currAction = this.actionList[this.actionIndex];
                if(!currAction) {
                    console.log('End action: ', this.actionIndex);
                    return;
                }
                this.mixer = new THREE.AnimationMixer( this.object );
                this.action = this.mixer.clipAction( this.object.animations[ 0 ] );
                this.action.play();
                this.action.time = currAction.start;
            }
        };
        self.addAnimationList(ani);
    }
};

Workspace.prototype.previewModelOnSlide = function(uid, val) {
    var self = this;
    var obj = self.getObjectByUID(uid);
    if(!obj) { return; }

    self.removeAnimationByUID(uid);

    var animation = obj.animations[ 0 ];

    var mixer = new THREE.AnimationMixer( obj );
    var action = mixer.clipAction( animation );
    action.play();
    action.time = Number(val);
    mixer.update(self.getClock());
};

Workspace.prototype.addFBX = function(data, callback) {
var self = this;

var resource = data.resource;
var url = resource.url;

var loader = new THREE.FBXLoader();
loader.load( url, function ( object ) {
    self.setData(object, data);
    object.traverse(function(child) {
        if(child.isMesh ) {
            child.castShadow = true;
            child.receiveShadow = true;
        }
    });

    self.glScene.add( object );
    self.addObject(object);
    //object.scale.set(0.1, 0.1, 0.1);

    object.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.link && objData.link.hasOwnProperty('url')) {
            window.open(objData.link.url, '_blank');
        }
    };

    return callback(object);
} );
};
Workspace.prototype.addOBJ = function(data, callback) {
var self = this;
var url = data.resource.url;
var loader = new THREE.OBJLoader();
loader.load( url, function ( object ) {
    self.setData(object, data);
    object.traverse(function(child) {
        if(child.isMesh ) {
            child.castShadow = true;
            child.receiveShadow = true;
        }
    });
    self.glScene.add(object);
    self.addObject(object);

    object.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.link && objData.link.hasOwnProperty('url')) {
            window.open(objData.link.url, '_blank');
        }
    };

    return callback(object);
} );


/*
var onProgress = function ( xhr ) {
    if ( xhr.lengthComputable ) {
        var percentComplete = xhr.loaded / xhr.total * 100;
        console.log( Math.round( percentComplete, 2 ) + '% downloaded' );
    }
};
var onError = function (e) {
    console.warn(e);
};

THREE.Loader.Handlers.add( /\.dds$/i, new THREE.DDSLoader() );
new THREE.MTLLoader()
    .setPath( './upload/models/66/' )
    .load( 'female02.mtl', function ( materials ) {
        materials.preload();
        new THREE.OBJLoader()
            .setMaterials( materials )
            .setPath( './upload/models/66/' )
            .load( 'female02.obj', function ( object ) {
                //self.setData(object, data);
                self.glScene.add(object);
                self.addObject(object);

                return callback(object);
            }, onProgress, onError );
    } );
*/

};
Workspace.prototype.addPDF = function(data, callback) {
    var self = this;
    var conf = getObjConf(data.type);

    var w = 10;
    var h = 10;

    var bgColor = data.color? data.color : conf.color;

    var iconGeometry = new THREE.PlaneGeometry(w, h);
    //var iconMaterial = new THREE.MeshBasicMaterial({ side: THREE.DoubleSide });
    var iconMaterial = self.decideMaterial(data, { side: THREE.DoubleSide });
    if(!iconMaterial) {
        return callback(null);
    }
    iconMaterial.transparent = true;

    var iconMesh = new THREE.Mesh(iconGeometry, iconMaterial);
    iconMesh.name = 'icon';
    var iconURL = './assets/images/pdf.light.png';
    self.imageTexture(iconURL, function(err, result) {
        if(err) {
            console.warn(err);
            return;
        }

        var map = result;
        iconMesh.material.map = map;
        iconMesh.material.map.needsUpdate = true;
        iconMesh.material.needsUpdate = true;
    });


    var t = self.textTexture(data.resource.title);
    var texture = t.texture;
    var size = t.size;
    var x = w*(size.width/size.height);
    var y = h;

    //var titleMaterial = new THREE.MeshBasicMaterial({ map: texture, side: THREE.DoubleSide });
    var titleMaterial = self.decideMaterial(data, { map: texture, side: THREE.DoubleSide });
    if(!titleMaterial) {
        return callback(null);
    }
    titleMaterial.transparent = true;
    titleMaterial.color.set(conf.fontColor);
    var titleGeometry = new THREE.PlaneGeometry(x, y);
    var titleMesh = new THREE.Mesh(titleGeometry, titleMaterial);
    titleMesh.name = 'title';


    var totalWidth = x + w + 2; // 4 is between space
    iconMesh.position.x = -(totalWidth/2 - (w/2))+1;
    titleMesh.position.x = (totalWidth)/2 - x/2-1;


    var bgGeometry = new THREE.PlaneGeometry(totalWidth, h);
    //var bgMaterial = new THREE.MeshBasicMaterial({ side: THREE.DoubleSide });
    var bgMaterial = self.decideMaterial(data, { side: THREE.DoubleSide });
    if(!bgMaterial) {
        return callback(null);
    }
    bgMaterial.transparent = true;
    //bgMaterial.depthTest = false; // Always behind order
    bgMaterial.color.set(bgColor);
    var bgMesh = new THREE.Mesh(bgGeometry, bgMaterial);
    bgMesh.name = 'group_bg';

    var group = new THREE.Group();
    group.add(bgMesh);
    group.add(titleMesh);
    group.add(iconMesh);

    group.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.resource && objData.resource.hasOwnProperty('url')) {
            window.open(objData.resource.url, '_blank');
            /*
            var html = '<div id="PDFView" style="width: 100%; height: 100%;"></div>';
            var box = new Box({
                contentsType: 'html',
                boxContents: html,
                boxEffect: 'fade',
                isShowCloseBtn: true,
                boxWidth: 500,
                boxHeight: 500,
                isHideByClick: true});
            box.loadBox();
            PDFObject.embed(objData.resource.url, '#PDFView');
            */
        }
    };

    self.setData(group, data);
    self.glScene.add(group);
    self.addObject(group);
    //titleMesh.renderOrder = 5;
    //iconMesh.renderOrder = 10;

    return callback(group);
};
Workspace.prototype.addWebsite = function(data, callback) {
    var self = this;
    var conf = getObjConf(data.type);

    var w = 10;
    var h = 10;

    var iconGeometry = new THREE.PlaneGeometry(w, h);
    //var iconMaterial = new THREE.MeshBasicMaterial({ side: THREE.DoubleSide });
    var iconMaterial = self.decideMaterial(data, { side: THREE.DoubleSide });
    if(!iconMaterial) {
        return callback(null);
    }
    iconMaterial.transparent = true;

    var iconMesh = new THREE.Mesh(iconGeometry, iconMaterial);
    iconMesh.name = 'icon';
    var iconURL = './assets/images/website.light.png';
    self.imageTexture(iconURL, function(err, result) {
        if(err) {
            console.warn(err);
            return;
        }

        var map = result;
        iconMesh.material.map = map;
        iconMesh.material.map.needsUpdate = true;
        iconMesh.material.needsUpdate = true;
    });


    var t = self.textTexture(data.resource.title);
    var texture = t.texture;
    var size = t.size;
    var x = w*(size.width/size.height);
    var y = h;

    //var titleMaterial = new THREE.MeshBasicMaterial({ map: texture, side: THREE.DoubleSide });
    var titleMaterial = self.decideMaterial(data, { map: texture, side: THREE.DoubleSide });
    if(!titleMaterial) {
        return callback(null);
    }
    titleMaterial.transparent = true;
    titleMaterial.color.set(conf.fontColor);
    var titleGeometry = new THREE.PlaneGeometry(x, y);
    var titleMesh = new THREE.Mesh(titleGeometry, titleMaterial);
    titleMesh.name = 'title';


    var totalWidth = x + w + 2; // 4 is between space
    iconMesh.position.x = -(totalWidth/2 - (w/2))+1;
    titleMesh.position.x = (totalWidth)/2 - x/2-1;


    var bgGeometry = new THREE.PlaneGeometry(totalWidth, h);
    //var bgMaterial = new THREE.MeshBasicMaterial({ side: THREE.DoubleSide });
    var bgMaterial = self.decideMaterial(data, { side: THREE.DoubleSide });
    if(!bgMaterial) {
        return callback(null);
    }
    bgMaterial.transparent = true;
    //bgMaterial.depthTest = false; // Always behind order
    bgMaterial.color.set(data.color? data.color : conf.fontColor);
    var bgMesh = new THREE.Mesh(bgGeometry, bgMaterial);
    bgMesh.name = 'group_bg';

    var group = new THREE.Group();
    group.add(bgMesh);
    group.add(titleMesh);
    group.add(iconMesh);

    group.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.resource && objData.resource.hasOwnProperty('url')) {
            var url = objData.resource.url;
            window.open(url, '_blank');
        }
    };

    self.setData(group, data);
    self.glScene.add(group);
    self.addObject(group);
    //titleMesh.renderOrder = 5;
    //iconMesh.renderOrder = 10;

    return callback(group);
};
Workspace.prototype.addYoutube = function(data, callback) {
    var conf = getObjConf(data.type);
    var self = this;

    var w = 10;
    var h = 10;

    var iconGeometry = new THREE.PlaneGeometry(w, h);
    //var iconMaterial = new THREE.MeshBasicMaterial({ side: THREE.DoubleSide });
    var iconMaterial = self.decideMaterial(data, { side: THREE.DoubleSide });
    if(!iconMaterial) {
        return callback(null);
    }
    iconMaterial.transparent = true;

    var iconMesh = new THREE.Mesh(iconGeometry, iconMaterial);

    iconMesh.name = 'icon';
    var iconURL = './assets/images/youtube.light.png';
    self.imageTexture(iconURL, function(err, result) {
        if(err) {
            console.warn(err);
            return;
        }

        var map = result;
        iconMesh.material.map = map;
        iconMesh.material.map.needsUpdate = true;
        iconMesh.material.needsUpdate = true;
    });


    var t = self.textTexture(data.resource.title, conf.fontColor);
    var texture = t.texture;
    var size = t.size;
    var x = w*(size.width/size.height);
    var y = h;

    //var titleMaterial = new THREE.MeshBasicMaterial({ map: texture, side: THREE.DoubleSide });
    var titleMaterial = self.decideMaterial(data, { map: texture, side: THREE.DoubleSide });
    if(!titleMaterial) {
        return callback(null);
    }
    titleMaterial.transparent = true;
    titleMaterial.color.set(conf.fontColor);
    var titleGeometry = new THREE.PlaneGeometry(x, y);
    var titleMesh = new THREE.Mesh(titleGeometry, titleMaterial);
    titleMesh.name = 'title';


    var totalWidth = x + w + 2; // 4 is between space
    iconMesh.position.x = -(totalWidth/2 - (w/2))+1;
    titleMesh.position.x = (totalWidth)/2 - x/2-1;


    var bgGeometry = new THREE.PlaneGeometry(totalWidth, h);
    //var bgMaterial = new THREE.MeshBasicMaterial({ side: THREE.DoubleSide });
    var bgMaterial = self.decideMaterial(data, { side: THREE.DoubleSide });
    if(!bgMaterial) {
        return callback(null);
    }
    bgMaterial.transparent = true;
    //bgMaterial.depthTest = false; // Always behind order
    bgMaterial.color.set(data.color);

    var bgMesh = new THREE.Mesh(bgGeometry, bgMaterial);
    bgMesh.name = 'group_bg';

    var group = new THREE.Group();
    group.add(bgMesh);
    group.add(iconMesh);
    group.add(titleMesh);

    group.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.resource && objData.resource.hasOwnProperty('url')) {
            var u = objData.resource.url;
            var url = '';
            if(u.indexOf("/") != -1) {
                url = u;
            } else {
                url = "https://www.youtube.com/embed/" + u
                    + "?autoplay=1&autohide=1&border=0&wmode=opaque&enablejsapi=1";
            }
            var html = '<embed src="'+url+'" width="100%" height="100%">';

            var box = new Box({
                contentsType: 'html',
                boxContents: html,
                boxEffect: 'fade',
                isShowCloseBtn: false,
                isShowLoading: true,
                boxWidth: 500,
                boxHeight: 300,
                isHideByClick: true,
            });
            box.loadBox();
        }
    };

    self.setData(group, data);
    self.glScene.add(group);
    self.addObject(group);
    //titleMesh.renderOrder = 5;
    //iconMesh.renderOrder = 10;
    return callback(group);
};
Workspace.prototype.addFacebook = function(data, callback) {
    var conf = getObjConf(data.type);
    var self = this;

    var w = 10;
    var h = 10;

    var iconGeometry = new THREE.PlaneGeometry(w, h);
    //var iconMaterial = new THREE.MeshBasicMaterial({ side: THREE.DoubleSide });
    var iconMaterial = self.decideMaterial(data, { side: THREE.DoubleSide });
    if(!iconMaterial) {
        return callback(null);
    }
    iconMaterial.transparent = true;

    var iconMesh = new THREE.Mesh(iconGeometry, iconMaterial);
    iconMesh.name = 'icon';
    var iconURL = './assets/images/facebook.light.png';
    self.imageTexture(iconURL, function(err, result) {
        if(err) {
            console.warn(err);
            return;
        }

        var map = result;
        iconMesh.material.map = map;
        iconMesh.material.map.needsUpdate = true;
        iconMesh.material.needsUpdate = true;
    });


    var t = self.textTexture(data.resource.title, conf.fontColor);
    var texture = t.texture;
    var size = t.size;
    var x = w*(size.width/size.height);
    var y = h;

    //var titleMaterial = new THREE.MeshBasicMaterial({ map: texture, side: THREE.DoubleSide });
    var titleMaterial = self.decideMaterial(data, { map: texture, side: THREE.DoubleSide });
    if(!titleMaterial) {
        return callback(null);
    }
    titleMaterial.transparent = true;
    titleMaterial.color.set(conf.fontColor);
    var titleGeometry = new THREE.PlaneGeometry(x, y);
    var titleMesh = new THREE.Mesh(titleGeometry, titleMaterial);
    titleMesh.name = 'title';


    var totalWidth = x + w + 2; // 4 is between space
    iconMesh.position.x = -(totalWidth/2 - (w/2))+1;
    titleMesh.position.x = (totalWidth)/2 - x/2-1;


    var bgGeometry = new THREE.PlaneGeometry(totalWidth, h);
    //var bgMaterial = new THREE.MeshBasicMaterial({ side: THREE.DoubleSide });
    var bgMaterial = self.decideMaterial(data, { side: THREE.DoubleSide });
    if(!bgMaterial) {
        return callback(null);
    }
    bgMaterial.transparent = true;
    //bgMaterial.depthTest = false; // Always behind order
    bgMaterial.color.set(data.color);
    var bgMesh = new THREE.Mesh(bgGeometry, bgMaterial);
    bgMesh.name = 'group_bg';

    var group = new THREE.Group();
    group.add(bgMesh);
    group.add(titleMesh);
    group.add(iconMesh);

    group.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.resource && objData.resource.hasOwnProperty('url')) {
            var url = objData.resource.url;
            window.open(url, '_blank');
        }
    };

    self.setData(group, data);
    self.glScene.add(group);
    self.addObject(group);
    //titleMesh.renderOrder = 5;
    //iconMesh.renderOrder = 10;

    return callback(group);
};
Workspace.prototype.addLinkedin = function(data, callback) {
    var conf = getObjConf(data.type);
    var self = this;

    var w = 10;
    var h = 10;

    var iconGeometry = new THREE.PlaneGeometry(w, h);
    //var iconMaterial = new THREE.MeshBasicMaterial({ side: THREE.DoubleSide });
    var iconMaterial = self.decideMaterial(data, { side: THREE.DoubleSide });
    if(!iconMaterial) {
        return callback(null);
    }
    iconMaterial.transparent = true;

    var iconMesh = new THREE.Mesh(iconGeometry, iconMaterial);
    iconMesh.name = 'icon';
    var iconURL = './assets/images/linkedin.light.png';
    self.imageTexture(iconURL, function(err, result) {
        if(err) {
            console.warn(err);
            return;
        }

        var map = result;
        iconMesh.material.map = map;
        iconMesh.material.map.needsUpdate = true;
        iconMesh.material.needsUpdate = true;
    });


    var t = self.textTexture(data.resource.title, conf.fontColor);
    var texture = t.texture;
    var size = t.size;
    var x = w*(size.width/size.height);
    var y = h;

    //var titleMaterial = new THREE.MeshBasicMaterial({ map: texture, side: THREE.DoubleSide });
    var titleMaterial = self.decideMaterial(data, { map: texture, side: THREE.DoubleSide });
    if(!titleMaterial) {
        return callback(null);
    }
    titleMaterial.transparent = true;
    titleMaterial.color.set(conf.fontColor);
    var titleGeometry = new THREE.PlaneGeometry(x, y);
    var titleMesh = new THREE.Mesh(titleGeometry, titleMaterial);
    titleMesh.name = 'title';


    var totalWidth = x + w + 2; // 4 is between space
    iconMesh.position.x = -(totalWidth/2 - (w/2))+1;
    titleMesh.position.x = (totalWidth)/2 - x/2-1;


    var bgGeometry = new THREE.PlaneGeometry(totalWidth, h);
    //var bgMaterial = new THREE.MeshBasicMaterial({ side: THREE.DoubleSide });
    var bgMaterial = self.decideMaterial(data, { side: THREE.DoubleSide });
    if(!bgMaterial) {
        return callback(null);
    }
    bgMaterial.transparent = true;
    //bgMaterial.depthTest = false; // Always behind order
    bgMaterial.color.set(data.color);
    var bgMesh = new THREE.Mesh(bgGeometry, bgMaterial);
    bgMesh.name = 'group_bg';

    var group = new THREE.Group();
    group.add(bgMesh);
    group.add(titleMesh);
    group.add(iconMesh);

    group.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.resource && objData.resource.hasOwnProperty('url')) {
            var url = objData.resource.url;
            window.open(url, '_blank');
        }
    };

    self.setData(group, data);
    self.glScene.add(group);
    self.addObject(group);
    //titleMesh.renderOrder = 5;
    //iconMesh.renderOrder = 10;

    return callback(group);
};
Workspace.prototype.addTwitter = function(data, callback) {
    var conf = getObjConf(data.type);
    var self = this;

    var w = 10;
    var h = 10;

    var iconGeometry = new THREE.PlaneGeometry(w, h);
    //var iconMaterial = new THREE.MeshBasicMaterial({ side: THREE.DoubleSide });
    var iconMaterial = self.decideMaterial(data, { side: THREE.DoubleSide });
    if(!iconMaterial) {
        return callback(null);
    }
    iconMaterial.transparent = true;

    var iconMesh = new THREE.Mesh(iconGeometry, iconMaterial);
    iconMesh.name = 'icon';
    var iconURL = './assets/images/twitter.light.png';
    self.imageTexture(iconURL, function(err, result) {
        if(err) {
            console.warn(err);
            return;
        }

        var map = result;
        iconMesh.material.map = map;
        iconMesh.material.map.needsUpdate = true;
        iconMesh.material.needsUpdate = true;
    });


    var t = self.textTexture(data.resource.title, conf.fontColor);
    var texture = t.texture;
    var size = t.size;
    var x = w*(size.width/size.height);
    var y = h;

    //var titleMaterial = new THREE.MeshBasicMaterial({ map: texture, side: THREE.DoubleSide });
    var titleMaterial = self.decideMaterial(data, { map: texture, side: THREE.DoubleSide });
    if(!titleMaterial) {
        return callback(null);
    }
    titleMaterial.transparent = true;
    titleMaterial.color.set(conf.fontColor);
    var titleGeometry = new THREE.PlaneGeometry(x, y);
    var titleMesh = new THREE.Mesh(titleGeometry, titleMaterial);
    titleMesh.name = 'title';


    var totalWidth = x + w + 2; // 4 is between space
    iconMesh.position.x = -(totalWidth/2 - (w/2))+1;
    titleMesh.position.x = (totalWidth)/2 - x/2-1;


    var bgGeometry = new THREE.PlaneGeometry(totalWidth, h);
    //var bgMaterial = new THREE.MeshBasicMaterial({ side: THREE.DoubleSide });
    var bgMaterial = self.decideMaterial(data, { side: THREE.DoubleSide });
    if(!bgMaterial) {
        return callback(null);
    }
    bgMaterial.transparent = true;
    //bgMaterial.depthTest = false; // Always behind order
    bgMaterial.color.set(data.color);
    var bgMesh = new THREE.Mesh(bgGeometry, bgMaterial);
    bgMesh.name = 'group_bg';

    var group = new THREE.Group();
    group.add(bgMesh);
    group.add(titleMesh);
    group.add(iconMesh);

    group.ondblclick = function() {
        var obj = self.getObjectByUID(data.uid);
        var objData = obj.userData;
        if(objData.resource && objData.resource.hasOwnProperty('url')) {
            var url = objData.resource.url;
            window.open(url, '_blank');
        }
    };

    self.setData(group, data);
    self.glScene.add(group);
    self.addObject(group);
    //titleMesh.renderOrder = 5;
    //iconMesh.renderOrder = 10;

    return callback(group);
};
Workspace.prototype.addAnimationList = function (method) {
    var self = this;
    self.animationList.push(method);
    return;
    if(typeof method === 'function') {
        self.animationList.push(method);
    }
};
Workspace.prototype.removeAnimationByUID = function (uid) {
    var self = this;
    var aaa = self.animationList.filter(function(item) {
        if(uid) {
            if(item.uid == uid) {
                if(typeof item.remove === 'function') {
                    item.remove();
                }

                return false;
            }
            return true;
        } else {
            if(typeof item.remove === 'function') {
                item.remove();
            }
            return false;
        }
    });
    self.animationList = aaa;
};
Workspace.prototype.removeAnimationByType = function (type) {
    var self = this;
    var aaa = self.animationList.filter(function(item) {
        if(item.type == type) {
            item.remove();
            return false;
        }
        return true;
    });
    self.animationList = aaa;
};
Workspace.prototype.getScene = function(data) {
    return this.glScene;
};


Workspace.prototype.setEffect_heart = function() {
    var self = this;
    var extrudeSettings = { depth: 8, bevelEnabled: true, bevelSegments: 2, steps: 2, bevelSize: 1, bevelThickness: 1 };
    // Heart
    var x = 0, y = 0;
    var heartShape = new THREE.Shape(); // From http://blog.burlock.org/html5/130-paths
    heartShape.moveTo( x + 25, y + 25 );
    heartShape.bezierCurveTo( x + 25, y + 25, x + 20, y, x, y );
    heartShape.bezierCurveTo( x - 30, y, x - 30, y + 35, x - 30, y + 35 );
    heartShape.bezierCurveTo( x - 30, y + 55, x - 10, y + 77, x + 25, y + 95 );
    heartShape.bezierCurveTo( x + 60, y + 77, x + 80, y + 55, x + 80, y + 35 );
    heartShape.bezierCurveTo( x + 80, y + 35, x + 80, y, x + 50, y );
    heartShape.bezierCurveTo( x + 35, y, x + 25, y + 25, x + 25, y + 25 );

    function addShape( shape, extrudeSettings, color, x, y, z, rx, ry, rz, s ) {
        var geometry = new THREE.ShapeBufferGeometry(shape);
        var mesh = new THREE.Mesh(geometry, new THREE.MeshPhongMaterial({color: color, side: THREE.DoubleSide}));
        mesh.position.set(x, y, z - 125);
        mesh.rotation.set(rx, ry, rz);
        mesh.scale.set(s, s, s);

        self.setData(mesh, {});
        self.glScene.add(mesh);
        self.addObject(mesh);
    }
    addShape( heartShape, extrudeSettings, 0xf00000, 60, 100, 0, 0, 0, Math.PI, 1 );
};

Workspace.prototype.getLightData = function(obj) {
    var self = this;
    var data = {
        uuid: obj.uuid,
        userData: copyObj(obj.userData),
        position: copyObj({x: obj.position.x, y: obj.position.y, z: obj.position.z}),
        rotation: copyObj({x: obj.rotation.x, y: obj.rotation.y, z: obj.rotation.z}),
        scale: copyObj({x: obj.scale.x, y: obj.scale.y, z: obj.scale.z}),
        opacity: Number(obj.userData.opacity),
        intensity: Number(obj.intensity),
        color: obj.color.getHexString(),
        distance: Number(obj.distance),

    };
    return data;
};
Workspace.prototype.setLightData = function(obj, data) {
    var self = this;
    var conf = getObjConf(data.type);

    obj.userData = data;

    var light = obj;

    if(data.color) {
        light.color = new THREE.Color(data.color);
    }
    if(data.position) {
        light.position.set(data.position.x, data.position.y, data.position.z);
    }
    if(data.hasOwnProperty('visible')) {
        light.visible = data.visible;
    }
    if(data.hasOwnProperty('intensity')) {
        light.intensity = data.intensity;
    }

    if(conf.type == 'directional') {
        // setting shadow...
    }
    if(conf.type == 'hemisphere') {
        if(data.groundColor) {
            light.groundColor = new THREE.Color(data.groundColor);
        }
    }
    if(conf.type == 'point' || conf.type == 'spot') {
        // setting shadow...

        if(data.hasOwnProperty('distance')) {
            light.distance = data.distance;
        }

        if(conf.type == 'spot') {
            if(data.hasOwnProperty('angle')) {
                light.angle = data.angle;
            }
        }
    }

    self.fireObjectChange(self.getLightData(obj));
};
Workspace.prototype.getLightTarget = function(data) {
    var self = this;
    var target = null;
    var uid = null;
    if(data.targetuid) {
        if(data.targetuid != 'scene') {
            var obj = self.getObjectByUID(data.targetuid);
            if(!obj) {
                target = self.getScene();
                uid = 'scene';
            } else {
                target = obj;
                uid = data.targetuid;
            }
        }
        if(data.targetuid == 'scene') {
            target = self.getScene();
            uid = 'scene';
        }
    } else {
        if(AREditor.isPreview === true) {
            target = self.getScene();
            uid = 'scene';
        } else {
            if(self.INTERSECTED) {
                var conf  = getObjConf(self.INTERSECTED.userData.type);
                if(conf.ctg == 'light') {
                    target = self.getScene();
                    uid = 'scene';
                } else {
                    target = self.INTERSECTED;
                    uid = self.INTERSECTED.userData.uid;
                }
            } else {
                target = self.getScene();
                uid = 'scene';
            }
        }
    }
    return {target: target, uid: uid};
};
Workspace.prototype.addDirectionalLight = function(data, callback) {
    var self = this;
    var light = new THREE.DirectionalLight();

    var target = self.getLightTarget(data);
    light.target = target.target;
    data.targetuid = target.uid;

    light.castShadow = true;

    self.setLightData(light, data);
    self.glScene.add(light);
    self.addObject(light);

    if(AREditor.isPreview === true) {
        return callback(light);
    }


    var helper = new THREE.DirectionalLightHelper( light, 5 );
    helper.userData = data;
    helper.userData.label = 'DirectionalLightHelper';


    self.glScene.add(helper);
    self.addObject(helper);

    var ani = {
        uid: data.uid,
        name: data.label,
        type: 'light_helper',
        helper: helper,
        light: light,
        go: function() {
            /*this.helper.updateMatrixWorld();
            this.helper.position.setFromMatrixPosition(this.light.matrixWorld);
            this.helper.updateMatrix();*/
            this.helper.update();
        },
        remove: function() {
            self.glScene.remove(helper);
            self.removeDepth(helper);
            this.helper = null;
        }
    };

    self.addAnimationList(ani);

    return callback(light);
};

Workspace.prototype.addSpotLight = function(data, callback) {
    var self = this;
    var light = new THREE.SpotLight();

    var target = self.getLightTarget(data);
    light.target = target.target;
    data.targetuid = target.uid;

    light.castShadow = true;

    self.setLightData(light, data);
    self.glScene.add(light);
    self.addObject(light);

    if(AREditor.isPreview === true) {
        return callback(light);
    }


    var helper = new THREE.SpotLightHelper( light );
    helper.userData = data;
    helper.userData.label = 'SpotLightHelper';
    self.glScene.add(helper);
    self.addObject(helper);

    var ani = {
        uid: data.uid,
        name: data.label,
        type: 'light_helper',
        helper: helper,
        go: function() {
            helper.update();
        },
        remove: function() {
            self.glScene.remove(helper);
            self.removeDepth(helper);
            this.helper = null;
        }
    };

    self.addAnimationList(ani);

    return callback(light);
};

Workspace.prototype.addPointLight = function(data, callback) {
    var self = this;
    var light = new THREE.PointLight();
    light.castShadow = true;

    self.setLightData(light, data);
    self.glScene.add(light);
    self.addObject(light);

    if(AREditor.isPreview === true) {
        return callback(light);
    }

    var helper = new THREE.PointLightHelper( light, 5 );
    helper.userData = data;
    helper.userData.label = 'PointLightHelper';
    self.glScene.add(helper);
    self.addObject(helper);

    var ani = {
        uid: data.uid,
        name: data.label,
        type: 'light_helper',
        helper: helper,
        go: function() {
            helper.update();
        },
        remove: function() {
            self.glScene.remove(helper);
            self.removeDepth(helper);
            this.helper = null;
        }
    };

    self.addAnimationList(ani);

    return callback(light);
};

Workspace.prototype.addHemisphereLight = function(data, callback) {
    var self = this;
    var light = new THREE.HemisphereLight();

    self.setLightData(light, data);
    self.glScene.add(light);
    self.addObject(light);

    if(AREditor.isPreview === true) {
        return callback(light);
    }


    var helper = new THREE.HemisphereLightHelper( light, 5 );
    helper.userData = data;
    helper.userData.label = 'HemisphereLightHelper';

    self.glScene.add(helper);
    self.addObject(helper);

    var ani = {
        uid: data.uid,
        name: data.label,
        type: 'light_helper',
        helper: helper,
        go: function() {
            helper.update();
        },
        remove: function() {
            self.glScene.remove(helper);
            self.removeDepth(helper);
            this.helper = null;
        }
    };

    self.addAnimationList(ani);

    return callback(light);
};

Workspace.prototype.addAmbientLight = function(data, callback) {
    var self = this;
    var light = new THREE.AmbientLight();

    self.setLightData(light, data);
    self.glScene.add(light);
    self.addObject(light);

    if(AREditor.isPreview === true) {
        return callback(light);
    }

    return callback(light);
};

Workspace.prototype.addGallery = function(data, callback) {
    var conf = getObjConf(data.type);
    var self = this;

    if(!Array.isArray(data.resource.urls)) {
        data.resource.urls = [];
    }
    var urls = data.resource.urls;

    var w = 30;
    var h = 20;

    var group = new THREE.Group();
    group.curr = 0;

    group.createImage = function(url, index, callback) {
        var image = new Image();
        var count = this.userData.resource.urls.length-1;
        var curr = this.curr;
        var next = (curr+1) > count? 0 : curr+1;
        var prev = (curr-1) < 0? count : curr-1;

        image.src= url;
        image.onload = function() {
            var geometry = new THREE.PlaneGeometry( w, h );
            var material = self.decideMaterial(data, {side: THREE.DoubleSide});
            if(!material) {
                return callback(null);
            }
            self.imageTexture(url, function(err, result) {
                if(err) {
                    console.warn(err);
                    return;
                }

                material.map = result;
                material.map.needsUpdate = true;
                material.needsUpdate = true;
                material.transparent = true;

                var mesh = new THREE.Mesh( geometry, material );

                if(index === next || index === prev || index === curr) {
                    mesh.visible = true;
                } else {
                    mesh.visible = false;
                }

                if(index === next) {
                    mesh.position.x = 20;
                    mesh.position.z = -10;
                    mesh.rotation.y = -Math.PI/4;
                    mesh.scale.set(0.6, 0.6, 0.6);
                    //child.material.opacity = 0.46;
                }
                if(index === prev) {
                    mesh.position.x = -20;
                    mesh.position.z = -10;
                    mesh.rotation.y = Math.PI/4;
                    mesh.scale.set(0.6, 0.6, 0.6);
                    //child.material.opacity = 0.46;
                }
                if(index === curr) {
                    mesh.position.x = 0;
                    mesh.position.z = 0;
                    mesh.rotation.y = 0;
                    mesh.scale.set(1.0, 1.0, 1.0);
                    //child.material.opacity = 1.0;
                }
                mesh.userData = {index: index, url: url};
                return callback(mesh);
            });
        };
    };

    group.onclick = function(mx) {
        clearTimeout(this.zoom_timer);
        var resource = this.userData.resource;
        if(resource.urls.length == 0) {return;}
        var count = resource.urls.length-1;

        if(this.children.length !== resource.urls.length) {
            console.log('Not equal: ', this.children.length, resource.urls.length);
            setTimeout(function() {
                group.onclick(mx);
            }, 100);
            return;
        }

        var curr = this.curr;
        var next = (curr+1) > count? 0 : curr+1;
        var prev = (curr-1) < 0? count : curr-1;

        var pos = mx===0? 0 : (mx - this.position.x);

        if(pos > 0) { // click right-side
            curr = (curr+1) > count? 0 : curr+1;
            next = (curr+1) > count? 0 : curr+1;
            prev = (curr-1) < 0? count : curr-1;
            this.curr = curr;
        }
        if(pos < 0) { // click left-side
            curr = (curr-1) < 0? count : curr-1;
            next = (curr+1) > count? 0 : curr+1;
            prev = (curr-1) < 0? count : curr-1;
            this.curr = curr;
        }

        var currObj = null;
        var prevObj = null;
        var nextObj = null;
        this.children.forEach(function(child, index) {
            if(index === next || index === prev || index === curr) {
                child.visible = true;
            } else {
                child.visible = false;
            }
            if(index === next) {
                nextObj = child;
            }
            if(index === prev) {
                prevObj = child;
            }
            if(index === curr) {
                currObj = child;
            }

            if(index === next) {
                child.position.x = 20;
                child.position.z = -10;
                child.rotation.y = -Math.PI/4;
                child.scale.set(0.6, 0.6, 0.6);
                //child.material.opacity = 0.46;
            }
            if(index === prev) {
                child.position.x = -20;
                child.position.z = -10;
                child.rotation.y = Math.PI/4;
                child.scale.set(0.6, 0.6, 0.6);
                //child.material.opacity = 0.46;
            }
            if(index === curr) {
                child.position.x = 0;
                child.position.z = 0;
                child.rotation.y = 0;
                child.scale.set(0, 0, 0);
                //child.material.opacity = 1.0;
            }
        });

        function zoom() {
            if(currObj.scale.x >= 1) {
                clearTimeout(this.zoom_timer);
                return;
            }
            currObj.scale.x += 0.1;
            currObj.scale.y += 0.1;
            currObj.scale.z += 0.1;
            this.zoom_timer = setTimeout(zoom, 20);
        }
        zoom();



        var prev_ani = {
            target: prevObj,
            start: 0,
            to: Math.PI/4,
            curr: 0,
            tick: (Math.PI/4)/10,
            update: function() {
                this.curr = (this.curr + this.tick);
                if(this.target) {
                    this.target.rotation.y = this.curr;
                }
                return this.curr;
            }
        };
        var next_ani = {
            target: nextObj,
            start: 0,
            to: Math.PI/4,
            curr: 0,
            tick: (Math.PI/4)/10,
            update: function() {
                this.curr = (this.curr - this.tick);
                if(this.target) {
                    this.target.rotation.y = this.curr;
                }
                return this.curr;
            }
        };
        var curr_ani_left = {
            target: currObj,
            start: -Math.PI/4,
            to: 0,
            curr: -Math.PI/4,
            tick: (Math.PI/4)/10,
            update: function() {
                this.curr = (this.curr + this.tick);
                if(this.target) {
                    this.target.rotation.y = this.curr;
                }
                return this.curr;
            }
        };
        var curr_ani_right = {
            target: currObj,
            start: Math.PI/4,
            to: 0,
            curr: Math.PI/4,
            tick: (Math.PI/4)/10,
            update: function() {
                this.curr = (this.curr - this.tick);
                if(this.target) {
                    this.target.rotation.y = this.curr;
                }
                return this.curr;
            }
        };


        var duration = 1000;
        var currtime = 0;
        var tick = 100;
        var timer = null;
        function animate() {
            currtime += tick;
            if(currtime > duration) {
                clearTimeout(timer);
                return;
            }

            next_ani.update();
            prev_ani.update();
            curr_ani_left.update();

            timer = setTimeout(animate, tick);
        }

        //animate();
    };

    self.setData(group, data);
    self.glScene.add(group);
    self.addObject(group);

    //group.onclick(0); // For init gallery

    return callback(group);
};

Workspace.prototype.removeMarker = function() {
    var self = this;
    var marker = null;
    self.glScene.traverse(function(child) {
        if(child.name == 'scene_marker') {
            marker = child;
        }
    });
    if(marker) {
        self.glScene.remove(marker);
        self.removeDepth(marker);
    }
};
Workspace.prototype.setMarker = function(data, callback) {
    var self = this;
    var url = data.url;
    var image = new Image();
    image.src= url;
    image.onload = function() {
        var geometry = new THREE.PlaneGeometry(image.width/10, image.height/10);
        var material = new THREE.MeshBasicMaterial({side: THREE.DoubleSide});
        if(!material) {
            if(callback) {
                return callback(null);
            }
        }
        var mesh = new THREE.Mesh( geometry, material );
        mesh.name = 'scene_marker';
        mesh.rotation.x = -Math.PI/2;
        mesh.receiveShadow = true;

        self.imageTexture(url, function (err, result) {
            if (err) {
                console.log(err);
            } else {
                var texture = result;

                mesh.material.map = texture;
                mesh.material.map.needsUpdate = true;
                mesh.material.needsUpdate = true;
            }
        });

        self.glScene.add(mesh);

        if(callback) {
            return callback(mesh);
        }
    };
};

Workspace.prototype.setParticleData = function(obj, data) {
    var self = this;
    var conf = getObjConf(data.type);

    obj.userData = data;
    var uid = obj.userData.uid;

    var particle = obj;

    if(data.position) {
        particle.position.x = data.position.x;
        particle.position.y = data.position.y;
        particle.position.z = data.position.z;
    }
    if(data.rotation) {
        particle.rotation.x = data.rotation.x;
        particle.rotation.y = data.rotation.y;
        particle.rotation.z = data.rotation.z;
    }
    if(data.scale) {
        particle.scale.x = data.scale.x;
        particle.scale.y = data.scale.y;
        particle.scale.z = data.scale.z;
    }

    if(data.hasOwnProperty('visible')) {
        particle.visible = data.visible;
    }

    self.fireObjectChange(self.getParticleData(obj));
};
Workspace.prototype.getParticleData = function(obj) {
    var self = this;
    var data = {
        uuid: obj.uuid,
        userData: copyObj(obj.userData),
        position: copyObj({x: obj.position.x, y: obj.position.y, z: obj.position.z}),
        rotation: copyObj({x: obj.rotation.x, y: obj.rotation.y, z: obj.rotation.z}),
        scale: copyObj({x: obj.scale.x, y: obj.scale.y, z: obj.scale.z}),
        opacity: Number(obj.userData.opacity),
    };
    return data;
};
Workspace.prototype.removeParticle = function(obj) {
    var self = this;
    self.removeObject(obj);
    //self.removeAnimationByUID(uid);
    //self.ee.trigger('unselected_object');
};
Workspace.prototype.addBubbling = function(data, callback) {
    var self = this;
    var conf = getObjConf(data.type);
    var uid = data.uid;
    var label = data.label;
    self.removeAnimationByUID(uid);

    var envMap = self.glScene.background;
    if(!envMap || !envMap.texture) {
        setTimeout(function(){self.addBubbling(data, callback);}, 100);
        return;
    }
    var geometry = new THREE.SphereGeometry( 10, 32, 16 );
    var material = new THREE.MeshBasicMaterial( { color: 0xffffff, envMap: envMap.texture, refractionRatio: 0.9  } );
    material.envMap.mapping = THREE.CubeRefractionMapping;
    var spheres = new THREE.Group();
    for ( var i = 0; i < 100; i ++ ) {
        var mesh = new THREE.Mesh( geometry, material );
        mesh.position.x = Math.random() * 100 - 50;
        mesh.position.y = Math.random() * 100 - 50;
        mesh.position.z = Math.random() * 100 - 50;
        mesh.scale.x = mesh.scale.y = mesh.scale.z = Math.random();
        mesh.material.opacity = 0.6;
        mesh.material.transparent = true;
        //mesh.material.depthWrite = false;
        // mesh.material.depthTest = false;
        mesh.material.needsUpdate = true;
        mesh.renderOrder = Infinity;
        spheres.add( mesh );
    }
    self.setParticleData(spheres, data);
    self.glScene.add( spheres );
    self.addObject(spheres);

    var ani = {
        uid: uid,
        label: label,
        type: conf.ctg,
        spheres: spheres,
        go: function() {
            var timer = 0.0001 * Date.now();
            for ( var i = 0, il = this.spheres.children.length; i < il; i ++ ) {
                var sphere = this.spheres.children[ i ];
                sphere.position.x = 100 * Math.cos( timer + i );
                sphere.position.y = 100 * Math.sin( timer + i * 1.1 );
            }
        },
        remove: function() {
            return; // Already animation was removed by removeObject()
            this.spheres.children.forEach(function(item) {
                self.glScene.remove(item);
                //self.removeDepth(item);
            });
            self.glScene.remove(this.spheres);
        }
    };

    self.addAnimationList(ani);

    return callback(spheres);
};

Workspace.prototype.addSpawning = function(data, callback) {
    var self = this;
    var conf = getObjConf(data.type);
    var uid = data.uid;
    var label = data.label;
    self.removeAnimationByUID(uid);

    var options = (data.settings&&data.settings.options)? data.settings.options : {
        position: new THREE.Vector3(),
        positionRandomness: .3,
        velocity: new THREE.Vector3(),
        velocityRandomness: .5,
        color: 0xaa88ff,
        colorRandomness: .2,
        turbulence: .5,
        lifetime: 2,
        size: 5,
        sizeRandomness: 1
    };
    var spawnerOptions = (data.settings&&data.settings.spawnerOptions)? data.settings.spawnerOptions : {
        spawnRate: 15000,
        horizontalSpeed: 1.5,
        verticalSpeed: 1.33,
        timeScale: 1
    };

    if(!data.settings) {
        data.settings = {};
    }
    data.settings.options = options;
    data.settings.spawnerOptions = spawnerOptions;

    var particle = new THREE.GPUParticleSystem( {
        maxParticles: 250000,
        texture_noise: './assets/images/textures/perlin-512.png',
        texture_sprite: './assets/images/textures/particle2.png'
    } );

    var onFinishUpdate = function(result) {
        self.fireObjectChange(self.getParticleData(particle));
    };
    particle.refreshSettings = function(settings) {
        if(this.gui) {
            this.gui.destroy();
            this.gui = null;
        }
        var gui = new dat.GUI( { width: 300, autoPlace: false } );
        var options = settings? settings.options : this.userData.settings.options;
        var spawnerOptions = settings? settings.spawnerOptions : this.userData.settings.spawnerOptions;

        gui.add( options, "velocityRandomness", 0, 3 ).onFinishChange(onFinishUpdate);
        gui.add( options, "positionRandomness", 0, 3 ).onFinishChange(onFinishUpdate);
        gui.add( options, "size", 1, 20 ).onFinishChange(onFinishUpdate);
        gui.add( options, "sizeRandomness", 0, 25 ).onFinishChange(onFinishUpdate);
        gui.add( options, "colorRandomness", 0, 1 ).onFinishChange(onFinishUpdate);
        gui.add( options, "lifetime", .1, 10 ).onFinishChange(onFinishUpdate);
        gui.add( options, "turbulence", 0, 1 ).onFinishChange(onFinishUpdate);
        gui.add( spawnerOptions, "spawnRate", 10, 30000 ).onFinishChange(onFinishUpdate);
        gui.add( spawnerOptions, "timeScale", - 1, 1 ).onFinishChange(onFinishUpdate);
        this.gui = gui;

        objById('gui_control').innerHTML = '';
        objById('gui_control').appendChild(this.gui.domElement);
        objById('gui_control').setAttribute('data-uid', this.userData.uid);
    };

    self.setParticleData(particle, data);
    self.glScene.add( particle );
    self.addObject(particle);

    particle.refreshSettings(data.settings);

    var ani = {
        uid: uid,
        label: label,
        type: conf.ctg,
        tick: 0,
        options: options,
        spawnerOptions: spawnerOptions,
        engine: particle,
        go: function(delta) {
            var delta = delta * this.spawnerOptions.timeScale;
            this.tick += delta;
            if ( this.tick < 0 ) this.tick = 0;
            if ( delta > 0 ) {
                this.options.position.x = Math.sin( this.tick * this.spawnerOptions.horizontalSpeed ) * 20;
                this.options.position.y = Math.sin( this.tick * this.spawnerOptions.verticalSpeed ) * 10;
                this.options.position.z = Math.sin( this.tick * this.spawnerOptions.horizontalSpeed + this.spawnerOptions.verticalSpeed ) * 5;
                for ( var x = 0; x < this.spawnerOptions.spawnRate * delta; x ++ ) {
                    this.engine.spawnParticle( this.options );
                }
            }
            this.engine.update( this.tick );
        },
        remove: function() {
            this.engine.gui.destroy();
            //self.glScene.remove(this.engine);
            //self.removeDepth(this.engine);
            //this.engine = null;
        },
    };

    self.addAnimationList(ani);

    return callback(particle);
};
Workspace.prototype.addFireSmoke = function(data, callback) {
    var self = this;
    var conf = getObjConf(data.type);
    var uid = data.uid;
    var label = data.label;
    self.removeAnimationByUID(uid);

    var options = (data.settings&&data.settings.options)? data.settings.options : {
        color1: '#ffffff',
        color2: '#ffa000',
        color3: '#000000',
        colorBias: 0.8,
        burnRate: 1.0,
        diffuse: 1.33,
        viscosity: 0.21,
        expansion: -0.25,
        swirl: 50,
        drag: 0.3,
        airSpeed: 15,
        windX: 0.0,
        windY: 0.5,
        speed: 500.0,
        massConservation: false,
        useText: false,
        text: 'Sample Text',
        multiple: false,
    };

    if(!data.settings) {
        data.settings = {};
    }
    data.settings.options = options;

    var plane = new THREE.PlaneBufferGeometry( 50, 50 );
    var particle = new THREE.Fire( plane, {
        textureWidth: 512,
        textureHeight: 512,
        debug: false
    } );

    particle.material.side = THREE.DoubleSide;
    particle.material.depthWrite = false;

    particle.refreshSettings = function(settings) {
        var mee = this;

        if(this.gui) {
            this.gui.destroy();
            this.gui = null;
        }
        var gui = new dat.GUI( { autoPlace: false } );
        var options = settings? settings.options : this.userData.settings.options;

        var onFinishUpdate = function(result) {
            self.fireObjectChange(self.getParticleData(particle));
        };

        function updateColor1( value ) {
            mee.color1.set( value );
        }

        function updateColor2( value ) {
            mee.color2.set( value );
        }

        function updateColor3( value ) {
            mee.color3.set( value );
        }

        function updateColorBias( value ) {
            mee.colorBias = value;
        }

        function updateBurnRate( value ) {
            mee.burnRate = value;
        }

        function updateDiffuse( value ) {
            mee.diffuse = value;
        }

        function updateViscosity( value ) {
            mee.viscosity = value;
        }

        function updateExpansion( value ) {
            mee.expansion = value;
        }

        function updateSwirl( value ) {
            mee.swirl = value;
        }

        function updateDrag( value ) {
            mee.drag = value;
        }

        function updateAirSpeed( value ) {
            mee.airSpeed = value;
        }

        function updateWindX( value ) {
            mee.windVector.x = value;
        }

        function updateWindY( value ) {
            mee.windVector.y = value;
        }

        function updateSpeed( value ) {
            mee.speed = value;
        }

        function updateMassConservation( value ) {
            mee.massConservation = value;
        }

        function updateUseText( value ) {
            if(value === true) {
                updateText(options.text);
            } else {
                //mee.setSourceMap( null );
            }
        }

        function updateText( value ) {
            if(options.useText === false) { return; }
            var text = value? value : 'Sample Text';
            var t = self.textTexture(text, '#ff0400');
            var texture = t.texture;
            var size = t.size;
            var x = 10*(size.width/size.height);
            var y = 10;

            //mee.geometry.parameters.width = x;
            //mee.geometry.parameters.height = y;
            //mee.scale.x = x/20;
            //mee.scale.y = y/20;

            mee.setSourceMap( texture );
        }

        function updateMultiple( value ) {
            if(value === true) {
                mee.clearSources();
                mee.addSource( 0.45, 0.1, 0.1, 0.5, 0.0, 1.0 );
                mee.addSource( 0.55, 0.1, 0.1, 0.5, 0.0, 1.0 );
            } else {
                mee.clearSources();
                mee.addSource( 0.5, 0.1, 0.1, 1.0, 0.0, 1.0 );
            }
        }

        gui.addColor( options, 'color1' ).onChange( updateColor1 ).onFinishChange(onFinishUpdate);
        gui.addColor( options, 'color2' ).onChange( updateColor2 ).onFinishChange(onFinishUpdate);
        gui.addColor( options, 'color3' ).onChange( updateColor3 ).onFinishChange(onFinishUpdate);
        gui.add( options, 'colorBias', 0.0, 1.0 ).onChange( updateColorBias ).onFinishChange(onFinishUpdate);
        gui.add( options, 'burnRate', 0.0, 10.0 ).onChange( updateBurnRate ).onFinishChange(onFinishUpdate);
        gui.add( options, 'diffuse', 0.0, 5.0 ).step( 0.01 ).onChange( updateDiffuse ).onFinishChange(onFinishUpdate);
        gui.add( options, 'viscosity', 0.0, 5.0 ).step( 0.01 ).onChange( updateViscosity ).onFinishChange(onFinishUpdate);
        gui.add( options, 'expansion', - 1.0, 1.0 ).step( 0.01 ).onChange( updateExpansion ).onFinishChange(onFinishUpdate);
        gui.add( options, 'swirl', 0.0, 50.0 ).step( 0.01 ).onChange( updateSwirl ).onFinishChange(onFinishUpdate);
        gui.add( options, 'drag', 0.0, 1.0 ).onChange( updateDrag ).onFinishChange(onFinishUpdate);
        gui.add( options, 'airSpeed', 0.0, 50.0 ).onChange( updateAirSpeed ).onFinishChange(onFinishUpdate);
        gui.add( options, 'windX', - 5, 5 ).step( 0.01 ).onChange( updateWindX ).onFinishChange(onFinishUpdate);
        gui.add( options, 'windY', - 5, 5 ).step( 0.01 ).onChange( updateWindY ).onFinishChange(onFinishUpdate);
        gui.add( options, 'speed', 0, 1000 ).onChange( updateSpeed ).onFinishChange(onFinishUpdate);
        gui.add( options, 'massConservation' ).onChange( updateMassConservation ).onFinishChange(onFinishUpdate);
        gui.add( options, 'useText' ).onChange( updateUseText ).onFinishChange(onFinishUpdate);
        gui.add( options, 'text' ).onChange( updateText ).onFinishChange(onFinishUpdate);
        gui.add( options, 'multiple' ).onChange( updateMultiple ).onFinishChange(onFinishUpdate);
        this.gui = gui;

        function updateAll() {

            updateColor1( options.color1 );
            updateColor2( options.color2 );
            updateColor3( options.color3 );
            updateColorBias( options.colorBias );
            updateBurnRate( options.burnRate );
            updateDiffuse( options.diffuse );
            updateViscosity( options.viscosity );
            updateExpansion( options.expansion );
            updateSwirl( options.swirl );
            updateDrag( options.drag );
            updateAirSpeed( options.airSpeed );
            updateWindX( options.windX );
            updateWindY( options.windY );
            updateSpeed( options.speed );
            updateMassConservation( options.massConservation );
            updateMultiple(options.multiple);

            if(options.useText) {
                updateText(options.text);
            } else {
                //mee.setSourceMap( null );
            }

            for ( var i in gui.__controllers ) {
                gui.__controllers[ i ].updateDisplay();
            }

        }

        updateAll();

        objById('gui_control').innerHTML = '';
        objById('gui_control').appendChild(this.gui.domElement);
        objById('gui_control').setAttribute('data-uid', this.userData.uid);
    };

    self.setParticleData(particle, data);
    self.glScene.add( particle );
    self.addObject(particle);

    particle.refreshSettings(data.settings);

    var ani = {
        uid: uid,
        label: label,
        type: conf.ctg,
        options: options,
        engine: particle,
        go: function(delta) {
            return;
        },
        remove: function() {
            this.engine.clearSources();
            this.engine.gui.destroy();
        },
    };

    self.addAnimationList(ani);

    return callback(particle);
};
Workspace.prototype.addSpark = function(data, callback) {
    var self = this;
    var conf = getObjConf(data.type);
    var uid = data.uid;
    var label = data.label;
    self.removeAnimationByUID(uid);

    var options = (data.settings && data.settings.options)? data.settings.options : {
        speed: 4,
        total: 200,
        itemSize: 20,
    };
    if(!data.settings) {
        data.settings = {};
    }
    data.settings.options = options;

    var particleTexture = new THREE.TextureLoader().load( './assets/images/textures/stemkoski/spark.png' );

    var particle = new THREE.Object3D();
    var particleAttributes = { startSize: [], startPosition: [], randomness: [] };

    makeItem(options);

    function makeItem(options) {
        var totalParticles = options.total? options.total : 200;
        var radiusRange = options.range? options.range : 40;
        var itemSize = options.itemSize? options.itemSize : 20;
        for( var i = 0; i < totalParticles; i++ ) {
            var spriteMaterial = new THREE.SpriteMaterial( { map: particleTexture, color: 0xffffff } );

            var sprite = new THREE.Sprite( spriteMaterial );
            sprite.scale.set( itemSize, itemSize, 1.0 ); // imageWidth, imageHeight
            sprite.position.set( Math.random() - 0.5, Math.random() - 0.5, Math.random() - 0.5 );
            // for a cube:
            // sprite.position.multiplyScalar( radiusRange );
            // for a solid sphere:
            // sprite.position.setLength( radiusRange * Math.random() );
            // for a spherical shell:
            sprite.position.setLength( radiusRange * (Math.random() * 0.1 + 0.9) );

             sprite.material.color.setHSL( Math.random(),  Math.random(),  Math.random() );
            //sprite.material.color.setHSL( Math.random(), 0.9, 0.7 );

            // sprite.opacity = 0.80; // translucent particles
            sprite.material.blending = THREE.AdditiveBlending; // "glowing" particles

            particle.add( sprite );
            // add variable qualities to arrays, if they need to be accessed later
            particleAttributes.startPosition.push( sprite.position.clone() );
            particleAttributes.randomness.push( Math.random() );
        }
    }


    particle.refreshSettings = function(settings) {
        var mee = this;
        if(mee.gui) {
            mee.gui.destroy();
            mee.gui = null;
        }
        var options = settings&&settings.options? settings.options : mee.userData.settings.options;

        var onFinishUpdate = function(result) {
            self.fireObjectChange(self.getParticleData(particle));
        };
        var onUpdateTotal = function(result) {
            particle.children = particle.children.filter(function(item) {
                return false;
            });
            makeItem(particle.userData.settings.options);
        };

        var gui = new dat.GUI( { autoPlace: false } );
        gui.add(options, 'speed', 0.0, 20.0).step( 0.1 ).onFinishChange(onFinishUpdate);
        gui.add(options, 'total', 1, 1000).step( 1 ).onChange(onUpdateTotal).onFinishChange(onFinishUpdate);
        gui.add(options, 'itemSize', 1, 100).step( 1 ).onChange(onUpdateTotal).onFinishChange(onFinishUpdate);
        this.gui = gui;

        objById('gui_control').innerHTML = '';
        objById('gui_control').appendChild(this.gui.domElement);
        objById('gui_control').setAttribute('data-uid', this.userData.uid);
    };

    self.setParticleData(particle, data);
    self.glScene.add( particle );
    self.addObject(particle);

    particle.refreshSettings(data.settings);

    var ani = {
        engine: particle,
        attributes: particleAttributes,
        options: options,
        go: function(delta) {
            var time = options.speed * self.clock.getElapsedTime();

            for ( var c = 0; c < this.engine.children.length; c ++ )
            {
                var sprite = this.engine.children[ c ];

                // particle wiggle
                // var wiggleScale = 2;
                // sprite.position.x += wiggleScale * (Math.random() - 0.5);
                // sprite.position.y += wiggleScale * (Math.random() - 0.5);
                // sprite.position.z += wiggleScale * (Math.random() - 0.5);

                // pulse away/towards center
                // individual rates of movement
                var a = particleAttributes.randomness[c] + 1;
                var pulseFactor = Math.sin(a * time) * 0.1 + 0.9;
                sprite.position.x = this.attributes.startPosition[c].x * pulseFactor;
                sprite.position.y = this.attributes.startPosition[c].y * pulseFactor;
                sprite.position.z = this.attributes.startPosition[c].z * pulseFactor;
            }

            // rotate the entire group
            //this.engine.rotation.x = time * 0.5;
            this.engine.rotation.y = time * 0.75;
            //this.engine.rotation.z = time * 1.0;

        },
        remove: function() {
            this.engine.gui.destroy();
            this.engine.children = this.engine.children.filter(function(item) {
                return false;
            });
        },

    };

    self.addAnimationList(ani);

    return callback(particle);
};
Workspace.prototype.addWater = function(data, callback) {
    var self =  this;
    var conf = getObjConf(data.type);
    var uid = data.uid;
    var label = data.label;
    self.removeAnimationByUID(uid);

    var vertexShader = [
        'uniform float time;',
        'varying vec2 textureUV;',
        'uniform sampler2D baseTexture;',
        'void main() ',
        '{ ',
        '   textureUV = uv;',
        '   vec2 repeatUV = textureUV * vec2(4.0, 1.0);',
        '   float sideDisplacement = 0.0;',
        '   // side displacement only applied near edge of texture',
        '   if (repeatUV.y < 0.15)',
        '   {',
        '      float angle = atan(position.y, position.x);',
        '      float s1 = sin(8.3 * angle);',
        '      float s2 = sin(7.1 * angle);',
        '      sideDisplacement = 0.08 * s1 * s1 * abs(sin(0.65*time)), 0.09 * s2 * s2 * abs(sin(0.74*time));',
        '   }',
        '   vec3 newPosition = position + normal * sideDisplacement;',
        '   gl_Position = projectionMatrix * modelViewMatrix * vec4( newPosition, 1.0 );',
        '}',
    ].join("\n");

    var fragmentShader = [
        'uniform float time;',
        'uniform sampler2D baseTexture;',
        'varying vec2 textureUV;',
        'void main() ',
        '{',
        '   vec2 repeatUV = textureUV * vec2(4.0, 1.0);',
        '   // displacement creates scrolling effect on texture',
        '   vec2 displaceUV = repeatUV + vec2(0.0, time/20.0);',
        '   float alpha = 1.0;',
        '   // transparency only applied near edge of texture',
        '   if (repeatUV.y < 0.15)',
        '      alpha = 1.0/0.15 * repeatUV.y;',
        '   vec4 tintColor = texture2D( baseTexture, displaceUV ) * vec4( 1.0, 1.0, 1.0, alpha );',
        '   gl_FragColor = tintColor;',
        '}  '
    ].join("\n");

    var geometry1 = new THREE.TorusGeometry(50, 50, 16, 100); // radius, tube radius

    var loader = new THREE.TextureLoader();
    var texture = loader.load( './assets/images/textures/water-2.jpg' );

    // repeat texture
    texture.wrapS = THREE.RepeatWrapping;
    texture.wrapT = THREE.RepeatWrapping;
    texture.repeat.set(8,2);

    // Shader-based material
    var material1 = new THREE.ShaderMaterial({
        uniforms: {
            time: { value: 1.0 },
            baseTexture: { value: texture },
        },
        vertexShader: vertexShader,
        fragmentShader: fragmentShader,
        transparent: true,
    });

    var particle = new THREE.Mesh( geometry1, material1 );

    self.setParticleData(particle, data);
    self.glScene.add( particle );
    self.addObject(particle);

    particle.rotation.x = -Math.PI/2;
    particle.scale.z = 0.10;

    var ani = {
        uid: uid,
        lable: label,
        engine: particle,
        go: function(delta) {
            this.engine.material.uniforms.time.value += delta;
        },
        remove: function() {},
    };

    self.addAnimationList(ani);

    return callback(particle);
};
Workspace.prototype.addCloudFog = function(data, callback) {
    var self = this;

    var conf = getObjConf(data.type);
    var uid = data.uid;
    var label = data.label;
    self.removeAnimationByUID(uid);

    var options0 = (data.settings&&data.settings.options0)? data.settings.options0 : {
        // ParticleGroup
        texture: './assets/images/textures/spe/cloud.png',
        blending: THREE.NormalBlending,
        fog: true,
        // Emitter
        particleCount: 2000,
        maxAge: {
            value: 3,
        },
        position: {
            value: {x: 0, y: -15, z: -50},
            spread: {x: 500, y: 30, z: 500},
            spreadClamp: {x: 0, y: 0, z: 0},
            //distribution: SPE.distributions.BOX,
            randomise: false
        },
        velocity: {
            value: {x: 0, y: 0, z: 30},
            spread: {x: 0, y: 0, z: 0},
            // distribution: SPE.distributions.BOX,
            randomise: false
        },
        wiggle: {
            spread: 10,
            value: 10
        },
        size: {
            value: 75,
            spread: 100
        },
        opacity: {
            value: {
                step1: 0,
                step2: 0,
                step3: 1,
                step4: 0
            }
        },
        color: {
            value: {
                step1: '#ffffff',
                step2: '#ffffff',
                step3: '#ffffff',
                step4: '#ffffff'
            },
            spread: ''
        },
        angle: {
            value: [ 0, Math.PI * 0.125 ]
        },
        acceleration: {
            value: new THREE.Vector3( 0, 0, 0 ),
            spread: new THREE.Vector3( 0, 0, 0 ),
            // distribution: SPE.distributions.BOX,
            randomise: false
        },
        speed: 1
    };

    var options1 = {
        texture: {
            value: new THREE.TextureLoader().load(options0.texture)
        },
        blending: options0.blending,
        fog: options0.fog
    };

    var options2 = {
        particleCount: options0.particleCount,
        maxAge: options0.maxAge,
        position: {
            value: new THREE.Vector3( options0.position.value.x, options0.position.value.y, options0.position.value.z ),
            spread: new THREE.Vector3( options0.position.spread.x, options0.position.spread.y, options0.position.spread.z ),
            spreadClamp: new THREE.Vector3( options0.position.spreadClamp.x, options0.position.spreadClamp.y, options0.position.spreadClamp.z ),
            //distribution: SPE.distributions.BOX,
            randomise: options0.position.randomise
        },
        velocity: {
            value: new THREE.Vector3( options0.velocity.value.x, options0.velocity.value.y, options0.velocity.value.z ),
            spread: new THREE.Vector3( options0.velocity.spread.x, options0.velocity.spread.y, options0.velocity.spread.z ),
            // distribution: SPE.distributions.BOX,
            randomise: options0.velocity.randomise
        },
        wiggle: options0.wiggle,
        size: options0.size,
        opacity: {
            value: [ options0.opacity.value.step1, options0.opacity.value.step2, options0.opacity.value.step3, options0.opacity.value.step4 ]
        },
        color: {
            value: [
                new THREE.Color().setStyle(options0.color.value.step1),
                new THREE.Color().setStyle(options0.color.value.step2),
                new THREE.Color().setStyle(options0.color.value.step3),
                new THREE.Color().setStyle(options0.color.value.step4)
            ],
            spread: new THREE.Color().setStyle(options0.color.spread)
        },
        angle: options0.angle,
        acceleration: {
            value: new THREE.Vector3( options0.acceleration.value.x, options0.acceleration.value.y, options0.acceleration.value.z ),
            spread: new THREE.Vector3( options0.acceleration.spread.x, options0.acceleration.spread.y, options0.acceleration.spread.z ),
            // distribution: SPE.distributions.BOX,
            randomise: false
        },
    };


    if(!data.settings) {
        data.settings = {};

    }
    data.settings.options0 = options0;

    var particleGroup = new SPE.Group(options1);

    var emitter = new SPE.Emitter(options2);


    particleGroup.addEmitter( emitter );
    var particle = particleGroup.mesh;

    particle.refreshSettings = function(settings) {
        //if(!settings) return false;
        var mee = this;
        if(mee.gui) {
            mee.gui.destroy();
            mee.gui = null;
        }

        var options0 = settings&&settings.options? settings.options0 : mee.userData.settings.options0;

        var onFinishUpdate = function(result) {
            //particle.userData = options0;
            self.fireObjectChange(self.getParticleData(particle));
        };
        var updateMaterial = function() {
            particleGroup.material.needsUpdate = true;
        };
        var vec3Components = ['x', 'y', 'z'], i;

        var gui = new dat.GUI( { autoPlace: false } );

        gui.add( options0, 'blending' ).min(0).max(5).step(1).onChange( function( value ) {
            particleGroup.material.blending = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        gui.add( options0, 'particleCount' ).min(0).max(50000).step(1).onChange( function( value ) {
            emitter.particleCount = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        gui.add( options0, 'speed' ).min(0.01).max(10).step(0.01).onChange( function( value ) {

        }).onFinishChange(onFinishUpdate);

        var size = gui.addFolder('Size');
        size.add(options0.size, 'spread', 1, 300).step(1).onChange(function(value) {
            emitter.size.spread = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        size.add(options0.size, 'value', 1, 300).step(1).onChange(function(value) {
            emitter.size.value = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);

        var wiggle = gui.addFolder('Wiggle');
        wiggle.add(options0.wiggle, 'spread', 1, 100).step(10).onChange(function(value) {
            emitter.wiggle.spread = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        wiggle.add(options0.wiggle, 'value', 1, 100).step(10).onChange(function(value) {
            emitter.wiggle.value = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);

        var velocityFolder = gui.addFolder( 'Velocity' );
        var velocityValue = velocityFolder.addFolder( 'Value' );
        var velocitySpread = velocityFolder.addFolder( 'Spread' );
        for( i = 0; i < vec3Components.length; ++i ) {
            velocityValue.add( options0.velocity.value, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.velocity.value = new THREE.Vector3( options0.velocity.value.x, options0.velocity.value.y, options0.velocity.value.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
            velocitySpread.add( options0.velocity.spread, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.velocity.spread = new THREE.Vector3( options0.velocity.spread.x, options0.velocity.spread.y, options0.velocity.spread.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
        }

        var positionFolder = gui.addFolder( 'Position' );
        var positionValue = positionFolder.addFolder( 'Value' );
        var positionSpread = positionFolder.addFolder( 'Spread' );
        var positionSpreadClamp = positionFolder.addFolder( 'Spread Clamp');
        for( i = 0; i < vec3Components.length; ++i ) {
            positionValue.add( options0.position.value, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.position.value = new THREE.Vector3( options0.position.value.x, options0.position.value.y, options0.position.value.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
            positionSpread.add( options0.position.spread, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.position.spread = new THREE.Vector3( options0.position.spread.x, options0.position.spread.y, options0.position.spread.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
            positionSpreadClamp.add( options0.position.spreadClamp, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.position.spreadClamp = new THREE.Vector3( options0.position.spreadClamp.x, options0.position.spreadClamp.y, options0.position.spreadClamp.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
        }

        var colorFolder = gui.addFolder( 'Color steps' );
        colorFolder.addColor(options0.color.value, 'step1' ).onChange( function( value ) {
            emitter.color.value[ 0 ].setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        colorFolder.addColor(options0.color.value, 'step2').onChange( function( value ) {
            emitter.color.value[ 1 ] = new THREE.Color().setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        colorFolder.addColor(options0.color.value, 'step3').onChange( function( value ) {
            emitter.color.value[ 2 ] = new THREE.Color().setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        colorFolder.addColor(options0.color.value, 'step4').onChange( function( value ) {
            emitter.color.value[ 3 ] = new THREE.Color().setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);


        var opacityFolder = gui.addFolder( 'Opacity steps' );
        opacityFolder.add( options0.opacity.value, 'step1' ).min(0).max(1).step(0.01).listen().onChange( function( value ) {
            emitter.opacity.value[ 0 ] = value;
            emitter.opacity.value = emitter.opacity.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        opacityFolder.add( options0.opacity.value, 'step2' ).min(0).max(1).step(0.01).listen().onChange( function( value ) {
            emitter.opacity.value[ 1 ] = value;
            emitter.opacity.value = emitter.opacity.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        opacityFolder.add( options0.opacity.value, 'step3' ).min(0).max(1).step(0.01).listen().onChange( function( value ) {
            emitter.opacity.value[ 2 ] = value;
            emitter.opacity.value = emitter.opacity.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        opacityFolder.add( options0.opacity.value, 'step4' ).min(0).max(1).step(0.01).listen().onChange( function( value ) {
            emitter.opacity.value[ 3 ] = value;
            emitter.opacity.value = emitter.opacity.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);


        var accelerationFolder = gui.addFolder( 'Acceleration' );
        var accelerationValue = accelerationFolder.addFolder( 'Value' );
        var accelerationSpread = accelerationFolder.addFolder( 'Spread' );

        for( i = 0; i < vec3Components.length; ++i ) {
            accelerationValue.add( options0.acceleration.value, vec3Components[ i ], -50, 50 ).listen().onChange( function() {
                emitter.acceleration.value = new THREE.Vector3( options0.acceleration.value.x, options0.acceleration.value.y, options0.acceleration.value.z ),
                updateMaterial();
            } ).onFinishChange(onFinishUpdate);
            accelerationSpread.add( options0.acceleration.spread, vec3Components[ i ], -50, 50 ).listen().onChange( function() {
                emitter.acceleration.spread = new THREE.Vector3( options0.acceleration.spread.x, options0.acceleration.spread.y, options0.acceleration.spread.z ),
                updateMaterial();
            } ).onFinishChange(onFinishUpdate);
        }

        this.gui = gui;

        objById('gui_control').innerHTML = '';
        objById('gui_control').appendChild(this.gui.domElement);
        objById('gui_control').setAttribute('data-uid', this.userData.uid);
    };

    self.setParticleData(particle, data);
    self.glScene.add( particle );
    self.addObject(particle);

    particle.refreshSettings(data.settings);

    var ani = {
        uid: uid,
        label: label,
        state: 0,
        mesh: particle,
        engine: particleGroup,
        go: function(delta) {
            var speed = this.mesh.userData.settings.options0.speed;
            if(this.state === 0) {
                speed = 10;
                this.state = 10;
            }
            this.engine.tick( delta*speed );
        },
        remove: function() {
            this.mesh.gui.destroy();
        }
    };

    self.addAnimationList(ani);

    return callback(particle);
};

Workspace.prototype.addFire = function(data, callback) {
    var self = this;

    var conf = getObjConf(data.type);
    var uid = data.uid;
    var label = data.label;
    self.removeAnimationByUID(uid);

    var options0 = (data.settings&&data.settings.options0)? data.settings.options0 : {
        // ParticleGroup
        texture: './assets/images/textures/spe/sprite-flame2.jpg',
        frames: {x: 8, y: 4},
        // frameCount: 8,
        loop: 2,
        depthTest: true,
        scale: self.size().height / 2.0,
        blending: THREE.NormalBlending,
        fog: true,
        // Emitter
        particleCount: 200,
        maxAge: {
            value: 3,
            spread: 0
        },
        position: {
            value: {x: 0, y: 0, z: 0},
            spread: {x: 10, y: 0, z: 0},
            spreadClamp: {x: 0, y: 0, z: 0},
            distribution: SPE.distributions.BOX,
            randomise: false
        },
        radius: {
            value: 5,
            spread: 0,
            scale: {x: 1, y: 1, z: 1},//new THREE.Vector3( 1, 1, 1 ),
            spreadClamp: {x: 2, y: 2, z: 2},//new THREE.Vector3( 2, 2, 2 ),
        },
        velocity: {
            value: {x: 0, y: 0, z: 0},
            spread: {x: 0, y: 0, z: 0},
            // distribution: SPE.distributions.BOX,
            randomise: false
        },
        wiggle: {
            spread: 0,
            value: 0
        },
        drag: {
            value: 0.5,
            spread: 0
        },
        rotation: {
            axis: {x: 0, y: 1, z: 0},//new THREE.Vector3( 0, 1, 0 ),
            axisSpread: {x: 0, y: 0, z: 0},//new THREE.Vector3( 0, 0, 0 ),
            angle:  0, // radians
            angleSpread: 0, // radians
            static: false,
            center: {x: 0, y: 0, z: 0},//new THREE.Vector3( 0, 0, 0 )
        },
        size: {
            value: 20,
            spread: 0
        },
        opacity: {
            value: {
                step1: 0,
                step2: 0,
                step3: 1,
                step4: 0
            }
        },
        color: {
            value: {
                step1: '#ffffff',
                step2: '#ffffff',
                step3: '#ffffff',
                step4: '#ffffff'
            },
            spread: ''
        },
        angle: {
            value: [ 0, Math.PI * 0.125 ]
        },
        acceleration: {
            value: new THREE.Vector3( 0, 0, 0 ),
            spread: new THREE.Vector3( 0, 0, 0 ),
            // distribution: SPE.distributions.BOX,
            randomise: false
        },
        speed: 1
    };

    var options1 = {
        texture: {
            value: new THREE.TextureLoader().load(options0.texture),
            frames: new THREE.Vector2( options0.frames.x, options0.frames.y ),
            loop: options0.loop,
        },
        blending: options0.blending,
        fog: options0.fog,
        depthTest: options0.depthTest,
        scale: options0.scale
    };

    var options2 = {
        particleCount: options0.particleCount,
        maxAge: options0.maxAge,
        position: {
            value: new THREE.Vector3( options0.position.value.x, options0.position.value.y, options0.position.value.z ),
            spread: new THREE.Vector3( options0.position.spread.x, options0.position.spread.y, options0.position.spread.z ),
            spreadClamp: new THREE.Vector3( options0.position.spreadClamp.x, options0.position.spreadClamp.y, options0.position.spreadClamp.z ),
            distribution: options0.distribution,
            randomise: options0.position.randomise
        },
        radius: {
            value: options0.radius.value,
            spread: options0.radius.spread,
            scale: new THREE.Vector3( options0.radius.scale.x, options0.radius.scale.y, options0.radius.scale.z ),
            spreadClamp: new THREE.Vector3( options0.radius.spreadClamp.x, options0.radius.spreadClamp.y, options0.radius.spreadClamp.z ),
        },
        velocity: {
            value: new THREE.Vector3( options0.velocity.value.x, options0.velocity.value.y, options0.velocity.value.z ),
            spread: new THREE.Vector3( options0.velocity.spread.x, options0.velocity.spread.y, options0.velocity.spread.z ),
            // distribution: SPE.distributions.BOX,
            randomise: options0.velocity.randomise
        },
        drag: options0.drag,
        wiggle: options0.wiggle,
        size: options0.size,
        opacity: {
            value: [ options0.opacity.value.step1, options0.opacity.value.step2, options0.opacity.value.step3, options0.opacity.value.step4 ]
        },
        color: {
            value: [
                new THREE.Color().setStyle(options0.color.value.step1),
                new THREE.Color().setStyle(options0.color.value.step2),
                new THREE.Color().setStyle(options0.color.value.step3),
                new THREE.Color().setStyle(options0.color.value.step4)
            ],
            spread: new THREE.Color().setStyle(options0.color.spread)
        },
        angle: options0.angle,
        acceleration: {
            value: new THREE.Vector3( options0.acceleration.value.x, options0.acceleration.value.y, options0.acceleration.value.z ),
            spread: new THREE.Vector3( options0.acceleration.spread.x, options0.acceleration.spread.y, options0.acceleration.spread.z ),
            // distribution: SPE.distributions.BOX,
            randomise: false
        },
        rotation: {
            axis: new THREE.Vector3( options0.rotation.axis.x, options0.rotation.axis.y, options0.rotation.axis.z ),
            axisSpread: new THREE.Vector3( options0.rotation.axisSpread.x, options0.rotation.axisSpread.y, options0.rotation.axisSpread.z ),
            angle:  0, // radians
            angleSpread: 0, // radians
            static: false,
            center: new THREE.Vector3( options0.rotation.center.x, options0.rotation.center.y, options0.rotation.center.z )
        }
    };


    if(!data.settings) {
        data.settings = {};

    }
    data.settings.options0 = options0;

    var particleGroup = new SPE.Group(options1);

    var emitter = new SPE.Emitter(options2);


    particleGroup.addEmitter( emitter );
    var particle = particleGroup.mesh;

    particle.refreshSettings = function(settings) {
        //if(!settings) return false;
        var mee = this;
        if(mee.gui) {
            mee.gui.destroy();
            mee.gui = null;
        }

        var options0 = settings&&settings.options? settings.options0 : mee.userData.settings.options0;

        var onFinishUpdate = function(result) {
            //particle.userData = options0;
            self.fireObjectChange(self.getParticleData(particle));
        };
        var updateMaterial = function() {
            particleGroup.material.needsUpdate = true;
        };
        var vec3Components = ['x', 'y', 'z'], i;

        var gui = new dat.GUI( { autoPlace: false } );

        gui.add( options0, 'blending' ).min(0).max(5).step(1).onChange( function( value ) {
            particleGroup.material.blending = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        gui.add( options0, 'particleCount' ).min(0).max(50000).step(1).onChange( function( value ) {
            emitter.particleCount = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        gui.add( options0, 'speed' ).min(0.01).max(10).step(0.01).onChange( function( value ) {

        }).onFinishChange(onFinishUpdate);

        var size = gui.addFolder('Size');
        size.add(options0.size, 'spread', 1, 300).step(1).onChange(function(value) {
            emitter.size.spread = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        size.add(options0.size, 'value', 1, 300).step(1).onChange(function(value) {
            emitter.size.value = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);

        var wiggle = gui.addFolder('Wiggle');
        wiggle.add(options0.wiggle, 'spread', 1, 100).step(10).onChange(function(value) {
            emitter.wiggle.spread = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        wiggle.add(options0.wiggle, 'value', 1, 100).step(10).onChange(function(value) {
            emitter.wiggle.value = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);

        var velocityFolder = gui.addFolder( 'Velocity' );
        var velocityValue = velocityFolder.addFolder( 'Value' );
        var velocitySpread = velocityFolder.addFolder( 'Spread' );
        for( i = 0; i < vec3Components.length; ++i ) {
            velocityValue.add( options0.velocity.value, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.velocity.value = new THREE.Vector3( options0.velocity.value.x, options0.velocity.value.y, options0.velocity.value.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
            velocitySpread.add( options0.velocity.spread, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.velocity.spread = new THREE.Vector3( options0.velocity.spread.x, options0.velocity.spread.y, options0.velocity.spread.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
        }

        var positionFolder = gui.addFolder( 'Position' );
        var positionValue = positionFolder.addFolder( 'Value' );
        var positionSpread = positionFolder.addFolder( 'Spread' );
        var positionSpreadClamp = positionFolder.addFolder( 'Spread Clamp');
        for( i = 0; i < vec3Components.length; ++i ) {
            positionValue.add( options0.position.value, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.position.value = new THREE.Vector3( options0.position.value.x, options0.position.value.y, options0.position.value.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
            positionSpread.add( options0.position.spread, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.position.spread = new THREE.Vector3( options0.position.spread.x, options0.position.spread.y, options0.position.spread.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
            positionSpreadClamp.add( options0.position.spreadClamp, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.position.spreadClamp = new THREE.Vector3( options0.position.spreadClamp.x, options0.position.spreadClamp.y, options0.position.spreadClamp.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
        }

        var colorFolder = gui.addFolder( 'Color steps' );
        colorFolder.addColor(options0.color.value, 'step1' ).onChange( function( value ) {
            emitter.color.value[ 0 ].setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        colorFolder.addColor(options0.color.value, 'step2').onChange( function( value ) {
            emitter.color.value[ 1 ] = new THREE.Color().setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        colorFolder.addColor(options0.color.value, 'step3').onChange( function( value ) {
            emitter.color.value[ 2 ] = new THREE.Color().setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        colorFolder.addColor(options0.color.value, 'step4').onChange( function( value ) {
            emitter.color.value[ 3 ] = new THREE.Color().setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);


        var opacityFolder = gui.addFolder( 'Opacity steps' );
        opacityFolder.add( options0.opacity.value, 'step1' ).min(0).max(1).step(0.01).listen().onChange( function( value ) {
            emitter.opacity.value[ 0 ] = value;
            emitter.opacity.value = emitter.opacity.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        opacityFolder.add( options0.opacity.value, 'step2' ).min(0).max(1).step(0.01).listen().onChange( function( value ) {
            emitter.opacity.value[ 1 ] = value;
            emitter.opacity.value = emitter.opacity.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        opacityFolder.add( options0.opacity.value, 'step3' ).min(0).max(1).step(0.01).listen().onChange( function( value ) {
            emitter.opacity.value[ 2 ] = value;
            emitter.opacity.value = emitter.opacity.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        opacityFolder.add( options0.opacity.value, 'step4' ).min(0).max(1).step(0.01).listen().onChange( function( value ) {
            emitter.opacity.value[ 3 ] = value;
            emitter.opacity.value = emitter.opacity.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);


        var accelerationFolder = gui.addFolder( 'Acceleration' );
        var accelerationValue = accelerationFolder.addFolder( 'Value' );
        var accelerationSpread = accelerationFolder.addFolder( 'Spread' );

        for( i = 0; i < vec3Components.length; ++i ) {
            accelerationValue.add( options0.acceleration.value, vec3Components[ i ], -50, 50 ).listen().onChange( function() {
                emitter.acceleration.value = new THREE.Vector3( options0.acceleration.value.x, options0.acceleration.value.y, options0.acceleration.value.z ),
                    updateMaterial();
            } ).onFinishChange(onFinishUpdate);
            accelerationSpread.add( options0.acceleration.spread, vec3Components[ i ], -50, 50 ).listen().onChange( function() {
                emitter.acceleration.spread = new THREE.Vector3( options0.acceleration.spread.x, options0.acceleration.spread.y, options0.acceleration.spread.z ),
                    updateMaterial();
            } ).onFinishChange(onFinishUpdate);
        }

        this.gui = gui;

        objById('gui_control').innerHTML = '';
        objById('gui_control').appendChild(this.gui.domElement);
        objById('gui_control').setAttribute('data-uid', this.userData.uid);
    };

    self.setParticleData(particle, data);
    self.glScene.add( particle );
    self.addObject(particle);

    particle.refreshSettings(data.settings);

    var ani = {
        uid: uid,
        label: label,
        state: 0,
        mesh: particle,
        engine: particleGroup,
        go: function(delta) {
            var speed = this.mesh.userData.settings.options0.speed;
            if(this.state === 0) {
                speed = 10;
                this.state = 10;
            }
            this.engine.tick( delta*speed );
        },
        remove: function() {
            this.mesh.gui.destroy();
        }
    };

    self.addAnimationList(ani);

    return callback(particle);
};

Workspace.prototype.addFlame = function(data, callback) {
    var self = this;

    var conf = getObjConf(data.type);
    var uid = data.uid;
    var label = data.label;
    self.removeAnimationByUID(uid);

    var options0 = (data.settings&&data.settings.options0)? data.settings.options0 : {
        // ParticleGroup
        texture: './assets/images/textures/spe/smokeparticle.png',
        // Emitter
        particleCount: 2000,
        maxAge: {
            value: 2,
        },
        position: {
            value: {x: 0, y: 0, z: 0},
            spread: {x: 0, y: 0, z: 0},
        },
        velocity: {
            value: {x: 0, y: 30, z: 0},
            spread: {x: 10, y: 15, z: 10},
        },
        size: {
            value: 1,
            spread: 0
        },
        color: {
            value: {
                step1: '#ffffff',
                step2: '#ff0000',
                step3: '#ffffff',
                step4: '#ffffff',
            }
        },
        acceleration: {
            value: new THREE.Vector3( 0, -10, 0 ),
            spread: new THREE.Vector3( 10, 0, 10 ),
        },
        activeMultiplier: 1,
        wiggle: {
            spread: 0,
            value: 0
        },
        speed: 1
    };

    var options1 = {
        texture: {
            value: new THREE.TextureLoader().load(options0.texture)
        },
        blending: options0.blending,
        fog: options0.fog
    };

    var options2 = {
        particleCount: options0.particleCount,
        maxAge: options0.maxAge,
        position: {
            value: new THREE.Vector3( options0.position.value.x, options0.position.value.y, options0.position.value.z ),
            spread: new THREE.Vector3( options0.position.spread.x, options0.position.spread.y, options0.position.spread.z ),
        },
        velocity: {
            value: new THREE.Vector3( options0.velocity.value.x, options0.velocity.value.y, options0.velocity.value.z ),
            spread: new THREE.Vector3( options0.velocity.spread.x, options0.velocity.spread.y, options0.velocity.spread.z ),
        },
        wiggle: options0.wiggle,
        size: options0.size,
        color: {
            value: [
                new THREE.Color().setStyle(options0.color.value.step1),
                new THREE.Color().setStyle(options0.color.value.step2),
                new THREE.Color().setStyle(options0.color.value.step3),
                new THREE.Color().setStyle(options0.color.value.step4)
            ]
        },
        acceleration: {
            value: new THREE.Vector3( options0.acceleration.value.x, options0.acceleration.value.y, options0.acceleration.value.z ),
            spread: new THREE.Vector3( options0.acceleration.spread.x, options0.acceleration.spread.y, options0.acceleration.spread.z ),
        },
        activeMultiplier: options0.activeMultiplier,
    };


    if(!data.settings) {
        data.settings = {};

    }
    data.settings.options0 = options0;

    var particleGroup = new SPE.Group(options1);

    var emitter = new SPE.Emitter(options2);


    particleGroup.addEmitter( emitter );
    var particle = particleGroup.mesh;

    particle.refreshSettings = function(settings) {
        //if(!settings) return false;
        var mee = this;
        if(mee.gui) {
            mee.gui.destroy();
            mee.gui = null;
        }

        var options0 = settings&&settings.options? settings.options0 : mee.userData.settings.options0;

        var onFinishUpdate = function(result) {
            //particle.userData = options0;
            self.fireObjectChange(self.getParticleData(particle));
        };
        var updateMaterial = function() {
            particleGroup.material.needsUpdate = true;
        };
        var vec3Components = ['x', 'y', 'z'], i;

        var gui = new dat.GUI( { autoPlace: false } );

        gui.add( options0, 'particleCount' ).min(0).max(50000).step(1).onChange( function( value ) {
            emitter.particleCount = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        gui.add( options0, 'speed' ).min(0.01).max(10).step(0.01).onChange( function( value ) {

        }).onFinishChange(onFinishUpdate);
        gui.add( options0, 'activeMultiplier' ).min(0.01).max(1).step(0.01).onChange( function( value ) {
            emitter.activeMultiplier = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);

        var size = gui.addFolder('Size');
        size.add(options0.size, 'spread', 0, 100).step(1).onChange(function(value) {
            emitter.size.spread = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        size.add(options0.size, 'value', 0, 100).step(1).onChange(function(value) {
            emitter.size.value = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);


        var velocityFolder = gui.addFolder( 'Velocity' );
        var velocityValue = velocityFolder.addFolder( 'Value' );
        var velocitySpread = velocityFolder.addFolder( 'Spread' );
        for( i = 0; i < vec3Components.length; ++i ) {
            velocityValue.add( options0.velocity.value, vec3Components[ i ], -100, 100 ).listen().onChange( function() {
                emitter.velocity.value = new THREE.Vector3( options0.velocity.value.x, options0.velocity.value.y, options0.velocity.value.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
            velocitySpread.add( options0.velocity.spread, vec3Components[ i ], -100, 100 ).listen().onChange( function() {
                emitter.velocity.spread = new THREE.Vector3( options0.velocity.spread.x, options0.velocity.spread.y, options0.velocity.spread.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
        }

        var positionFolder = gui.addFolder( 'Position' );
        var positionValue = positionFolder.addFolder( 'Value' );
        var positionSpread = positionFolder.addFolder( 'Spread' );
        for( i = 0; i < vec3Components.length; ++i ) {
            positionValue.add( options0.position.value, vec3Components[ i ], -100, 100 ).listen().onChange( function() {
                emitter.position.value = new THREE.Vector3( options0.position.value.x, options0.position.value.y, options0.position.value.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
            positionSpread.add( options0.position.spread, vec3Components[ i ], -100, 100 ).listen().onChange( function() {
                emitter.position.spread = new THREE.Vector3( options0.position.spread.x, options0.position.spread.y, options0.position.spread.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
        }

        var colorFolder = gui.addFolder( 'Color steps' );
        colorFolder.addColor(options0.color.value, 'step1' ).onChange( function( value ) {
            emitter.color.value[ 0 ].setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        colorFolder.addColor(options0.color.value, 'step2').onChange( function( value ) {
            emitter.color.value[ 1 ] = new THREE.Color().setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        colorFolder.addColor(options0.color.value, 'step3').onChange( function( value ) {
            emitter.color.value[ 2 ] = new THREE.Color().setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        colorFolder.addColor(options0.color.value, 'step4').onChange( function( value ) {
            emitter.color.value[ 3 ] = new THREE.Color().setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);


        var accelerationFolder = gui.addFolder( 'Acceleration' );
        var accelerationValue = accelerationFolder.addFolder( 'Value' );
        var accelerationSpread = accelerationFolder.addFolder( 'Spread' );

        for( i = 0; i < vec3Components.length; ++i ) {
            accelerationValue.add( options0.acceleration.value, vec3Components[ i ], -50, 50 ).listen().onChange( function() {
                emitter.acceleration.value = new THREE.Vector3( options0.acceleration.value.x, options0.acceleration.value.y, options0.acceleration.value.z ),
                    updateMaterial();
            } ).onFinishChange(onFinishUpdate);
            accelerationSpread.add( options0.acceleration.spread, vec3Components[ i ], -50, 50 ).listen().onChange( function() {
                emitter.acceleration.spread = new THREE.Vector3( options0.acceleration.spread.x, options0.acceleration.spread.y, options0.acceleration.spread.z ),
                    updateMaterial();
            } ).onFinishChange(onFinishUpdate);
        }

        var wiggle = gui.addFolder('Wiggle');
        wiggle.add(options0.wiggle, 'spread', 1, 100).step(10).onChange(function(value) {
            emitter.wiggle.spread = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        wiggle.add(options0.wiggle, 'value', 1, 100).step(10).onChange(function(value) {
            emitter.wiggle.value = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);

        this.gui = gui;

        objById('gui_control').innerHTML = '';
        objById('gui_control').appendChild(this.gui.domElement);
        objById('gui_control').setAttribute('data-uid', this.userData.uid);
    };

    self.setParticleData(particle, data);
    self.glScene.add( particle );
    self.addObject(particle);

    particle.refreshSettings(data.settings);

    var ani = {
        uid: uid,
        label: label,
        state: 0,
        mesh: particle,
        engine: particleGroup,
        go: function(delta) {
            var speed = this.mesh.userData.settings.options0.speed;
            if(this.state === 0) {
                speed = 10;
                this.state = 10;
            }
            this.engine.tick( delta*speed );
        },
        remove: function() {
            this.mesh.gui.destroy();
        }
    };

    self.addAnimationList(ani);

    return callback(particle);
};
Workspace.prototype.addSnow = function(data, callback) {
    var self = this;

    var conf = getObjConf(data.type);
    var uid = data.uid;
    var label = data.label;
    self.removeAnimationByUID(uid);

    var options0 = (data.settings&&data.settings.options0)? data.settings.options0 : {
        // ParticleGroup
        texture: './assets/images/textures/stemkoski/snowflake.png',
        blending: THREE.NormalBlending,
        fog: true,
        // Emitter
        particleCount: 5000,
        maxAge: {
            value: 3,
        },
        position: {
            value: {x: 0, y: 100, z: 0},
            spread: {x: 500, y: 30, z: 500},
            spreadClamp: {x: 0, y: 0, z: 0},
            //distribution: SPE.distributions.BOX,
            randomise: false
        },
        velocity: {
            value: {x: 0, y: 0, z: 30},
            spread: {x: 0, y: 0, z: 0},
            // distribution: SPE.distributions.BOX,
            randomise: false
        },
        wiggle: {
            spread: 10,
            value: 50
        },
        size: {
            value: 6,
            spread: 6
        },
        opacity: {
            value: {
                step1: 0,
                step2: 0,
                step3: 1,
                step4: 0
            }
        },
        color: {
            value: {
                step1: '#ffffff',
                step2: '#ffffff',
                step3: '#ffffff',
                step4: '#ffffff'
            },
            spread: ''
        },
        angle: {
            value: [ 0, Math.PI * 0.125 ]
        },
        acceleration: {
            value: new THREE.Vector3( 0, -50, 0 ),
            spread: new THREE.Vector3( 50, 50, 50 ),
            // distribution: SPE.distributions.BOX,
            randomise: false
        },
        speed: 0.2
    };

    var options1 = {
        texture: {
            value: new THREE.TextureLoader().load(options0.texture)
        },
        blending: options0.blending,
        fog: options0.fog
    };

    var options2 = {
        particleCount: options0.particleCount,
        maxAge: options0.maxAge,
        position: {
            value: new THREE.Vector3( options0.position.value.x, options0.position.value.y, options0.position.value.z ),
            spread: new THREE.Vector3( options0.position.spread.x, options0.position.spread.y, options0.position.spread.z ),
            spreadClamp: new THREE.Vector3( options0.position.spreadClamp.x, options0.position.spreadClamp.y, options0.position.spreadClamp.z ),
            //distribution: SPE.distributions.BOX,
            randomise: options0.position.randomise
        },
        velocity: {
            value: new THREE.Vector3( options0.velocity.value.x, options0.velocity.value.y, options0.velocity.value.z ),
            spread: new THREE.Vector3( options0.velocity.spread.x, options0.velocity.spread.y, options0.velocity.spread.z ),
            // distribution: SPE.distributions.BOX,
            randomise: options0.velocity.randomise
        },
        wiggle: options0.wiggle,
        size: options0.size,
        opacity: {
            value: [ options0.opacity.value.step1, options0.opacity.value.step2, options0.opacity.value.step3, options0.opacity.value.step4 ]
        },
        color: {
            value: [
                new THREE.Color().setStyle(options0.color.value.step1),
                new THREE.Color().setStyle(options0.color.value.step2),
                new THREE.Color().setStyle(options0.color.value.step3),
                new THREE.Color().setStyle(options0.color.value.step4)
            ],
            spread: new THREE.Color().setStyle(options0.color.spread)
        },
        angle: options0.angle,
        acceleration: {
            value: new THREE.Vector3( options0.acceleration.value.x, options0.acceleration.value.y, options0.acceleration.value.z ),
            spread: new THREE.Vector3( options0.acceleration.spread.x, options0.acceleration.spread.y, options0.acceleration.spread.z ),
            // distribution: SPE.distributions.BOX,
            randomise: false
        },
    };


    if(!data.settings) {
        data.settings = {};

    }
    data.settings.options0 = options0;

    var particleGroup = new SPE.Group(options1);

    var emitter = new SPE.Emitter(options2);

    particleGroup.addEmitter( emitter );
    var particle = particleGroup.mesh;

    particle.refreshSettings = function(settings) {
        //if(!settings) return false;
        var mee = this;
        if(mee.gui) {
            mee.gui.destroy();
            mee.gui = null;
        }

        var options0 = settings&&settings.options? settings.options0 : mee.userData.settings.options0;

        var onFinishUpdate = function(result) {
            //particle.userData = options0;
            self.fireObjectChange(self.getParticleData(particle));
        };
        var updateMaterial = function() {
            particleGroup.material.needsUpdate = true;
        };
        var vec3Components = ['x', 'y', 'z'], i;

        var gui = new dat.GUI( { autoPlace: false } );

        gui.add( options0, 'blending' ).min(0).max(5).step(1).onChange( function( value ) {
            particleGroup.material.blending = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        gui.add( options0, 'particleCount' ).min(0).max(50000).step(1).onChange( function( value ) {
            emitter.particleCount = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        gui.add( options0, 'speed' ).min(0.01).max(10).step(0.01).onChange( function( value ) {

        }).onFinishChange(onFinishUpdate);

        var size = gui.addFolder('Size');
        size.add(options0.size, 'spread', 1, 300).step(1).onChange(function(value) {
            emitter.size.spread = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        size.add(options0.size, 'value', 1, 300).step(1).onChange(function(value) {
            emitter.size.value = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);

        var wiggle = gui.addFolder('Wiggle');
        wiggle.add(options0.wiggle, 'spread', 1, 100).step(10).onChange(function(value) {
            emitter.wiggle.spread = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        wiggle.add(options0.wiggle, 'value', 1, 100).step(10).onChange(function(value) {
            emitter.wiggle.value = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);

        var velocityFolder = gui.addFolder( 'Velocity' );
        var velocityValue = velocityFolder.addFolder( 'Value' );
        var velocitySpread = velocityFolder.addFolder( 'Spread' );
        for( i = 0; i < vec3Components.length; ++i ) {
            velocityValue.add( options0.velocity.value, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.velocity.value = new THREE.Vector3( options0.velocity.value.x, options0.velocity.value.y, options0.velocity.value.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
            velocitySpread.add( options0.velocity.spread, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.velocity.spread = new THREE.Vector3( options0.velocity.spread.x, options0.velocity.spread.y, options0.velocity.spread.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
        }

        var positionFolder = gui.addFolder( 'Position' );
        var positionValue = positionFolder.addFolder( 'Value' );
        var positionSpread = positionFolder.addFolder( 'Spread' );
        var positionSpreadClamp = positionFolder.addFolder( 'Spread Clamp');
        for( i = 0; i < vec3Components.length; ++i ) {
            positionValue.add( options0.position.value, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.position.value = new THREE.Vector3( options0.position.value.x, options0.position.value.y, options0.position.value.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
            positionSpread.add( options0.position.spread, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.position.spread = new THREE.Vector3( options0.position.spread.x, options0.position.spread.y, options0.position.spread.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
            positionSpreadClamp.add( options0.position.spreadClamp, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.position.spreadClamp = new THREE.Vector3( options0.position.spreadClamp.x, options0.position.spreadClamp.y, options0.position.spreadClamp.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
        }

        var colorFolder = gui.addFolder( 'Color steps' );
        colorFolder.addColor(options0.color.value, 'step1' ).onChange( function( value ) {
            emitter.color.value[ 0 ].setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        colorFolder.addColor(options0.color.value, 'step2').onChange( function( value ) {
            emitter.color.value[ 1 ] = new THREE.Color().setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        colorFolder.addColor(options0.color.value, 'step3').onChange( function( value ) {
            emitter.color.value[ 2 ] = new THREE.Color().setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        colorFolder.addColor(options0.color.value, 'step4').onChange( function( value ) {
            emitter.color.value[ 3 ] = new THREE.Color().setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);


        var opacityFolder = gui.addFolder( 'Opacity steps' );
        opacityFolder.add( options0.opacity.value, 'step1' ).min(0).max(1).step(0.01).listen().onChange( function( value ) {
            emitter.opacity.value[ 0 ] = value;
            emitter.opacity.value = emitter.opacity.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        opacityFolder.add( options0.opacity.value, 'step2' ).min(0).max(1).step(0.01).listen().onChange( function( value ) {
            emitter.opacity.value[ 1 ] = value;
            emitter.opacity.value = emitter.opacity.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        opacityFolder.add( options0.opacity.value, 'step3' ).min(0).max(1).step(0.01).listen().onChange( function( value ) {
            emitter.opacity.value[ 2 ] = value;
            emitter.opacity.value = emitter.opacity.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        opacityFolder.add( options0.opacity.value, 'step4' ).min(0).max(1).step(0.01).listen().onChange( function( value ) {
            emitter.opacity.value[ 3 ] = value;
            emitter.opacity.value = emitter.opacity.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);


        var accelerationFolder = gui.addFolder( 'Acceleration' );
        var accelerationValue = accelerationFolder.addFolder( 'Value' );
        var accelerationSpread = accelerationFolder.addFolder( 'Spread' );

        for( i = 0; i < vec3Components.length; ++i ) {
            accelerationValue.add( options0.acceleration.value, vec3Components[ i ], -50, 50 ).listen().onChange( function() {
                emitter.acceleration.value = new THREE.Vector3( options0.acceleration.value.x, options0.acceleration.value.y, options0.acceleration.value.z ),
                    updateMaterial();
            } ).onFinishChange(onFinishUpdate);
            accelerationSpread.add( options0.acceleration.spread, vec3Components[ i ], -50, 50 ).listen().onChange( function() {
                emitter.acceleration.spread = new THREE.Vector3( options0.acceleration.spread.x, options0.acceleration.spread.y, options0.acceleration.spread.z ),
                    updateMaterial();
            } ).onFinishChange(onFinishUpdate);
        }

        this.gui = gui;

        objById('gui_control').innerHTML = '';
        objById('gui_control').appendChild(this.gui.domElement);
        objById('gui_control').setAttribute('data-uid', this.userData.uid);
    };

    self.setParticleData(particle, data);
    self.glScene.add( particle );
    self.addObject(particle);

    particle.refreshSettings(data.settings);

    var ani = {
        uid: uid,
        label: label,
        state: 0,
        mesh: particle,
        engine: particleGroup,
        go: function(delta) {
            var speed = this.mesh.userData.settings.options0.speed;
            if(this.state === 0) {
                speed = 10;
                this.state = 10;
            }
            this.engine.tick( delta*speed );
        },
        remove: function() {
            this.mesh.gui.destroy();
        }
    };

    self.addAnimationList(ani);

    return callback(particle);
};
Workspace.prototype.addRain = function(data, callback) {
    var self = this;

    var conf = getObjConf(data.type);
    var uid = data.uid;
    var label = data.label;
    self.removeAnimationByUID(uid);

    var options0 = (data.settings&&data.settings.options0)? data.settings.options0 : {
        // ParticleGroup
        texture: './assets/images/textures/stemkoski/raindrop2flip.png',
        blending: THREE.NormalBlending,
        fog: true,
        // Emitter
        particleCount: 20000,
        maxAge: {
            value: 3,
        },
        position: {
            value: {x: 0, y: 100, z: 0},
            spread: {x: 500, y: 30, z: 500},
            spreadClamp: {x: 0, y: 0, z: 0},
            //distribution: SPE.distributions.BOX,
            randomise: false
        },
        velocity: {
            value: {x: 0, y: 0, z: 30},
            spread: {x: 0, y: 0, z: 0},
            // distribution: SPE.distributions.BOX,
            randomise: false
        },
        wiggle: {
            spread: 10,
            value: 10
        },
        size: {
            value: 2,
            spread: 1
        },
        opacity: {
            value: {
                step1: 0.2,
                step2: 0.2,
                step3: 0.2,
                step4: 0.2
            }
        },
        color: {
            value: {
                step1: '#ffffff',
                step2: '#ffffff',
                step3: '#ffffff',
                step4: '#ffffff'
            },
            spread: ''
        },
        angle: {
            value: [ 0, Math.PI * 0.125 ]
        },
        acceleration: {
            value: new THREE.Vector3( 0, -50, 0 ),
            spread: new THREE.Vector3( 0, 0, 0 ),
            // distribution: SPE.distributions.BOX,
            randomise: false
        },
        speed: 2.5
    };

    var options1 = {
        texture: {
            value: new THREE.TextureLoader().load(options0.texture)
        },
        blending: options0.blending,
        fog: options0.fog
    };

    var options2 = {
        particleCount: options0.particleCount,
        maxAge: options0.maxAge,
        position: {
            value: new THREE.Vector3( options0.position.value.x, options0.position.value.y, options0.position.value.z ),
            spread: new THREE.Vector3( options0.position.spread.x, options0.position.spread.y, options0.position.spread.z ),
            spreadClamp: new THREE.Vector3( options0.position.spreadClamp.x, options0.position.spreadClamp.y, options0.position.spreadClamp.z ),
            //distribution: SPE.distributions.BOX,
            randomise: options0.position.randomise
        },
        velocity: {
            value: new THREE.Vector3( options0.velocity.value.x, options0.velocity.value.y, options0.velocity.value.z ),
            spread: new THREE.Vector3( options0.velocity.spread.x, options0.velocity.spread.y, options0.velocity.spread.z ),
            // distribution: SPE.distributions.BOX,
            randomise: options0.velocity.randomise
        },
        wiggle: options0.wiggle,
        size: options0.size,
        opacity: {
            value: [ options0.opacity.value.step1, options0.opacity.value.step2, options0.opacity.value.step3, options0.opacity.value.step4 ]
        },
        color: {
            value: [
                new THREE.Color().setStyle(options0.color.value.step1),
                new THREE.Color().setStyle(options0.color.value.step2),
                new THREE.Color().setStyle(options0.color.value.step3),
                new THREE.Color().setStyle(options0.color.value.step4)
            ],
            spread: new THREE.Color().setStyle(options0.color.spread)
        },
        angle: options0.angle,
        acceleration: {
            value: new THREE.Vector3( options0.acceleration.value.x, options0.acceleration.value.y, options0.acceleration.value.z ),
            spread: new THREE.Vector3( options0.acceleration.spread.x, options0.acceleration.spread.y, options0.acceleration.spread.z ),
            // distribution: SPE.distributions.BOX,
            randomise: false
        },
    };


    if(!data.settings) {
        data.settings = {};

    }
    data.settings.options0 = options0;

    var particleGroup = new SPE.Group(options1);

    var emitter = new SPE.Emitter(options2);


    particleGroup.addEmitter( emitter );
    var particle = particleGroup.mesh;

    particle.refreshSettings = function(settings) {
        //if(!settings) return false;
        var mee = this;
        if(mee.gui) {
            mee.gui.destroy();
            mee.gui = null;
        }

        var options0 = settings&&settings.options? settings.options0 : mee.userData.settings.options0;

        var onFinishUpdate = function(result) {
            //particle.userData = options0;
            self.fireObjectChange(self.getParticleData(particle));
        };
        var updateMaterial = function() {
            particleGroup.material.needsUpdate = true;
        };
        var vec3Components = ['x', 'y', 'z'], i;

        var gui = new dat.GUI( { autoPlace: false } );

        gui.add( options0, 'blending' ).min(0).max(5).step(1).onChange( function( value ) {
            particleGroup.material.blending = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        gui.add( options0, 'particleCount' ).min(0).max(50000).step(1).onChange( function( value ) {
            emitter.particleCount = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        gui.add( options0, 'speed' ).min(0.01).max(10).step(0.01).onChange( function( value ) {

        }).onFinishChange(onFinishUpdate);

        var size = gui.addFolder('Size');
        size.add(options0.size, 'spread', 1, 300).step(1).onChange(function(value) {
            emitter.size.spread = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        size.add(options0.size, 'value', 1, 300).step(1).onChange(function(value) {
            emitter.size.value = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);

        var wiggle = gui.addFolder('Wiggle');
        wiggle.add(options0.wiggle, 'spread', 1, 100).step(10).onChange(function(value) {
            emitter.wiggle.spread = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);
        wiggle.add(options0.wiggle, 'value', 1, 100).step(10).onChange(function(value) {
            emitter.wiggle.value = value;
            updateMaterial();
        }).onFinishChange(onFinishUpdate);

        var velocityFolder = gui.addFolder( 'Velocity' );
        var velocityValue = velocityFolder.addFolder( 'Value' );
        var velocitySpread = velocityFolder.addFolder( 'Spread' );
        for( i = 0; i < vec3Components.length; ++i ) {
            velocityValue.add( options0.velocity.value, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.velocity.value = new THREE.Vector3( options0.velocity.value.x, options0.velocity.value.y, options0.velocity.value.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
            velocitySpread.add( options0.velocity.spread, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.velocity.spread = new THREE.Vector3( options0.velocity.spread.x, options0.velocity.spread.y, options0.velocity.spread.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
        }

        var positionFolder = gui.addFolder( 'Position' );
        var positionValue = positionFolder.addFolder( 'Value' );
        var positionSpread = positionFolder.addFolder( 'Spread' );
        var positionSpreadClamp = positionFolder.addFolder( 'Spread Clamp');
        for( i = 0; i < vec3Components.length; ++i ) {
            positionValue.add( options0.position.value, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.position.value = new THREE.Vector3( options0.position.value.x, options0.position.value.y, options0.position.value.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
            positionSpread.add( options0.position.spread, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.position.spread = new THREE.Vector3( options0.position.spread.x, options0.position.spread.y, options0.position.spread.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
            positionSpreadClamp.add( options0.position.spreadClamp, vec3Components[ i ], -500, 500 ).listen().onChange( function() {
                emitter.position.spreadClamp = new THREE.Vector3( options0.position.spreadClamp.x, options0.position.spreadClamp.y, options0.position.spreadClamp.z );
                updateMaterial();
            }).onFinishChange(onFinishUpdate);
        }

        var colorFolder = gui.addFolder( 'Color steps' );
        colorFolder.addColor(options0.color.value, 'step1' ).onChange( function( value ) {
            emitter.color.value[ 0 ].setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        colorFolder.addColor(options0.color.value, 'step2').onChange( function( value ) {
            emitter.color.value[ 1 ] = new THREE.Color().setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        colorFolder.addColor(options0.color.value, 'step3').onChange( function( value ) {
            emitter.color.value[ 2 ] = new THREE.Color().setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        colorFolder.addColor(options0.color.value, 'step4').onChange( function( value ) {
            emitter.color.value[ 3 ] = new THREE.Color().setStyle( value );
            emitter.color.value = emitter.color.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);


        var opacityFolder = gui.addFolder( 'Opacity steps' );
        opacityFolder.add( options0.opacity.value, 'step1' ).min(0).max(1).step(0.01).listen().onChange( function( value ) {
            emitter.opacity.value[ 0 ] = value;
            emitter.opacity.value = emitter.opacity.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        opacityFolder.add( options0.opacity.value, 'step2' ).min(0).max(1).step(0.01).listen().onChange( function( value ) {
            emitter.opacity.value[ 1 ] = value;
            emitter.opacity.value = emitter.opacity.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        opacityFolder.add( options0.opacity.value, 'step3' ).min(0).max(1).step(0.01).listen().onChange( function( value ) {
            emitter.opacity.value[ 2 ] = value;
            emitter.opacity.value = emitter.opacity.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);
        opacityFolder.add( options0.opacity.value, 'step4' ).min(0).max(1).step(0.01).listen().onChange( function( value ) {
            emitter.opacity.value[ 3 ] = value;
            emitter.opacity.value = emitter.opacity.value;
            updateMaterial();
        } ).onFinishChange(onFinishUpdate);


        var accelerationFolder = gui.addFolder( 'Acceleration' );
        var accelerationValue = accelerationFolder.addFolder( 'Value' );
        var accelerationSpread = accelerationFolder.addFolder( 'Spread' );

        for( i = 0; i < vec3Components.length; ++i ) {
            accelerationValue.add( options0.acceleration.value, vec3Components[ i ], -50, 50 ).listen().onChange( function() {
                emitter.acceleration.value = new THREE.Vector3( options0.acceleration.value.x, options0.acceleration.value.y, options0.acceleration.value.z ),
                    updateMaterial();
            } ).onFinishChange(onFinishUpdate);
            accelerationSpread.add( options0.acceleration.spread, vec3Components[ i ], -50, 50 ).listen().onChange( function() {
                emitter.acceleration.spread = new THREE.Vector3( options0.acceleration.spread.x, options0.acceleration.spread.y, options0.acceleration.spread.z ),
                    updateMaterial();
            } ).onFinishChange(onFinishUpdate);
        }

        this.gui = gui;

        objById('gui_control').innerHTML = '';
        objById('gui_control').appendChild(this.gui.domElement);
        objById('gui_control').setAttribute('data-uid', this.userData.uid);
    };

    self.setParticleData(particle, data);
    self.glScene.add( particle );
    self.addObject(particle);

    particle.refreshSettings(data.settings);

    var ani = {
        uid: uid,
        label: label,
        state: 0,
        mesh: particle,
        engine: particleGroup,
        go: function(delta) {
            var speed = this.mesh.userData.settings.options0.speed;
            if(this.state === 0) {
                speed = 10;
                this.state = 10;
            }
            this.engine.tick( delta*speed );
        },
        remove: function() {
            this.mesh.gui.destroy();
        }
    };

    self.addAnimationList(ani);

    return callback(particle);
};
